@@ -17,7 +17,6 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 package net.sf.freecol;
 
 import java.awt.Dimension;
@@ -30,6 +29,7 @@ import java.io.BufferedReader;
 import java.io.File;
 import java.io.FileReader;
 import java.io.IOException;
+import java.lang.Thread.UncaughtExceptionHandler;
 import java.util.Locale;
 import java.util.jar.Attributes;
 import java.util.jar.Manifest;
@@ -45,6 +45,7 @@ import javax.swing.UIManager;
 import javax.swing.UnsupportedLookAndFeelException;
 import javax.xml.stream.XMLInputFactory;
 import javax.xml.stream.XMLStreamReader;
+import javax.xml.stream.events.XMLEvent;
 
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
@@ -55,24 +56,31 @@ import net.sf.freecol.client.gui.sound.MusicLibrary;
 import net.sf.freecol.client.gui.sound.SfxLibrary;
 import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.Specification;
+import net.sf.freecol.common.io.FreeColDataFile;
+import net.sf.freecol.common.io.FreeColSavegameFile;
+import net.sf.freecol.common.io.FreeColTcFile;
 import net.sf.freecol.common.logging.DefaultHandler;
 import net.sf.freecol.common.networking.NoRouteToServerException;
 import net.sf.freecol.common.option.LanguageOption;
+import net.sf.freecol.common.resources.ResourceManager;
 import net.sf.freecol.server.FreeColServer;
 
 
 /**
-* This class is responsible for handling the command-line arguments
-* and starting either the stand-alone server or the client-GUI.
-*
-* @see net.sf.freecol.client.FreeColClient FreeColClient
-* @see net.sf.freecol.server.FreeColServer FreeColServer
-*/
+ * This class is responsible for handling the command-line arguments
+ * and starting either the stand-alone server or the client-GUI.
+ *
+ * @see net.sf.freecol.client.FreeColClient FreeColClient
+ * @see net.sf.freecol.server.FreeColServer FreeColServer
+ */
 public final class FreeCol {
-    
 
     public static final String  META_SERVER_ADDRESS = "meta.freecol.org";
     public static final int     META_SERVER_PORT = 3540;
+    
+    public static final String CLIENT_THREAD = "FreeColClient:";
+    public static final String SERVER_THREAD = "FreeColServer:";
+    public static final String METASERVER_THREAD = "FreeColMetaServer:";
 
     /**
      * The space not being used in windowed mode.
@@ -80,19 +88,15 @@ public final class FreeCol {
     private static final int     DEFAULT_WINDOW_SPACE = 100;
 
     private static final Logger logger = Logger.getLogger(FreeCol.class.getName());
-    
-    /**
-     * Initialized on first access. @See #getSpecification
-     */
-    private static Specification specification;
 
-    private static final String FREECOL_VERSION = "0.7.4";
+    private static final String FREECOL_VERSION = "0.8.3";
     private static String FREECOL_REVISION;
     
     private static final String MIN_JDK_VERSION = "1.5";
     private static final String FILE_SEP = System.getProperty("file.separator");
 
     private static final String DEFAULT_SPLASH_FILE = "splash.jpg";
+    private static final String DEFAULT_TC = "freecol";
 
     private static boolean  windowed = false,
                             sound = true,
@@ -100,24 +104,34 @@ public final class FreeCol {
                             memoryCheck = true,
                             consoleLogging = false;
     private static Dimension windowSize = new Dimension(-1, -1);
-    private static String   dataFolder = "";
+    private static String   dataFolder = "data" + FILE_SEP;
     
     private static FreeColClient freeColClient;
 
     private static boolean standAloneServer = false;
+    private static boolean publicServer = true;
     private static boolean inDebugMode = false;
 
     private static int serverPort;
     private static String serverName = null;
-    
+    private static final int DEFAULT_PORT = 3541;
+
+    private static File mainUserDirectory;
+
     private static File saveDirectory;
     
+    private static File tcUserDirectory;
+    
+    private static String tc = DEFAULT_TC;
+    
     private static File savegameFile = null;
     
     private static File clientOptionsFile = null;
 
     private static Level logLevel = Level.INFO;
 
+    private static boolean checkIntegrity = false;
+
     private FreeCol() {
         // Hide constructor
     }
@@ -139,7 +153,8 @@ public final class FreeCol {
             FREECOL_REVISION = FREECOL_VERSION;
         }
 
-        initLogging();
+        // parse command line arguments
+        handleArgs(args);
         
         // Display splash screen:
         JWindow splash = null;
@@ -155,15 +170,17 @@ public final class FreeCol {
         }
 
         createAndSetDirectories();
+        initLogging();
+
         Locale.setDefault(getLocale());
-        handleArgs(args);
+        
         
         if (javaCheck && !checkJavaVersion()) {
             removeSplash(splash);
             System.err.println("Java version " + MIN_JDK_VERSION +
-                            " or better is recommended in order to run FreeCol." +
-                            " Use --no-java-check to skip this check.");
-            return;
+                               " or better is recommended in order to run FreeCol." +
+                               " Use --no-java-check to skip this check.");
+            System.exit(1);
         }
 
         int  minMemory = 128;  // million bytes
@@ -171,7 +188,12 @@ public final class FreeCol {
             removeSplash(splash);
             System.out.println("You need to assign more memory to the JVM. Restart FreeCol with:");
             System.out.println("java -Xmx" + minMemory + "M -jar FreeCol.jar");
-            return;
+            System.exit(1);
+        }
+        
+        if (!initializeResourceFolders()) {
+            removeSplash(splash);
+            System.exit(1);
         }
 
         if (standAloneServer) {
@@ -179,10 +201,12 @@ public final class FreeCol {
             try {
                 final FreeColServer freeColServer;
                 if (savegameFile != null) {
+                    FreeColSavegameFile fis = null;
                     try {
                         // Get suggestions for "singleplayer" and "public game" settings from the file:
-                        XMLStreamReader in = FreeColServer.createXMLStreamReader(savegameFile);
-                        in.nextTag();                    
+                        fis = new FreeColSavegameFile(savegameFile);
+                        XMLStreamReader in = FreeColServer.createXMLStreamReader(fis);
+                        in.nextTag();
                         final boolean defaultSingleplayer = Boolean.valueOf(in.getAttributeValue(null, "singleplayer")).booleanValue();
                         final boolean defaultPublicServer;
                         final String publicServerStr =  in.getAttributeValue(null, "publicServer");
@@ -197,11 +221,16 @@ public final class FreeCol {
                     } catch (Exception e) {
                         removeSplash(splash);
                         System.out.println("Could not load savegame.");
+                        System.exit(1);
                         return;
+                    } finally {
+                        if (fis!=null) {
+                            fis.close();
+                        }
                     }
                 } else {
                     try {
-                        freeColServer = new FreeColServer(true, false, serverPort, serverName);
+                        freeColServer = new FreeColServer(publicServer, false, serverPort, serverName);
                     } catch (NoRouteToServerException e) {
                         removeSplash(splash);
                         System.out.println(Messages.message("server.noRouteToServer"));
@@ -212,14 +241,18 @@ public final class FreeCol {
 
                 Runtime runtime = Runtime.getRuntime();
                 runtime.addShutdownHook(new Thread() {
-                    public void run() {
-                        freeColServer.getController().shutdown();
-                    }
-                });
+                        public void run() {
+                            freeColServer.getController().shutdown();
+                        }
+                    });
+                if (checkIntegrity) {
+                    System.exit((freeColServer.getIntegrity())
+                                ? 0 : 1);
+                }
             } catch (IOException e) {
                 removeSplash(splash);
                 System.err.println("Error while loading server: " + e);
-                System.exit(-1);
+                System.exit(1);
             }
         } else {
             final Rectangle bounds = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
@@ -234,7 +267,7 @@ public final class FreeCol {
             } else {
                 preloadSize = new Dimension(bounds.width, bounds.height);
             }
-
+            ResourceManager.preload(preloadSize);
             try {
                 UIManager.setLookAndFeel(new FreeColLookAndFeel(dataFolder, preloadSize));
             } catch (UnsupportedLookAndFeelException e) {
@@ -247,10 +280,12 @@ public final class FreeCol {
                 System.out.println("then run the game with a command-line parameter:");
                 System.out.println("");
                 printUsage();
-                return;
+                System.exit(1);
             }
 
-            // TODO: don't use same datafolder for both images and music because the images are best kept inside the .JAR file.
+            // TODO: don't use same datafolder for both images and
+            // music because the images are best kept inside the .JAR
+            // file.
 
             logger.info("Now starting to load images.");
 
@@ -265,6 +300,7 @@ public final class FreeCol {
                 System.out.println("then run the game with a command-line parameter:");
                 System.out.println("");
                 printUsage();
+                System.exit(1);
                 return;
             }
 
@@ -290,10 +326,10 @@ public final class FreeCol {
                 final FreeColClient theFreeColClient = freeColClient;
                 final File theSavegameFile = savegameFile;
                 SwingUtilities.invokeLater(new Runnable() {
-                    public void run() {
-                        theFreeColClient.getConnectController().loadGame(theSavegameFile);
-                    }
-                });
+                        public void run() {
+                            theFreeColClient.getConnectController().loadGame(theSavegameFile);
+                        }
+                    });
             }
         }
         
@@ -341,7 +377,7 @@ public final class FreeCol {
             baseLogger.removeHandler(handlers[i]);
         }
         try {
-            baseLogger.addHandler(new DefaultHandler(consoleLogging));
+            baseLogger.addHandler(new DefaultHandler(consoleLogging, mainUserDirectory));
             if (inDebugMode) {
                 logLevel = Level.FINEST;
             } 
@@ -350,6 +386,11 @@ public final class FreeCol {
         } catch (FreeColException e) {
             e.printStackTrace();
         }
+        Thread.setDefaultUncaughtExceptionHandler(new UncaughtExceptionHandler() {
+                public void uncaughtException(Thread thread, Throwable e) {
+                    baseLogger.log(Level.WARNING, "Uncaught exception from thread: " + thread, e);
+                }
+            });
     }
     
     /**
@@ -360,16 +401,30 @@ public final class FreeCol {
      */
     public static Locale getLocale() { 
         XMLInputFactory xif = XMLInputFactory.newInstance();
-
         XMLStreamReader in = null;
         try {
             in = xif.createXMLStreamReader(new BufferedReader(new FileReader(getClientOptionsFile())));
             in.nextTag();
-            while (!ClientOptions.LANGUAGE.equals(in.getLocalName())) {
+            /** 
+             * The following code was contributed by armcode to fix
+             * bug #[ 2045521 ] "Exception in Freecol.log on starting
+             * game". I was never able to reproduce the bug, but the
+             * patch did no harm either.
+             */
+            for(int eventid = in.getEventType();eventid != XMLEvent.END_DOCUMENT; eventid = in.getEventType()) { 
+
+                //TODO: Is checking for XMLEvent.ATTRIBUTE needed?
+                if(eventid == XMLEvent.START_ELEMENT) {
+                    if (ClientOptions.LANGUAGE.equals(in.getAttributeValue(null, "id"))) {
+                        return LanguageOption.getLocale(in.getAttributeValue(null, "value")); 
+                    }
+                }
                 in.nextTag();
             }
-            Locale l = LanguageOption.getLocale(in.getAttributeValue(null, "value"));
-            return l;
+            //We don't have a language option in our file, it is either not there or the file is corrupt
+            logger.log(Level.WARNING, "Language setting not found in client options file.  Using default.");
+            return Locale.getDefault();
+            
         } catch (Exception e) {
             logger.log(Level.WARNING, "Exception while loading options.", e);
             return Locale.getDefault();
@@ -385,6 +440,14 @@ public final class FreeCol {
         }
     }
 
+    /**
+     * Returns the default server network port.
+     * @return The port number.
+     */
+    public static int getDefaultPort() {
+        return DEFAULT_PORT;
+    }
+    
     /**
      * Returns the file containing the client options.
      * @return The file.
@@ -399,10 +462,7 @@ public final class FreeCol {
      * @return the specification to be used by all other classes.
      */
     public static Specification getSpecification() {
-        if (specification == null){
-            specification = new Specification();
-        }
-        return specification;
+        return Specification.getSpecification();
     }
 
     /**
@@ -422,38 +482,86 @@ public final class FreeCol {
      * and underneath that directory a "save" directory will
      * be created.
      */
-    private static void createAndSetDirectories() {
+    private static File createAndSetDirectories() {
         // TODO: The location of the save directory should be determined by the installer.;
         
-        File mainUserDirectory = new File(System.getProperty("user.home"), "freecol");
-        if (mainUserDirectory.exists() && mainUserDirectory.isFile()) {
-            logger.warning("Could not create .freecol under "
-                    + System.getProperty("user.home") + " because there "
-                    + "already exists a regular file with the same name.");
-            return;
-        } else if (!mainUserDirectory.exists()) {
+        String freeColDirectoryName = "/".equals(System.getProperty("file.separator")) ?
+            ".freecol" : "freecol";
+
+        if (mainUserDirectory == null) {
+            mainUserDirectory = new File(System.getProperty("user.home"), freeColDirectoryName);
+        }
+        if (mainUserDirectory.exists()) {
+            if (mainUserDirectory.isFile()) {
+                System.out.println("Could not create " + freeColDirectoryName + " under "
+                                   + System.getProperty("user.home") + " because there "
+                                   + "already exists a regular file with the same name.");
+                return null;
+            }
+        } else {
             mainUserDirectory.mkdir();
         }
-        clientOptionsFile = new File(mainUserDirectory, "options.xml");
-        saveDirectory = new File(mainUserDirectory, "save");
-        if (saveDirectory.exists() && saveDirectory.isFile()) {
-            logger.warning("Could not create freecol/save under "
-                    + System.getProperty("user.home") + " because there "
-                    + "already exists a regular file with the same name.");
-            return;
-        } else if (!saveDirectory.exists()) {
+        if (saveDirectory == null) {
+            saveDirectory = new File(mainUserDirectory, "save");
+        }
+        if (saveDirectory.exists()) {
+            if (saveDirectory.isFile()) {
+                System.out.println("Could not create freecol/save under "
+                                   + System.getProperty("user.home") + " because there "
+                                   + "already exists a regular file with the same name.");
+                return null;
+            }
+        } else {
             saveDirectory.mkdir();
         }
+        tcUserDirectory = new File(mainUserDirectory, tc);
+        if (tcUserDirectory.exists()) {
+            if (tcUserDirectory.isFile()) {
+                System.out.println("Could not create freecol/" + tc + " under "
+                                   + System.getProperty("user.home") + " because there "
+                                   + "already exists a regular file with the same name.");
+                return null;
+            }
+        } else {
+            tcUserDirectory.mkdir();
+        }
+        clientOptionsFile = new File(tcUserDirectory, "options.xml");
+        return mainUserDirectory;
+    }
+
+    /**
+     * Set up the save file and directory
+     * @param name the name of the save file to use
+     */
+    private static void setSavegame(String name) {
+        savegameFile = new File(name);
+        if (!savegameFile.exists() || !savegameFile.isFile()) {
+            savegameFile = new File(getSaveDirectory(), name);
+            if (!savegameFile.exists() || !savegameFile.isFile()) {
+                System.out.println("Could not find savegame file: " + name);
+                System.exit(1);
+            }
+        } else {
+            setSaveDirectory(savegameFile.getParentFile());
+        }
     }
 
     /**
      * Returns the directory where the savegames should be put.
      * @return The directory where the savegames should be put.
-    */
+     */
     public static File getSaveDirectory() {
         return saveDirectory;
     }
-    
+
+    /**
+     * Set the directory where the savegames should be put.
+     * @param saveDirectory a <code>File</code> value for the savegame directory
+     */
+    public static void setSaveDirectory(File saveDirectory) {
+        FreeCol.saveDirectory = saveDirectory;
+    }
+
     /**
      * Returns the data directory.
      * @return The directory where the data files are located.
@@ -466,6 +574,14 @@ public final class FreeCol {
         }
     }
     
+    /**
+     * Returns the mods directory.
+     * @return The directory where the mods are located.
+     */
+    public static File getModsDirectory() {
+        return new File(getDataDirectory(), "mods");
+    }
+    
     /**
      * Returns the directory where the autogenerated savegames 
      * should be put.
@@ -475,13 +591,51 @@ public final class FreeCol {
     public static File getAutosaveDirectory() {
         return saveDirectory;
     }
+    
+    public static boolean initializeResourceFolders() {
+        
+        try {
+            FreeColDataFile baseData = new FreeColDataFile(new File(dataFolder, "base"));
+            try {
+                ResourceManager.setBaseMapping(baseData.getResourceMapping());
+            } finally {
+                baseData.close();
+            }
+        } catch (IOException e) {
+            System.err.println("Could not find base data directory.");
+            return false;
+        }
+        FreeColTcFile tcData;
+        try {
+            tcData = new FreeColTcFile(new File(dataFolder, tc));
+            try {
+                ResourceManager.setTcMapping(tcData.getResourceMapping());
+            } finally {
+                tcData.close();
+            }
+        } catch (IOException e) {
+            System.err.println("Could not load: " + tc);
+            return false;
+        }
+        
+        // This needs to be initialized before ImageLibrary
+        try {
+            Specification.createSpecification(tcData.getSpecificationInputStream());
+        } catch (IOException e) {
+            System.err.println("Could not load specification.xml for: " + tc);
+            return false;
+        }
+
+        ResourceManager.update();
+        return true;
+    }
 
     /**
-    * Ensure that the Java version is good enough. JDK 1.4 or better is
-    * required.
-    *
-    * @return true if Java version is at least 1.4.0.
-    */
+     * Ensure that the Java version is good enough. JDK 1.4 or better is
+     * required.
+     *
+     * @return true if Java version is at least 1.5.0.
+     */
     private static boolean checkJavaVersion() {
         // Must use string comparison because some JVM's provide
         // versions like "1.4.1"
@@ -511,7 +665,23 @@ public final class FreeCol {
                     }
                 } else {
                     printUsage();
-                    System.exit(0);
+                    System.exit(1);
+                }
+            } else if (args[i].equals("--tc")) {
+                i++;
+                if (i < args.length) {
+                    tc = args[i];
+                } else {
+                    printUsage();
+                    System.exit(1);
+                }
+            } else if (args[i].equals("--home-directory")) {
+                i++;
+                if (i < args.length) {
+                    mainUserDirectory = new File(args[i]);
+                } else {
+                    printUsage();
+                    System.exit(1);
                 }
             } else if (args[i].equals("--log-console")) {
                 consoleLogging = true;
@@ -529,7 +699,7 @@ public final class FreeCol {
                     }
                 } else {
                     printUsage();
-                    System.exit(0);
+                    System.exit(1);
                 }
             } else if (args[i].equals("--no-java-check")) {
                 javaCheck = false;
@@ -557,11 +727,11 @@ public final class FreeCol {
                         windowSize = new Dimension(x, y);
                     } catch (Exception e) {
                         printUsage();
-                        System.exit(0);
+                        System.exit(1);
                     }
                 } else if (args[i].length() != 10) {
                     printUsage();
-                    System.exit(0);
+                    System.exit(1);
                 }
                 
                 windowed = true;
@@ -576,7 +746,7 @@ public final class FreeCol {
                     Locale.setDefault(LanguageOption.getLocale(languageID));
                 } else {
                     printUsage();
-                    System.exit(0);
+                    System.exit(1);
                 }
             } else if (args[i].equals("--no-sound")) {
                 sound = false;
@@ -606,20 +776,15 @@ public final class FreeCol {
                     System.out.println("The text after the \"--server\" option should be a valid port number.");
                     System.exit(1);
                 }
+            } else if (args[i].equals("--private")) {
+                publicServer = false;
             } else if (args[i].equals("--load-savegame")) {
                 i++;
                 if (i < args.length) {
-                    savegameFile = new File(args[i]);
-                    if (!savegameFile.exists() || !savegameFile.isFile()) {                        
-                        savegameFile = new File(getSaveDirectory(), args[i]);
-                        if (!savegameFile.exists() || !savegameFile.isFile()) {
-                            System.out.println("The given savegame file could not be found: " + args[i]);
-                            System.exit(1);
-                        }
-                    }
+                    setSavegame(args[i]);
                 } else {
                     printUsage();
-                    System.exit(0);
+                    System.exit(1);
                 }
             } else if (args[i].equals("--server-help")) {
                 printServerUsage();
@@ -638,6 +803,19 @@ public final class FreeCol {
                 serverName = args[i];
             } else if (args[i].startsWith("--splash")) {
                 // Ignore - already handled;
+            } else if (args[i].equals("--check-savegame")) {
+                // Debugging aid, checks the integrity of the specified save
+                // game, and exits with corresponding status
+                i++;
+                if (i < args.length) {
+                    setSavegame(args[i]);
+                } else {
+                    printUsage();
+                    System.exit(1);
+                }
+                checkIntegrity = true;
+                standAloneServer = true;
+                serverPort = DEFAULT_PORT;
             } else {
                 printUsage();
                 System.exit(1);
@@ -647,11 +825,11 @@ public final class FreeCol {
 
 
     /**
-    * Gets the current version of game.
-    *
-    * @return The current version of the game using the format "x.y.z",
-    *         where "x" is major, "y" is minor and "z" is revision.
-    */
+     * Gets the current version of game.
+     *
+     * @return The current version of the game using the format "x.y.z",
+     *         where "x" is major, "y" is minor and "z" is revision.
+     */
     public static String getVersion() {
         return FREECOL_VERSION;
     }
@@ -666,29 +844,29 @@ public final class FreeCol {
     }
 
     /**
-    * Checks if the program is in "Debug mode".
-    * @return <code>true</code> if the program is in debug
-    *       mode and <code>false</code> otherwise.
-    */
+     * Checks if the program is in "Debug mode".
+     * @return <code>true</code> if the program is in debug
+     *       mode and <code>false</code> otherwise.
+     */
     public static boolean isInDebugMode() {
         return inDebugMode;
     }
 
     
     /**
-    * Sets the "debug mode" to be active or not.
-    * @param debug Should be <code>true</code> in order
-    *       to active debug mode and <code>false</code>
-    *       otherwise.
-    */
+     * Sets the "debug mode" to be active or not.
+     * @param debug Should be <code>true</code> in order
+     *       to active debug mode and <code>false</code>
+     *       otherwise.
+     */
     public static void setInDebugMode(boolean debug) {
         inDebugMode = debug;
     }
 
 
     /**
-    * Prints the command-line usage for the server options.
-    */
+     * Prints the command-line usage for the server options.
+     */
     private static void printServerUsage() {
         System.out.println("Usage: java -Xmx512M -jar FreeCol.jar --server PORT [OPTIONS]");
         System.out.println("");
@@ -698,7 +876,7 @@ public final class FreeCol {
         System.out.println("--server-name NAME");
         System.out.println("  specifies a custom name for the server");
         System.out.println("--load-savegame SAVEGAME_FILE");
-        System.out.println("  loads the given savegame.");      
+        System.out.println("  loads the given savegame.");
         System.out.println("--no-java-check");
         System.out.println("  skips the java version check");        
         System.out.println();
@@ -706,38 +884,46 @@ public final class FreeCol {
 
 
     /**
-    * Prints the command-line usage (the 'help' for command-line
-    * arguments).
-    */
+     * Prints the command-line usage (the 'help' for command-line
+     * arguments).
+     */
     private static void printUsage() {
         System.out.println("Usage: java -Xmx512M -jar FreeCol.jar [OPTIONS]");
         System.out.println("");
         System.out.println("Options:");
+        System.out.println("--home-directory DIR");
+        System.out.println("  sets the FreeCol home directory, defaults to user home.");
+        System.out.println("--default-locale=LANGUAGE[_COUNTRY[_VARIANT]]");
+        System.out.println("  sets the default locale.");
         System.out.println("--freecol-data DIR");
         System.out.println("  DIR should be the directory with FreeCol's data files, it");
         System.out.println("  has a subdirectory called 'images'");
-        System.out.println("--windowed[[=]WIDTHxHEIGHT]");
-        System.out.println("  runs FreeCol in windowed mode instead of full screen mode");
         System.out.println("--load-savegame SAVEGAME_FILE");
         System.out.println("  loads the given savegame.");
-        System.out.println("--default-locale=LANGUAGE[_COUNTRY[_VARIANT]]");
-        System.out.println("  sets the default locale.");
-        System.out.println("--splash[=SPLASH_IMAGE_FILE]");
-        System.out.println("  displays a splash screen while loading the game");
-        System.out.println("--no-sound");
-        System.out.println("  runs FreeCol without sound");
+        System.out.println("--log-level=LOGLEVEL");
+        System.out.println("  sets the log-level to LOGLEVEL.");
         System.out.println("--no-java-check");
         System.out.println("  skips the java version check");
         System.out.println("--no-memory-check");
         System.out.println("  skips the memory check");        
+        System.out.println("--no-sound");
+        System.out.println("  runs FreeCol without sound");
+        System.out.println("--server PORT");
+        System.out.println("  starts a stand-alone server on the specifed port");
+        System.out.println("--private");
+        System.out.println("  starts a private server (not published to the metaserver)");
+        System.out.println("--server-help");
+        System.out.println("  displays a help screen for the more advanced server options");
+        System.out.println("--splash[=SPLASH_IMAGE_FILE]");
+        System.out.println("  displays a splash screen while loading the game");
+        System.out.println("--tc NAME");
+        System.out.println("  Loads the total conversion with the given NAME");
         System.out.println("--usage");
         System.out.println("  displays this help screen");
         System.out.println("--version");
         System.out.println("  displays the version number");
-        System.out.println("--server PORT");
-        System.out.println("  starts a stand-alone server on the specifed port");
-        System.out.println("--server-help");
-        System.out.println("  displays a help screen for the more advanced server options");
+        System.out.println("--windowed[[=]WIDTHxHEIGHT]");
+        System.out.println("  runs FreeCol in windowed mode instead of full screen mode");
         System.out.println();
     }
 }
