@@ -35,6 +35,7 @@ import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.CanvasMapEditorMouseListener;
 import net.sf.freecol.client.gui.CanvasMapEditorMouseMotionListener;
 import net.sf.freecol.client.gui.GUI;
+import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.MapEditorMenuBar;
 import net.sf.freecol.client.gui.action.MapControlsAction;
 import net.sf.freecol.client.gui.i18n.Messages;
@@ -45,6 +46,7 @@ import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.networking.NoRouteToServerException;
 import net.sf.freecol.server.FreeColServer;
+import net.sf.freecol.server.generator.IMapGenerator;
 import net.sf.freecol.server.generator.MapGenerator;
 import net.sf.freecol.server.generator.MapGeneratorOptions;
 
@@ -96,10 +98,16 @@ public final class MapEditorController {
             canvas.closeMenus();            
             gui.setInGame(true);
             
+            // We may need to reset the zoom value to the default value
+            ImageLibrary im = freeColClient.getImageLibrary();
+            if(gui.getImageLibrary() != im){
+            	gui.setImageLibrary(im);
+            }
+            
             freeColClient.getCanvas().setJMenuBar(new MapEditorMenuBar(freeColClient));
             JInternalFrame f = freeColClient.getCanvas().addAsToolBox(new MapEditorTransformPanel(canvas));
             f.setLocation(f.getX(), 50);
-            
+            	
             canvas.repaint();            
 
             canvas.addMouseListener(new CanvasMapEditorMouseListener(canvas, gui));
@@ -157,18 +165,22 @@ public final class MapEditorController {
     public void newMap() {
         final Canvas canvas = freeColClient.getCanvas();
         final Game game = freeColClient.getGame();
-        final MapGenerator mapGenerator = freeColClient.getFreeColServer().getMapGenerator();        
+        final IMapGenerator mapGenerator = freeColClient.getFreeColServer().getMapGenerator();        
         
         boolean ok = canvas.showMapGeneratorOptionsDialog(true, mapGenerator.getMapGeneratorOptions());
         if (!ok) {
             return;
         }
         
-        mapGenerator.createMap(game);        
-        
-        freeColClient.getGUI().setFocus(1, 1);
-        freeColClient.getActionManager().update();
-        canvas.refresh();
+        try {
+            mapGenerator.createMap(game);
+            freeColClient.getGUI().setFocus(1, 1);
+            freeColClient.getActionManager().update();
+            canvas.refresh();
+        } catch (FreeColException e) {
+            canvas.closeMenus();
+            canvas.errorMessage( e.getMessage() );
+        }
     }
     
     /**
@@ -192,7 +204,7 @@ public final class MapEditorController {
         final Canvas canvas = freeColClient.getCanvas();
 
         canvas.showStatusPanel(Messages.message("status.savingGame"));
-        Thread t = new Thread() {
+        Thread t = new Thread(FreeCol.CLIENT_THREAD+"Saving Map") {
             public void run() {
                 try {
                     freeColClient.getFreeColServer().saveGame(file, "mapEditor");
@@ -254,7 +266,7 @@ public final class MapEditorController {
                 canvas.closeMenus();
                 canvas.errorMessage( message );
             }
-        };
+        }
 
         canvas.showStatusPanel(Messages.message("status.loadingGame"));
         
@@ -274,48 +286,32 @@ public final class MapEditorController {
                         }
                     } );                    
                 } catch (NoRouteToServerException e) {
-                    final FreeColServer theFreeColServer = freeColServer;
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            if (theFreeColServer != null) {
-                                theFreeColServer.getServer().shutdown();
-                            }
                             freeColClient.getCanvas().closeMainPanel();
                             freeColClient.getCanvas().showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("server.noRouteToServer") );
                 } catch (FileNotFoundException e) {                    
-                    final FreeColServer theFreeColServer = freeColServer;
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            if (theFreeColServer != null) {
-                                theFreeColServer.getServer().shutdown();
-                            }
                             freeColClient.getCanvas().closeMainPanel();
                             freeColClient.getCanvas().showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("fileNotFound") );
                 } catch (IOException e) {
-                    final FreeColServer theFreeColServer = freeColServer;
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            if (theFreeColServer != null) {
-                                theFreeColServer.getServer().shutdown();
-                            }
                             freeColClient.getCanvas().closeMainPanel();
                             freeColClient.getCanvas().showMainPanel();
                         }
                     });
                     SwingUtilities.invokeLater( new ErrorJob("server.couldNotStart") );
                 } catch (FreeColException e) {
-                    final FreeColServer theFreeColServer = freeColServer;
                     SwingUtilities.invokeLater(new Runnable() {
                         public void run() {
-                            if (theFreeColServer != null) {
-                                theFreeColServer.getServer().shutdown();
-                            }
                             freeColClient.getCanvas().closeMainPanel();
                             freeColClient.getCanvas().showMainPanel();
                         }
