@@ -32,8 +32,11 @@ import net.sf.freecol.client.gui.CanvasMouseMotionListener;
 import net.sf.freecol.client.gui.GUI;
 import net.sf.freecol.client.gui.InGameMenuBar;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.client.gui.sound.SoundPlayer;
 import net.sf.freecol.common.model.GameOptions;
 import net.sf.freecol.common.model.ModelMessage;
+import net.sf.freecol.common.model.Nation;
+import net.sf.freecol.common.model.NationType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
@@ -107,21 +110,37 @@ public final class PreGameController {
 
 
     /**
-    * Sets this client's player's nation.
-    * @param nation Which nation this player wishes to set.
-    */
-    public void setNation(int nation) {
+     * Sets this client's player's nation.
+     * @param nation Which nation this player wishes to set.
+     */
+    public void setNation(Nation nation) {
         // Make the change:
         freeColClient.getMyPlayer().setNation(nation);
 
         // Inform the server:
         Element nationElement = Message.createNewRootElement("setNation");
-        nationElement.setAttribute("value", Integer.toString(nation));
+        nationElement.setAttribute("value", nation.getId());
 
         freeColClient.getClient().sendAndWait(nationElement);
     }
 
 
+    /**
+     * Sets this client's player's nation type.
+     * @param nationType Which nation this player wishes to set.
+     */
+    public void setNationType(NationType nationType) {
+        // Make the change:
+        freeColClient.getMyPlayer().setNationType(nationType);
+
+        // Inform the server:
+        Element nationTypeElement = Message.createNewRootElement("setNationType");
+        nationTypeElement.setAttribute("value", nationType.getId());
+
+        freeColClient.getClient().sendAndWait(nationTypeElement);
+    }
+
+
     /**
     * Sets this client's player's color.
     * @param color Which color this player wishes to set.
@@ -132,7 +151,10 @@ public final class PreGameController {
 
         // Inform the server:
         Element colorElement = Message.createNewRootElement("setColor");
-        colorElement.setAttribute("value", Player.convertColorToString(color));
+        // the substring hack is necessary to prevent an integer
+        // overflow, as the hex string will be unsigned, and
+        // Integer.decode() expects a signed int
+        colorElement.setAttribute("value", "#" + Integer.toHexString(color.getRGB()).substring(2));
 
         freeColClient.getClient().sendAndWait(colorElement);
     }
@@ -201,19 +223,28 @@ public final class PreGameController {
         Canvas canvas = freeColClient.getCanvas();
         GUI gui = freeColClient.getGUI();
 
-        canvas.closeMainPanel();
-        canvas.closeMenus();
+        if (!freeColClient.isHeadless()) {
+            canvas.closeMainPanel();
+            canvas.closeMenus();
+        
+            // TODO: Nation specific intro-music:
+            freeColClient.playMusicOnce("england", SoundPlayer.STANDARD_DELAY);
+        }
 
         InGameController inGameController = freeColClient.getInGameController();
         InGameInputHandler inGameInputHandler = freeColClient.getInGameInputHandler();
 
         freeColClient.getClient().setMessageHandler(inGameInputHandler);
-        gui.setInGame(true);
 
-        freeColClient.getCanvas().setJMenuBar(new InGameMenuBar(freeColClient));
+        if (!freeColClient.isHeadless()) {
+            gui.setInGame(true);
+            freeColClient.getCanvas().setJMenuBar(new InGameMenuBar(freeColClient));
+        }
+
         if (freeColClient.getGame().getTurn().getNumber() == 1) {
             Player player = freeColClient.getMyPlayer();
-            player.addModelMessage(new ModelMessage(player, "tutorial.startGame", null, ModelMessage.TUTORIAL, player));
+            player.addModelMessage(new ModelMessage(player, ModelMessage.MessageType.TUTORIAL, 
+                                                    player, "tutorial.startGame"));
         }
 
         Unit activeUnit = freeColClient.getMyPlayer().getNextActiveUnit();
@@ -228,7 +259,7 @@ public final class PreGameController {
         canvas.addKeyListener(new CanvasKeyListener(canvas, inGameController));
         canvas.addMouseListener(new CanvasMouseListener(canvas, gui));
         canvas.addMouseMotionListener(new CanvasMouseMotionListener(canvas, gui, freeColClient.getGame().getMap()));
-
+        
         if (freeColClient.getMyPlayer().equals(freeColClient.getGame().getCurrentPlayer())) {
             canvas.requestFocus();
             freeColClient.getInGameController().nextModelMessage();
