@@ -27,26 +27,25 @@ import java.util.logging.Logger;
 
 import javax.swing.SwingUtilities;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.common.model.Map;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Tile;
 
 /**
  * Listens to the mouse being moved at the level of the Canvas.
  */
 public final class CanvasMapEditorMouseMotionListener implements MouseMotionListener {
+
     private static final Logger logger = Logger.getLogger(CanvasMapEditorMouseMotionListener.class.getName());
 
-
-
-
     private final Canvas canvas;
 
     private final GUI gui;
 
     private ScrollThread scrollThread;
 
-    // private static final int SCROLLSPACE = 3;
     private static final int SCROLLSPACE = 100;
 
 
@@ -61,6 +60,18 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
         gui = g;
         scrollThread = null;
     }
+    
+    
+    /**
+     * This method can be called to make sure the map is loaded
+     * There is no point executing mouse events if the map is not loaded
+     */
+    private Map getMap() {
+        Map map = null;
+        if (canvas.getClient().getGame() != null)
+            map = canvas.getClient().getGame().getMap();
+        return map;
+    }
 
     /**
      * Invoked when the mouse has been moved.
@@ -68,6 +79,9 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseMoved(MouseEvent e) {
+        if (getMap() == null) {
+            return;
+        }
         /*
          * if (e.getComponent().isEnabled()) { scroll(e.getX(), e.getY()); }
          * else if (scrollThread != null) { scrollThread.stopScrolling();
@@ -80,6 +94,9 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
     }
 
     private void scroll(int x, int y) {
+        if (getMap() == null) {
+            return;
+        }
         if (!canvas.getClient().getClientOptions().getBoolean(ClientOptions.MAP_SCROLL_ON_DRAG)) {
             return;
         }
@@ -89,31 +106,31 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
          * if (y < canvas.getMenuBarHeight() + SCROLLSPACE) { y -=
          * canvas.getMenuBarHeight(); }
          */
-        int direction;
+        Direction direction;
         if ((x < SCROLLSPACE) && (y < SCROLLSPACE)) {
             // Upper-Left
-            direction = Map.NW;
+            direction = Direction.NW;
         } else if ((x >= gui.getWidth() - SCROLLSPACE) && (y < SCROLLSPACE)) {
             // Upper-Right
-            direction = Map.NE;
+            direction = Direction.NE;
         } else if ((x >= gui.getWidth() - SCROLLSPACE) && (y >= gui.getHeight() - SCROLLSPACE)) {
             // Bottom-Right
-            direction = Map.SE;
+            direction = Direction.SE;
         } else if ((x < SCROLLSPACE) && (y >= gui.getHeight() - SCROLLSPACE)) {
             // Bottom-Left
-            direction = Map.SW;
+            direction = Direction.SW;
         } else if (y < SCROLLSPACE) {
             // Top
-            direction = Map.N;
+            direction = Direction.N;
         } else if (x >= gui.getWidth() - SCROLLSPACE) {
             // Right
-            direction = Map.E;
+            direction = Direction.E;
         } else if (y >= gui.getHeight() - SCROLLSPACE) {
             // Bottom
-            direction = Map.S;
+            direction = Direction.S;
         } else if (x < SCROLLSPACE) {
             // Left
-            direction = Map.W;
+            direction = Direction.W;
         } else {
             // Center
             if (scrollThread != null) {
@@ -122,11 +139,14 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
             }
             return;
         }
-
+        
         if (scrollThread != null) {
+            // continue scrolling in a (perhaps new) direction
             scrollThread.setDirection(direction);
         } else {
-            scrollThread = new ScrollThread(canvas.getClient().getGame().getMap(), gui);
+            // start scrolling in a direction
+            scrollThread = new ScrollThread(getMap(), gui);
+            scrollThread.setDirection(direction);
             scrollThread.start();
         }
     }
@@ -137,6 +157,9 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
      * @param e The MouseEvent that holds all the information.
      */
     public void mouseDragged(MouseEvent e) {
+        if (getMap() == null) {
+            return;
+        }
         Map.Position p = gui.convertToMapCoordinates(e.getX(), e.getY());
 
         if (e.getComponent().isEnabled()) {
@@ -146,7 +169,7 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
             scrollThread = null;
         }
 
-        if (p == null || !canvas.getClient().getGame().getMap().isValid(p)) {
+        if (p == null || !getMap().isValid(p)) {
             return;
         }
         
@@ -154,7 +177,7 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
             return;
         }
 
-        Tile tile = canvas.getClient().getGame().getMap().getTile(p);
+        Tile tile = getMap().getTile(p);
         if (tile != null) {
             canvas.getClient().getMapEditorController().transform(tile);
             canvas.refresh();
@@ -166,11 +189,12 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
      * Scrolls the view of the Map by moving its focus.
      */
     private class ScrollThread extends Thread {
+
         private final Map map;
 
         private final GUI gui;
 
-        private int direction;
+        private Direction direction;
 
         private boolean cont;
 
@@ -182,6 +206,7 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
          * @param g The GUI that holds information such as screen resolution.
          */
         public ScrollThread(Map m, GUI g) {
+            super(FreeCol.CLIENT_THREAD+"Mouse scroller");
             map = m;
             gui = g;
             cont = true;
@@ -192,7 +217,7 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
          * 
          * @param d The direction in which this ScrollThread will scroll.
          */
-        public void setDirection(int d) {
+        public void setDirection(Direction d) {
             direction = d;
         }
 
@@ -218,7 +243,7 @@ public final class CanvasMapEditorMouseMotionListener implements MouseMotionList
                         public void run() {
                             try {
                                 int x, y;
-                                Tile t = map.getTileOrNull(gui.getFocus().getX(), gui.getFocus().getY());
+                                Tile t = map.getTile(gui.getFocus().getX(), gui.getFocus().getY());
                                 if (t == null) {
                                     return;
                                 }
