@@ -70,6 +70,6 @@ public class ColopediaGoodsAction extends MapboardAction {
      * @param e The <code>ActionEvent</code>.
      */
     public void actionPerformed(ActionEvent e) {
-        freeColClient.getCanvas().showColopediaPanel(ColopediaPanel.COLOPEDIA_GOODS);
+        freeColClient.getCanvas().showColopediaPanel(ColopediaPanel.PanelType.GOODS);
     }
 }
