@@ -17,8 +17,6 @@
  *  along with FreeCol.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
-
 package net.sf.freecol.client.gui.action;
 
 import java.awt.event.ActionEvent;
@@ -33,22 +31,21 @@ import net.sf.freecol.client.gui.panel.MapControls;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.Unit;
 
-
 /**
-* An action for unloading the currently selected unit.
-*/
+ * An action for unloading the currently selected unit.
+ */
 public class UnloadAction extends MapboardAction {
+
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(UnloadAction.class.getName());
 
-
     public static final String id = "unloadAction";
 
     /**
      * Creates this action.
      * @param freeColClient The main controller object for the client.
      */
-    UnloadAction(FreeColClient freeColClient) {
+    public UnloadAction(FreeColClient freeColClient) {
         super(freeColClient, "menuBar.orders.unload", null, KeyStroke.getKeyStroke('U', 0));
     }
 
@@ -66,16 +63,11 @@ public class UnloadAction extends MapboardAction {
      * @return <code>true</code> if there is a carrier active
      */
     protected boolean shouldBeEnabled() {
-        if (!super.shouldBeEnabled()) return false;
-        GUI gui = getFreeColClient().getGUI();
-        if (gui == null) return false;
-    
-        Unit unit = getFreeColClient().getGUI().getActiveUnit();
-        if (unit != null && unit.isCarrier()) {
-            if (unit.getGoodsCount() > 0) {
-                return true;
-            } else if (unit.getUnitCount() > 0 && (unit.isInEurope() || unit.getColony() != null)) {
-                return true;
+        if (super.shouldBeEnabled()) {
+            GUI gui = getFreeColClient().getGUI();
+            if (gui != null) {
+                Unit unit = getFreeColClient().getGUI().getActiveUnit();
+                return (unit != null && unit.isCarrier() && unit.getGoodsCount() > 0);
             }
         }
         return false;
