@@ -24,20 +24,23 @@ import java.awt.Dimension;
 import java.awt.Image;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.util.List;
 import java.util.logging.Logger;
 
 import javax.swing.ImageIcon;
 import javax.swing.JButton;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
+import javax.swing.JScrollPane;
 import javax.swing.JTabbedPane;
 import javax.swing.JTextArea;
-import javax.swing.UIManager;
 import javax.swing.border.EmptyBorder;
 
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.FoundingFather;
+import net.sf.freecol.common.model.FoundingFather.FoundingFatherType;
+import net.sf.freecol.common.resources.ResourceManager;
 
 /**
  * This panel displays the different founding fathers the player can work
@@ -48,13 +51,10 @@ import net.sf.freecol.common.model.FoundingFather;
 public final class ChooseFoundingFatherDialog extends FreeColDialog implements ActionListener {
     private static final Logger logger = Logger.getLogger(ChooseFoundingFatherDialog.class.getName());
 
-
-
-
     @SuppressWarnings("unused")
     private final Canvas parent;
 
-    private FoundingFatherPanel[] foundingFatherPanels = new FoundingFatherPanel[FoundingFather.TYPE_COUNT];
+    //    private ArrayList<FoundingFatherPanel> foundingFatherPanels = new ArrayList<FoundingFatherPanel>();
 
     private final JTabbedPane tb;
 
@@ -75,13 +75,6 @@ public final class ChooseFoundingFatherDialog extends FreeColDialog implements A
         setOpaque(false);
 
         tb = new JTabbedPane(JTabbedPane.TOP);
-
-        for (int i = 0; i < foundingFatherPanels.length; i++) {
-            foundingFatherPanels[i] = new FoundingFatherPanel(i);
-            tb.addTab(FoundingFather.getTypeAsString(i), null, foundingFatherPanels[i], null);
-        }
-
-        add(tb);
     }
 
     /**
@@ -93,17 +86,19 @@ public final class ChooseFoundingFatherDialog extends FreeColDialog implements A
      *            identifies a <code>FoundingFather</code> to be picked in
      *            each of those categories.
      */
-    public void initialize(int[] possibleFoundingFathers) {
+    public void initialize(List<FoundingFather> possibleFoundingFathers) {
         boolean hasSelectedTab = false;
-        for (int i = 0; i < possibleFoundingFathers.length; i++) {
-            foundingFatherPanels[i].initialize(possibleFoundingFathers[i]);
-            tb.setEnabledAt(i, foundingFatherPanels[i].isEnabled());
-            if (!hasSelectedTab && foundingFatherPanels[i].isEnabled()) {
-                tb.setSelectedIndex(i);
+        for (int index = 0; index < possibleFoundingFathers.size(); index++) {
+            FoundingFather father = possibleFoundingFathers.get(index);
+            FoundingFatherPanel panel = new FoundingFatherPanel(father.getType());
+            panel.initialize(father, index);
+            tb.addTab(father.getTypeAsString(), null, panel, null);
+            if (!hasSelectedTab && panel.isEnabled()) {
+                tb.setSelectedIndex(index);
                 hasSelectedTab = true;
             }
         }
-
+        add(tb);
         setSize(tb.getPreferredSize());
     }
 
@@ -120,7 +115,7 @@ public final class ChooseFoundingFatherDialog extends FreeColDialog implements A
     public void actionPerformed(ActionEvent event) {
         String command = event.getActionCommand();
         try {
-            setResponse(Integer.valueOf(command));
+            setResponse(Integer.valueOf(command).intValue());
         } catch (NumberFormatException e) {
             logger.warning("Invalid Actioncommand: not a number.");
             setResponse(null);
@@ -133,8 +128,7 @@ public final class ChooseFoundingFatherDialog extends FreeColDialog implements A
      */
     protected class FoundingFatherPanel extends JPanel {
 
-        // private final int type;
-        private int foundingFather = -1;
+        private FoundingFather foundingFather = null;
 
         private JLabel header;
 
@@ -144,9 +138,6 @@ public final class ChooseFoundingFatherDialog extends FreeColDialog implements A
 
         private JButton ok;
 
-        private JPanel p1;
-
-
         /**
          * Creates a <code>FoundingFatherPanel</code> for a given type of
          * founding fathers.
@@ -154,50 +145,63 @@ public final class ChooseFoundingFatherDialog extends FreeColDialog implements A
          * @param type The type of founding fathers to be displayed in this
          *            <code>FoundingFatherPanel</code>.
          */
-        public FoundingFatherPanel(int type) {
-            // this.type = type;
+        public FoundingFatherPanel(FoundingFatherType type) {
 
             setLayout(new BorderLayout());
 
+            // header area to display the father name
             header = getDefaultHeader("");
-
             add(header, BorderLayout.NORTH);
 
-            p1 = new JPanel();
+            // content area with image and text
+            JPanel contentPanel = createContentPanel(type);
+            add(contentPanel, BorderLayout.CENTER);
+
+            // selection button at the bottom
+            JPanel p3 = new JPanel(new BorderLayout());
+            p3.setOpaque(false);
+            p3.setBorder(new EmptyBorder(0, 160, 20, 160));
+            ok = new JButton(Messages.message("chooseThisFoundingFather"));
+            ok.addActionListener(chooseFoundingFatherDialog);
+            ok.setSize(ok.getPreferredSize());
+            enterPressesWhenFocused(ok);
+            p3.add(ok, BorderLayout.CENTER);
+            add(p3, BorderLayout.SOUTH);
+        }
+        
+        private JPanel createContentPanel(FoundingFatherType type) {
+            
+            JPanel p1 = new JPanel();
             p1.setLayout(new BorderLayout(20, 20));
             p1.setOpaque(false);
             p1.setBorder(new EmptyBorder(20, 20, 20, 20));
 
-            Image image = null;
-            switch (type) {
-            case 0:
-                image = (Image) UIManager.get("FoundingFather.trade");
-                break;
-            case 1:
-                image = (Image) UIManager.get("FoundingFather.exploration");
-                break;
-            case 2:
-                image = (Image) UIManager.get("FoundingFather.military");
-                break;
-            case 3:
-                image = (Image) UIManager.get("FoundingFather.political");
-                break;
-            case 4:
-                image = (Image) UIManager.get("FoundingFather.religious");
-                break;
-            }
-
+            // the image on the top/left
+            Image image = ResourceManager.getImage("FoundingFather." + type.toString().toLowerCase());
             JLabel imageLabel;
             if (image != null) {
                 imageLabel = new JLabel(new ImageIcon(image));
             } else {
                 imageLabel = new JLabel();
             }
+            JPanel imagePanel = new JPanel(new BorderLayout());
+            imagePanel.add(imageLabel, BorderLayout.NORTH);
+            p1.add(imagePanel, BorderLayout.WEST);
 
-            p1.add(imageLabel, BorderLayout.WEST);
-
+            // text on the right side
             JPanel p2 = new JPanel(new BorderLayout());
             p2.setOpaque(false);
+            // use a scroll panel if the text is too long in some languages
+            JScrollPane scrollPane = new JScrollPane(p2,
+                    JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
+                    JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
+            scrollPane.getViewport().setOpaque(false);
+            scrollPane.getVerticalScrollBar().setUnitIncrement( 16 );
+            scrollPane.setOpaque(false);
+            scrollPane.setBorder(null);
+            p1.add(scrollPane, BorderLayout.CENTER);
+            // and control the overall window size, for the scroll panel to work
+            this.setPreferredSize(new Dimension(570, 400));
 
             description = new JTextArea();
             description.setBorder(null);
@@ -216,53 +220,34 @@ public final class ChooseFoundingFatherDialog extends FreeColDialog implements A
             text.setWrapStyleWord(true);
             text.setFocusable(false);
             p2.add(text, BorderLayout.CENTER);
-
-            JPanel p3 = new JPanel(new BorderLayout());
-            p3.setOpaque(false);
-            p3.setBorder(new EmptyBorder(0, 160, 20, 160));
-            ok = new JButton(Messages.message("chooseThisFoundingFather"));
-            ok.addActionListener(chooseFoundingFatherDialog);
-            ok.setSize(ok.getPreferredSize());
-            enterPressesWhenFocused(ok);
-            p3.add(ok, BorderLayout.CENTER);
-
-            p1.add(p2, BorderLayout.CENTER);
-            add(p1, BorderLayout.CENTER);
-            add(p3, BorderLayout.SOUTH);
+            
+            return p1;
         }
 
         public void requestFocus() {
             ok.requestFocus();
         }
 
-        public Dimension getPreferredSize() {
-            return new Dimension(570, super.getPreferredSize().height);
-        }
-
         /**
          * Prepares this panel to be displayed.
          * 
-         * @param foundingFather The founding father to be displayed or
+         * @param father The founding father to be displayed or
          *            <code>-1</code> if there is none.
          */
-        public void initialize(int foundingFather) {
-            this.foundingFather = foundingFather;
+        public void initialize(FoundingFather father, int index) {
+            this.foundingFather = father;
 
-            if (foundingFather != -1) {
-                header.setText(Messages.message(FoundingFather.getName(foundingFather)));
-                description.setText(Messages.message(FoundingFather.getDescription(foundingFather)));
-                text.setText("\n" + "[" + Messages.message(FoundingFather.getBirthAndDeath(foundingFather)) + "] "
-                        + Messages.message(FoundingFather.getText(foundingFather)));
-                ok.setActionCommand(Integer.toString(foundingFather));
+            if (father != null) {
+                header.setText(father.getName());
+                description.setText(father.getDescription());
+                text.setText("\n" + "[" + father.getBirthAndDeath() + "] "
+                        + father.getText());
+                ok.setActionCommand(String.valueOf(index));
             }
         }
 
         public boolean isEnabled() {
-            if (foundingFather != -1) {
-                return true;
-            } else {
-                return false;
-            }
+            return (foundingFather != null);
         }
     }
 }
