@@ -24,13 +24,12 @@ import java.awt.Color;
 import java.awt.Component;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.util.Vector;
+import java.util.List;
 import java.util.logging.Logger;
 
 import javax.swing.AbstractCellEditor;
 import javax.swing.JButton;
 import javax.swing.JColorChooser;
-import javax.swing.JLayeredPane;
 import javax.swing.JPanel;
 import javax.swing.JTable;
 import javax.swing.table.TableCellEditor;
@@ -60,7 +59,7 @@ public final class ColorCellEditor extends AbstractCellEditor implements TableCe
     private final Canvas               canvas;
 
     private Color currentColor;
-    private Vector<Player> players;
+    private List<Player> players;
     private int lastRow;
     private Player thisPlayer;
 
@@ -128,7 +127,7 @@ public final class ColorCellEditor extends AbstractCellEditor implements TableCe
     * @param players The players that should be edited in the table.
     * @param owningPlayer The player running the client that is displaying the table.
     */
-    public void setData(Vector<Player> players, Player owningPlayer) {
+    public void setData(List<Player> players, Player owningPlayer) {
         this.players = players;
         thisPlayer = owningPlayer;
     }
@@ -144,18 +143,21 @@ public final class ColorCellEditor extends AbstractCellEditor implements TableCe
     }
 
     /**
-    * This function analyses an event and calls the right methods to take
+    * This function analyzes an event and calls the right methods to take
     * care of the user's requests.
     * @param event The incoming ActionEvent.
     */
     public void actionPerformed(ActionEvent event) {
         if (event.getActionCommand().equals(EDIT)) {
-            colorChooser.setColor(currentColor);
-
-            // Add the colorChooserPanel.
-            canvas.add(colorChooserPanel, JLayeredPane.PALETTE_LAYER);
-            parent.setEnabled(false);
-            // No repainting needed apparently.
+            if (!canvas.isAncestorOf(colorChooserPanel)) {
+                colorChooser.setColor(currentColor);
+    
+                // Add the colorChooserPanel.
+                canvas.addAsFrame(colorChooserPanel);
+                colorChooserPanel.requestFocus();
+                parent.setEnabled(false);
+                // No repainting needed apparently.
+            }
         }
         else if (event.getActionCommand().equals(OK)) {
             currentColor = colorChooser.getColor();
