@@ -40,6 +40,8 @@ import javax.swing.UIManager;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.FAFile;
+import net.sf.freecol.common.resources.ResourceManager;
+import net.sf.freecol.common.util.Utils;
 
 /**
  * This panel displays the signing of the Declaration of Independence.
@@ -74,7 +76,7 @@ public final class DeclarationDialog extends FreeColDialog {
 
         setLayout(null);
 
-        Image image = (Image) UIManager.get("Declaration.image");
+        Image image = ResourceManager.getImage("Declaration.image");
         setSize(image.getWidth(null), image.getHeight(null));
         setOpaque(false);
         setBorder(null);
@@ -104,7 +106,7 @@ public final class DeclarationDialog extends FreeColDialog {
      * Paints an image of the Declaration of Independence on this panel.
      */
     public void paintComponent(Graphics g) {
-        Image image = (Image) UIManager.get("Declaration.image");
+        Image image = ResourceManager.getImage("Declaration.image");
         g.drawImage(image, 0, 0, null);
     }
 
@@ -171,12 +173,12 @@ public final class DeclarationDialog extends FreeColDialog {
             String[] partNames = name.split(" ");
 
             // Abbreviate middle names:
-            for (int i = 1; i < partNames.length - 1 && isTooLarge(join(partNames, " ")); i++) {
+            for (int i = 1; i < partNames.length - 1 && isTooLarge(Utils.join(" ", partNames)); i++) {
                 partNames[i] = partNames[i].charAt(0) + ".";
             }
 
             // Remove middle names:
-            while (partNames.length > 2 && isTooLarge(join(partNames, " "))) {
+            while (partNames.length > 2 && isTooLarge(Utils.join(" ", partNames))) {
                 String[] newPartNames = new String[partNames.length - 1];
                 newPartNames[0] = partNames[0];
                 for (int i = 1; i < newPartNames.length; i++) {
@@ -185,8 +187,8 @@ public final class DeclarationDialog extends FreeColDialog {
                 partNames = newPartNames;
             }
 
-            if (!isTooLarge(join(partNames, " "))) {
-                return join(partNames, " ");
+            if (!isTooLarge(Utils.join(" ", partNames))) {
+                return Utils.join(" ", partNames);
             } else if (!isTooLarge(partNames[0].charAt(0) + ". " + partNames[1])) {
                 return partNames[0].charAt(0) + ". " + partNames[1];
             } else if (!isTooLarge(partNames[0] + " " + partNames[1].charAt(0) + ".")) {
@@ -196,23 +198,6 @@ public final class DeclarationDialog extends FreeColDialog {
             }
         }
 
-        /**
-         * Joins the given strings.
-         * 
-         * @param strings The strings to be joined.
-         * @param delim The delimiter to place between the individual strings.
-         * @return Each of the strings in the given array delimited by the given
-         *         string.
-         */
-        private String join(String[] strings, String delim) {
-            String s = strings[0];
-            for (int i = 1; i < strings.length; i++) {
-                s += delim;
-                s += strings[i];
-            }
-            return s;
-        }
-
         /**
          * Checks if the given string is to large to be displayed within this
          * panel.
