@@ -43,6 +43,7 @@ import java.awt.event.MouseEvent;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.util.logging.Logger;
+import java.util.Set;
 
 import javax.swing.ImageIcon;
 import javax.swing.JComponent;
@@ -51,7 +52,11 @@ import javax.swing.JPanel;
 import javax.swing.TransferHandler;
 
 import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.model.Modifier;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.UnitState;
 
 /**
 * The transferhandler that is capable of creating ImageSelection objects.
@@ -168,14 +173,14 @@ public final class DefaultTransferHandler extends TransferHandler {
                   If not, assume that the user wished to drop the unit/cargo on the panel below.
                 */
                 if (((UnitLabel) comp).getUnit().isCarrier() && ((UnitLabel) comp).getParent() instanceof EuropePanel.InPortPanel) {
-                    if (data instanceof UnitLabel && ((UnitLabel) data).getUnit().getLocation() instanceof Unit
+                    if (data instanceof UnitLabel && ((UnitLabel) data).getUnit().isOnCarrier()
                             || data instanceof GoodsLabel && ((GoodsLabel) data).getGoods().getLocation() instanceof Unit) {
                         oldSelectedUnit = ((EuropePanel) parentPanel).getSelectedUnitLabel();
                     }
                     ((EuropePanel) parentPanel).setSelectedUnitLabel((UnitLabel) comp);
                     comp = ((EuropePanel) parentPanel).getCargoPanel();
                 } else if (((UnitLabel) comp).getUnit().isCarrier() && ((UnitLabel) comp).getParent() instanceof ColonyPanel.InPortPanel) {
-                    if (data instanceof UnitLabel && ((UnitLabel) data).getUnit().getLocation() instanceof Unit
+                    if (data instanceof UnitLabel && ((UnitLabel) data).getUnit().isOnCarrier()
                             || data instanceof GoodsLabel && ((GoodsLabel) data).getGoods().getLocation() instanceof Unit) {
                         oldSelectedUnit = ((ColonyPanel) parentPanel).getSelectedUnitLabel();
                     }
@@ -218,15 +223,15 @@ public final class DefaultTransferHandler extends TransferHandler {
                     return false;
                 }
                 
-                if ((unit.getState() == Unit.TO_AMERICA) && (!(comp instanceof EuropePanel.ToEuropePanel))) {
+                if ((unit.getState() == UnitState.TO_AMERICA) && (!(comp instanceof EuropePanel.ToEuropePanel))) {
                     return false;
                 }
 
-                if ((unit.getState() == Unit.TO_EUROPE) && (!(comp instanceof EuropePanel.ToAmericaPanel))) {
+                if ((unit.getState() == UnitState.TO_EUROPE) && (!(comp instanceof EuropePanel.ToAmericaPanel))) {
                     return false;
                 }
 
-                if ((unit.getState() != Unit.TO_AMERICA) && ((comp instanceof EuropePanel.ToEuropePanel))) {
+                if ((unit.getState() != UnitState.TO_AMERICA) && ((comp instanceof EuropePanel.ToEuropePanel))) {
                     return false;
                 }
 
@@ -241,12 +246,11 @@ public final class DefaultTransferHandler extends TransferHandler {
                     return false;
                 }
 
-                if (unit.isNaval() && (comp instanceof EuropePanel.CargoPanel
-                        || comp instanceof EuropePanel.DocksPanel
+                if (unit.isNaval() && (comp instanceof EuropePanel.DocksPanel
                         || comp instanceof ColonyPanel.OutsideColonyPanel
                         || comp instanceof ColonyPanel.BuildingsPanel.ASingleBuildingPanel
                         || comp instanceof ColonyPanel.TilePanel.ASingleTilePanel
-                        || comp instanceof ColonyPanel.CargoPanel)) {
+                        || comp instanceof CargoPanel)) {
                     return false;
                 }
 
@@ -263,14 +267,25 @@ public final class DefaultTransferHandler extends TransferHandler {
                         ((EuropePanel.ToAmericaPanel)comp).add(data, true);
                     } else if (comp instanceof EuropePanel.DocksPanel) {
                         ((EuropePanel.DocksPanel)comp).add(data, true);
-                    } else if (comp instanceof EuropePanel.CargoPanel) {
-                        ((EuropePanel.CargoPanel)comp).add(data, true);
                     } else if (comp instanceof ColonyPanel.BuildingsPanel.ASingleBuildingPanel) {
                         ((ColonyPanel.BuildingsPanel.ASingleBuildingPanel) comp).add(data, true);
                     } else if (comp instanceof ColonyPanel.OutsideColonyPanel) {
-                        ((ColonyPanel.OutsideColonyPanel) comp).add(data, true);
-                    } else if (comp instanceof ColonyPanel.CargoPanel) {
-                        ((ColonyPanel.CargoPanel)comp).add(data, true);
+                        ColonyPanel.OutsideColonyPanel outside = ((ColonyPanel.OutsideColonyPanel) comp);
+                        if (outside.getColony().canReducePopulation()) {
+                            outside.add(data, true);
+                        } else {
+                            String message = "";
+                            Set<Modifier> modifierSet = outside.getColony().getFeatureContainer()
+                                .getModifierSet("model.modifier.minimumColonySize");
+                            for (Modifier modifier : modifierSet) {
+                                message += Messages.message("colonyPanel.minimumColonySize",
+                                                            "%object%", modifier.getSource().getName())
+                                    + "\n";
+                            }
+                            canvas.showInformationMessage(message);
+                        }
+                    } else if (comp instanceof CargoPanel) {
+                        ((CargoPanel)comp).add(data, true);
                     } else if (comp instanceof ColonyPanel.TilePanel.ASingleTilePanel) {
                         ((ColonyPanel.TilePanel.ASingleTilePanel)comp).add(data, true);
                     } else {
@@ -304,12 +319,14 @@ public final class DefaultTransferHandler extends TransferHandler {
                         return false;
                     }
                     label.getGoods().setAmount(amount);
+                } else if (label.getGoods().getAmount() > 100) {
+                    label.getGoods().setAmount(100);
                 }
 
-                if (!(comp instanceof ColonyPanel.WarehousePanel || comp instanceof ColonyPanel.CargoPanel
-                        || comp instanceof EuropePanel.MarketPanel || comp instanceof EuropePanel.CargoPanel)
-                    || (comp instanceof EuropePanel.CargoPanel && !((EuropePanel.CargoPanel) comp).isActive())
-                    || (comp instanceof ColonyPanel.CargoPanel && !((ColonyPanel.CargoPanel) comp).isActive())) {
+                if (!(comp instanceof ColonyPanel.WarehousePanel || 
+                      comp instanceof CargoPanel ||
+                      comp instanceof EuropePanel.MarketPanel) || 
+                    (comp instanceof CargoPanel && !((CargoPanel) comp).isActive())) {
 
                     return false;
                 }
@@ -323,12 +340,10 @@ public final class DefaultTransferHandler extends TransferHandler {
 
                     if (comp instanceof ColonyPanel.WarehousePanel) {
                         ((ColonyPanel.WarehousePanel)comp).add(data, true);
-                    } else if (comp instanceof ColonyPanel.CargoPanel) {
-                        ((ColonyPanel.CargoPanel)comp).add(data, true);
+                    } else if (comp instanceof CargoPanel) {
+                        ((CargoPanel)comp).add(data, true);
                     } else if (comp instanceof EuropePanel.MarketPanel) {
                         ((EuropePanel.MarketPanel)comp).add(data, true);
-                    } else if (comp instanceof EuropePanel.CargoPanel) {
-                        ((EuropePanel.CargoPanel)comp).add(data, true);
                     } else {
                         logger.warning("The receiving component is of an invalid type.");
                         return false;
@@ -370,8 +385,8 @@ public final class DefaultTransferHandler extends TransferHandler {
                     // Be not removing MarketLabels from their home. -sjm
                     //data.getParent().remove(data);
 
-                    if (comp instanceof EuropePanel.CargoPanel) {
-                        ((EuropePanel.CargoPanel)comp).add(data, true);
+                    if (comp instanceof CargoPanel) {
+                        ((CargoPanel)comp).add(data, true);
                     } else {
                         logger.warning("The receiving component is of an invalid type.");
                         return false;
@@ -398,7 +413,7 @@ public final class DefaultTransferHandler extends TransferHandler {
     /**
     * Displays an input dialog box where the user should specify a goods transfer amount.
     */
-    private int getAmount(int goodsType, int available, boolean needToPay) {
+    private int getAmount(GoodsType goodsType, int available, boolean needToPay) {
         return canvas.showSelectAmountDialog(goodsType, available, needToPay);
     }
 
