@@ -21,11 +21,9 @@ package net.sf.freecol.client.gui.panel;
 
 import java.awt.Color;
 import java.awt.Component;
-import java.awt.Container;
 import java.awt.Graphics;
 import java.awt.GridLayout;
 import java.awt.Image;
-import java.awt.MediaTracker;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.KeyEvent;
@@ -46,12 +44,12 @@ import javax.swing.JTextPane;
 import javax.swing.KeyStroke;
 import javax.swing.ScrollPaneConstants;
 import javax.swing.SwingUtilities;
-import javax.swing.UIManager;
 import javax.swing.border.TitledBorder;
 import javax.swing.text.SimpleAttributeSet;
 import javax.swing.text.StyleConstants;
 import javax.swing.text.StyledDocument;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.ClientOptions;
 import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.control.InGameController;
@@ -60,9 +58,12 @@ import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.TransactionListener;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.common.resources.ResourceManager;
 import cz.autel.dmi.HIGLayout;
 
 /**
@@ -71,13 +72,9 @@ import cz.autel.dmi.HIGLayout;
  */
 public final class EuropePanel extends FreeColPanel implements ActionListener {
 
-
-
     private static Logger logger = Logger.getLogger(EuropePanel.class.getName());
 
-    private static final int EXIT = 0, RECRUIT = 1, PURCHASE = 2, TRAIN = 3, UNLOAD = 4;
-
-    // private static final int TITLE_FONT_SIZE = 12;
+    public static enum EuropeAction { EXIT, RECRUIT, PURCHASE, TRAIN, UNLOAD, SAIL }
 
     private final Canvas parent;
 
@@ -93,7 +90,7 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
 
     private final DocksPanel docksPanel;
 
-    private final CargoPanel cargoPanel;
+    private final EuropeCargoPanel cargoPanel;
 
     private final MarketPanel marketPanel;
     
@@ -128,44 +125,70 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
 
         // Use ESCAPE for closing the ColonyPanel:
         exitButton = new JButton(Messages.message("close"));
+        exitButton.setActionCommand(EuropeAction.EXIT.toString());
+        exitButton.addActionListener(this);
         InputMap closeInputMap = new ComponentInputMap(exitButton);
         closeInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false), "pressed");
         closeInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, true), "released");
         SwingUtilities.replaceUIInputMap(exitButton, JComponent.WHEN_IN_FOCUSED_WINDOW, closeInputMap);
+        enterPressesWhenFocused(exitButton);
 
         // train button
         JButton trainButton = new JButton(Messages.message("train"));
+        trainButton.setActionCommand(EuropeAction.TRAIN.toString());
+        trainButton.addActionListener(this);
         InputMap trainInputMap = new ComponentInputMap(trainButton);
         trainInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_T, 0, false), "pressed");
         trainInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_T, 0, true), "released");
         SwingUtilities.replaceUIInputMap(trainButton, JComponent.WHEN_IN_FOCUSED_WINDOW, trainInputMap);
+        enterPressesWhenFocused(trainButton);
 
         // purchase button
         JButton purchaseButton = new JButton(Messages.message("purchase"));
+        purchaseButton.setActionCommand(EuropeAction.PURCHASE.toString());
+        purchaseButton.addActionListener(this);
         InputMap purchaseInputMap = new ComponentInputMap(purchaseButton);
         purchaseInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_P, 0, false), "pressed");
         purchaseInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_P, 0, true), "released");
         SwingUtilities.replaceUIInputMap(purchaseButton, JComponent.WHEN_IN_FOCUSED_WINDOW, purchaseInputMap);
+        enterPressesWhenFocused(purchaseButton);
 
         // recruit button
         JButton recruitButton = new JButton(Messages.message("recruit"));
+        recruitButton.setActionCommand(EuropeAction.RECRUIT.toString());
+        recruitButton.addActionListener(this);
         InputMap recruitInputMap = new ComponentInputMap(recruitButton);
         recruitInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_R, 0, false), "pressed");
         recruitInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_R, 0, true), "released");
         SwingUtilities.replaceUIInputMap(recruitButton, JComponent.WHEN_IN_FOCUSED_WINDOW, recruitInputMap);
+        enterPressesWhenFocused(recruitButton);
 
         // unload button
         JButton unloadButton = new JButton(Messages.message("unload"));
+        unloadButton.setActionCommand(EuropeAction.UNLOAD.toString());
+        unloadButton.addActionListener(this);
         InputMap unloadInputMap = new ComponentInputMap(unloadButton);
         unloadInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_U, 0, false), "pressed");
         unloadInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_U, 0, true), "released");
         SwingUtilities.replaceUIInputMap(unloadButton, JComponent.WHEN_IN_FOCUSED_WINDOW, unloadInputMap);
+        enterPressesWhenFocused(unloadButton);
+
+        // sail button
+        JButton sailButton = new JButton(Messages.message("sail"));
+        sailButton.setActionCommand(EuropeAction.SAIL.toString());
+        sailButton.addActionListener(this);
+        InputMap sailInputMap = new ComponentInputMap(sailButton);
+        sailInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_S, 0, false), "pressed");
+        sailInputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_S, 0, true), "released");
+        SwingUtilities.replaceUIInputMap(sailButton, JComponent.WHEN_IN_FOCUSED_WINDOW, sailInputMap);
+        enterPressesWhenFocused(sailButton);
 
         toAmericaPanel = new ToAmericaPanel(this);
         toEuropePanel = new ToEuropePanel(this);
         inPortPanel = new InPortPanel();
         docksPanel = new DocksPanel(this);
-        cargoPanel = new CargoPanel(this);
+        cargoPanel = new EuropeCargoPanel(parent);
+        cargoPanel.setParentPanel(this);
         marketPanel = new MarketPanel(this);
         
         log = new TransactionLog();
@@ -197,8 +220,8 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
         toEuropePanel.addMouseListener(releaseListener);
         inPortPanel.addMouseListener(releaseListener);
         docksPanel.addMouseListener(releaseListener);
-        cargoPanel.addMouseListener(releaseListener);
         marketPanel.addMouseListener(releaseListener);
+        cargoPanel.addMouseListener(releaseListener);
 
         toAmericaPanel.setLayout(new GridLayout(0, 2));
         toEuropePanel.setLayout(new GridLayout(0, 2));
@@ -208,25 +231,28 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
 
         JScrollPane toAmericaScroll = new JScrollPane(toAmericaPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
+        toAmericaScroll.getVerticalScrollBar().setUnitIncrement( 16 );
         JScrollPane toEuropeScroll = new JScrollPane(toEuropePanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
+        toEuropeScroll.getVerticalScrollBar().setUnitIncrement( 16 );
         JScrollPane inPortScroll = new JScrollPane(inPortPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
+        inPortScroll.getVerticalScrollBar().setUnitIncrement( 16 );
         JScrollPane docksScroll = new JScrollPane(docksPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
+        docksScroll.getVerticalScrollBar().setUnitIncrement( 16 );
         JScrollPane cargoScroll = new JScrollPane(cargoPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER,
                 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
         JScrollPane marketScroll = new JScrollPane(marketPanel, ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER,
                 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
         JScrollPane logScroll = new JScrollPane(log, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                 ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
+        logScroll.getVerticalScrollBar().setUnitIncrement( 16 );
 
         toAmericaPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(), Messages
                 .message("goingToAmerica")));
         toEuropePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(), Messages
                 .message("goingToEurope")));
-        cargoPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(), Messages
-                .message("cargo")));
         docksPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(), Messages
                 .message("docks")));
         inPortPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEmptyBorder(), Messages
@@ -254,17 +280,17 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
         unloadButton.setOpaque(false);
 
         int[] widths = { 0, 315, margin, 103, margin, 198, margin, 0, 0 };
-        int[] heights = { 0, // top margin
-                120, margin, // log
-                // sailing to America, sailing to Europe
-                39, margin, // recruit button
-                39, margin, // buy button
-                39, margin, // train button
-                39, margin, // unload button
-                116, margin, // in port
-                116, margin, // cargo
-                75, 39, // market
-                0 // bottom margin
+        int[] heights = {
+            30, // top margin
+            // sailing to America, sailing to Europe
+            39, margin, // recruit button
+            39, margin, // buy button
+            39, margin, // train button
+            39, margin, // unload button
+            116, margin, // in port
+            75, 39, margin, // cargo
+            75, 39, margin, // market
+            120, margin // log
         };
 
         HIGLayout layout = new HIGLayout(widths, heights);
@@ -275,8 +301,6 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
         setLayout(layout);
 
         int row = 2;
-        add(logScroll, higConst.rcwh(row, 2, 6, 1));
-        row += 2;
         add(toAmericaScroll, higConst.rcwh(row, 2, 1, 7));
         add(toEuropeScroll, higConst.rcwh(row, 4, 3, 7));
         add(recruitButton, higConst.rc(row, 8));
@@ -288,31 +312,18 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
         add(unloadButton, higConst.rc(row, 8));
         row += 2;
         add(inPortScroll, higConst.rcwh(row, 2, 3, 1));
-        add(docksScroll, higConst.rcwh(row, 6, 1, 3));
+        add(docksScroll, higConst.rcwh(row, 6, 1, 4));
         row += 2;
-        add(cargoScroll, higConst.rcwh(row, 2, 3, 1));
+        add(cargoScroll, higConst.rcwh(row, 2, 3, 2));
+        row += 1;
+        add(sailButton, higConst.rc(row, 8));
         row += 2;
         add(marketScroll, higConst.rcwh(row, 2, 5, 2));
         row += 1;
         add(exitButton, higConst.rc(row, 8));
+        row += 2;
+        add(logScroll, higConst.rcwh(row, 2, 6, 1));
 
-        exitButton.setActionCommand(String.valueOf(EXIT));
-        recruitButton.setActionCommand(String.valueOf(RECRUIT));
-        purchaseButton.setActionCommand(String.valueOf(PURCHASE));
-        trainButton.setActionCommand(String.valueOf(TRAIN));
-        unloadButton.setActionCommand(String.valueOf(UNLOAD));
-        
-        enterPressesWhenFocused(exitButton);
-        enterPressesWhenFocused(recruitButton);
-        enterPressesWhenFocused(purchaseButton);
-        enterPressesWhenFocused(trainButton);
-        enterPressesWhenFocused(unloadButton);
-
-        exitButton.addActionListener(this);
-        recruitButton.addActionListener(this);
-        purchaseButton.addActionListener(this);
-        trainButton.addActionListener(this);
-        unloadButton.addActionListener(this);
         setBorder(null);
 
         selectedUnit = null;
@@ -346,36 +357,11 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
         int width = getWidth();
         int height = getHeight();
 
-        Image bgImage = (Image) UIManager.get("EuropeBackgroundImage.scaled");
-        if (bgImage == null) {
-            bgImage = (Image) UIManager.get("EuropeBackgroundImage");
-        }
+        Image bgImage = ResourceManager.getImage("EuropeBackgroundImage", parent.getSize());
         if (bgImage != null) {
-            if (bgImage.getWidth(null) != parent.getWidth() || bgImage.getHeight(null) != parent.getHeight()) {
-                final Image fullSizeBgImage = (Image) UIManager.get("EuropeBackgroundImage");
-                bgImage = fullSizeBgImage.getScaledInstance(parent.getWidth(), parent.getHeight(), Image.SCALE_SMOOTH);
-                UIManager.put("EuropeBackgroundImage.scaled", bgImage);
-
-                /*
-                 * We have to use a MediaTracker to ensure that the image has
-                 * been scaled before we paint it.
-                 */
-                MediaTracker mt = new MediaTracker(freeColClient.getCanvas());
-                mt.addImage(bgImage, 0, parent.getWidth(), parent.getHeight());
-
-                try {
-                    mt.waitForID(0);
-                } catch (InterruptedException e) {
-                    g.setColor(Color.black);
-                    g.fillRect(0, 0, parent.getWidth(), parent.getHeight());
-                    return;
-                }
-
-            }
-
-            g.drawImage(bgImage, 0, 0, null);
+            g.drawImage(bgImage, 0, 0, this);
         } else {
-            Image tempImage = (Image) UIManager.get("BackgroundImage");
+            Image tempImage = ResourceManager.getImage("BackgroundImage");
 
             if (tempImage != null) {
                 for (int x = 0; x < width; x += tempImage.getWidth(null)) {
@@ -394,14 +380,15 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
      * Refreshes the components on this panel that need to be refreshed after
      * the user has recruited a new unit.
      */
-    public void refreshBuyRecruit() {
+    public void refreshDocks() {
         docksPanel.removeAll();
 
         Iterator<Unit> unitIterator = europe.getUnitIterator();
         while (unitIterator.hasNext()) {
             Unit unit = unitIterator.next();
 
-            if (((unit.getState() == Unit.ACTIVE) || (unit.getState() == Unit.SENTRY)) && (!unit.isNaval())) {
+            if (((unit.getState() == UnitState.ACTIVE) ||
+                 (unit.getState() == UnitState.SENTRY)) && (!unit.isNaval())) {
                 UnitLabel unitLabel = new UnitLabel(unit, parent);
                 unitLabel.setTransferHandler(defaultTransferHandler);
                 unitLabel.addMouseListener(pressListener);
@@ -417,29 +404,24 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
     /**
      * Refreshes the components on this panel that need to be refreshed after
      * the user has purchased a new unit.
-     * 
-     * @param type The type of unit that was just purchased. This is needed to
-     *            know which component needs to be refreshed.
-     */
-    public void refreshBuyPurchase(int type) {
-        if (type == Unit.ARTILLERY) {
-            refreshBuyRecruit();
-        } else {
-            inPortPanel.removeAll();
+      */
+    public void refreshInPort() {
+        inPortPanel.removeAll();
 
-            List<Unit> unitIterator = europe.getUnitList();
-            for (Unit unit : unitIterator) {
-                if ((unit.getState() == Unit.ACTIVE) && (unit.isNaval())) {
-                    UnitLabel unitLabel = new UnitLabel(unit, parent);
-                    unitLabel.setTransferHandler(defaultTransferHandler);
-                    unitLabel.addMouseListener(pressListener);
-                    inPortPanel.add(unitLabel);
-                }
+        List<Unit> units = europe.getUnitList();
+        for (Unit unit : units) {
+            if ((unit.getState() == UnitState.ACTIVE) && (unit.isNaval())) {
+                UnitLabel unitLabel = new UnitLabel(unit, parent);
+                unitLabel.setTransferHandler(defaultTransferHandler);
+                unitLabel.addMouseListener(pressListener);
+                inPortPanel.add(unitLabel);
             }
+        }
 
-            // Only one component will be repainted!
-            inPortPanel.repaint(0, 0, inPortPanel.getWidth(), inPortPanel.getHeight());
-            setSelectedUnit(unitIterator.get(unitIterator.size() - 1));
+        // Only one component will be repainted!
+        inPortPanel.repaint(0, 0, inPortPanel.getWidth(), inPortPanel.getHeight());
+        if (!units.isEmpty()) {
+            setSelectedUnit(units.get(units.size() - 1));
         }
     }
 
@@ -486,32 +468,36 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
                 // Naval units can either be in the port, going to europe or
                 // going to america.
                 switch (unit.getState()) {
-                case Unit.ACTIVE:
+                case ACTIVE:
+                default:
                     lastCarrier = unitLabel;
                     inPortPanel.add(unitLabel);
                     break;
-                case Unit.TO_EUROPE:
+                case TO_EUROPE:
                     toEuropePanel.add(unitLabel, false);
                     break;
-                case Unit.TO_AMERICA:
+                case TO_AMERICA:
                     toAmericaPanel.add(unitLabel, false);
                     break;
-                default:
-                    // This should normally not happen, but until the problem is solved, i comment it out.
-                    // throw new RuntimeException("Naval unit in Europe is in an invalid state.");
-                    // TODO: Find the cause of units not arriving active in Europe.
                 }
             }
         }
         // We set the last carrier in the list active.
         setSelectedUnitLabel(lastCarrier);
 
+        if (lastCarrier != null) {
+            cargoPanel.setCarrier(lastCarrier.getUnit());
+        }
+
         Player player = freeColClient.getMyPlayer();
-        for (int i = 0; i < Goods.NUMBER_OF_TYPES; i++) {
-            MarketLabel marketLabel = new MarketLabel(i, player.getMarket(), parent);
-            marketLabel.setTransferHandler(defaultTransferHandler);
-            marketLabel.addMouseListener(pressListener);
-            marketPanel.add(marketLabel);
+        List<GoodsType> goodsTypes = FreeCol.getSpecification().getGoodsTypeList();
+        for (GoodsType goodsType : goodsTypes) {
+            if (goodsType.isStorable()) {
+                MarketLabel marketLabel = new MarketLabel(goodsType, player.getMarket(), parent);
+                marketLabel.setTransferHandler(defaultTransferHandler);
+                marketLabel.addMouseListener(pressListener);
+                marketPanel.add(marketLabel);
+            }
         }
 
         String newLandName = player.getNewLandName();
@@ -548,58 +534,21 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
             selectedUnit.setSelected(false);
         }
         selectedUnit = unitLabel;
+        if (unitLabel == null) {
+            cargoPanel.setCarrier(null);
+        } else {
+            cargoPanel.setCarrier(unitLabel.getUnit());
+            unitLabel.setSelected(true);
+        }
+
         updateCargoPanel();
-        updateCargoLabel();
 
         cargoPanel.revalidate();
         refresh();
     }
 
     private void updateCargoPanel() {
-        cargoPanel.removeAll();
-
-        if (selectedUnit != null) {
-            selectedUnit.setSelected(true);
-            Unit selUnit = selectedUnit.getUnit();
-
-            Iterator<Unit> unitIterator = selUnit.getUnitIterator();
-            while (unitIterator.hasNext()) {
-                Unit unit = unitIterator.next();
-
-                UnitLabel label = new UnitLabel(unit, parent);
-                label.setTransferHandler(defaultTransferHandler);
-                label.addMouseListener(pressListener);
-
-                cargoPanel.add(label, false);
-            }
-
-            Iterator<Goods> goodsIterator = selUnit.getGoodsIterator();
-            while (goodsIterator.hasNext()) {
-                Goods g = goodsIterator.next();
-
-                GoodsLabel label = new GoodsLabel(g, parent);
-                label.setTransferHandler(defaultTransferHandler);
-                label.addMouseListener(pressListener);
-
-                cargoPanel.add(label, false);
-            }
-        }
-
-    }
-
-    /**
-     * Updates the label that is placed above the cargo panel. It shows the name
-     * of the unit whose cargo is displayed and the amount of space left on that
-     * unit.
-     */
-    private void updateCargoLabel() {
-        if (selectedUnit != null) {
-            ((TitledBorder) cargoPanel.getBorder()).setTitle(Messages.message("cargo") + " ("
-                    + selectedUnit.getUnit().getName() + ") " + Messages.message("spaceLeft") + ": "
-                    + selectedUnit.getUnit().getSpaceLeft());
-        } else {
-            ((TitledBorder) cargoPanel.getBorder()).setTitle(Messages.message("cargo"));
-        }
+        cargoPanel.initialize();
     }
 
     /**
@@ -633,7 +582,6 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
                 Goods goods = goodsIterator.next();
                 if (player.canTrade(goods)) {
                     inGameController.sellGoods(goods);
-                    updateCargoLabel();
                     updateCargoPanel();
                 } else {
                     inGameController.payArrears(goods);
@@ -645,11 +593,10 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
             while (unitIterator.hasNext()) {
                 Unit newUnit = unitIterator.next();
                 inGameController.leaveShip(newUnit);
-                updateCargoLabel();
                 updateCargoPanel();
                 getCargoPanel().revalidate();
                 // update docks panel
-                refreshBuyRecruit();
+                refreshDocks();
                 docksPanel.revalidate();
                 refresh();
             }
@@ -666,11 +613,11 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
         String command = event.getActionCommand();
         try {
             // Get Command
-            int intCommand = Integer.valueOf(command).intValue();
+            EuropeAction europeAction = Enum.valueOf(EuropeAction.class, command);
             // Close any open Europe Dialog, and show new one if required
-            int Response = parent.showEuropeDialog(intCommand);
-            // Process Command
-            switch (intCommand) {
+            parent.showEuropeDialog(europeAction);
+            // Refresh if necessary
+            switch (europeAction) {
             case EXIT:
                 freeColClient.getMyPlayer().getMarket().removeTransactionListener(log);
                 parent.remove(this);
@@ -679,20 +626,24 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
             case RECRUIT:
             case PURCHASE:
             case TRAIN:
-                if (intCommand == PURCHASE) {
-                    if (Response > -1) {
-                        refreshBuyPurchase(Response);
-                    }
-                } else {
-                    if (Response == 0) {
-                        refreshBuyRecruit();
-                    }
-                }
+                refreshDocks();
+                refreshInPort();
                 revalidate();
                 break;
             case UNLOAD:
                 unload();
                 break;
+            case SAIL:
+                Unit unit = getSelectedUnit();
+                if (unit != null && unit.isNaval()) {
+                    freeColClient.getInGameController().moveToAmerica(unit);
+                    UnitLabel unitLabel = getSelectedUnitLabel();
+                    inPortPanel.remove(unitLabel);
+                    toAmericaPanel.add(unitLabel, false);
+                    refreshInPort();
+                    revalidate();
+                }
+                break;
             default:
                 logger.warning("Invalid action command");
             }
@@ -704,16 +655,45 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
     /**
      * Asks for pay arrears of a type of goods, if those goods are boycotted
      * 
-     * @param typeOfGoods The type of goods for paying arrears
+     * @param goodsType The type of goods for paying arrears
      */
-    public void payArrears(int typeOfGoods) {
-        if (freeColClient.getMyPlayer().getArrears(typeOfGoods) > 0) {
-            inGameController.payArrears(typeOfGoods);
+    public void payArrears(GoodsType goodsType) {
+        if (freeColClient.getMyPlayer().getArrears(goodsType) > 0) {
+            inGameController.payArrears(goodsType);
             getMarketPanel().revalidate();
             refresh();
         }
     }
 
+    public void loadedGoods(Goods goods) {
+        marketPanel.revalidate();
+        revalidate();
+        refresh();
+    }
+
+    public void loadedUnit(Unit unit) {
+        refreshDocks();
+    }
+
+    public final class EuropeCargoPanel extends CargoPanel {
+        public EuropeCargoPanel(Canvas parent) {
+            super(parent, true);
+        }
+        
+        @Override
+        public Component add(Component comp, boolean editState) {
+            Component result = super.add(comp, editState);
+            updateCargoPanel();
+            getCargoPanel().revalidate();
+            refresh();
+            return result;
+        }
+
+        @Override
+        public String getUIClassID() {
+            return "EuropeCargoPanelUI";
+        }
+    }
 
     /**
      * A panel that holds UnitsLabels that represent Units that are going to
@@ -757,9 +737,7 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
                                 "europe.leaveColonists",
                                 "yes",
                                 "no",
-                                new String[][] {
-                                    {"%newWorld%", unit.getOwner().getNewLandName()}
-                                });
+                                "%newWorld%", unit.getOwner().getNewLandName());
                         if (!leaveColonists) {
                             // Remain in Europe.
                             return null;
@@ -787,6 +765,7 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
             }
             setSelectedUnitLabel(null);
             Component c = add(comp);
+            toAmericaPanel.revalidate();
             europePanel.refresh();
             return c;
         }
@@ -901,15 +880,18 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
             if (editState) {
                 if (comp instanceof UnitLabel) {
                     comp.getParent().remove(comp);
+                    /* Unnecessary, leaveShip() is called in CargoPanel.remove()
                     Unit unit = ((UnitLabel) comp).getUnit();
                     inGameController.leaveShip(unit);
+                    */
                 } else {
                     logger.warning("An invalid component got dropped on this DocksPanel.");
                     return null;
                 }
             }
-            updateCargoLabel();
+
             Component c = add(comp);
+            revalidate();
             europePanel.refresh();
             return c;
         }
@@ -929,135 +911,8 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
         }
     }
 
-    /**
-     * A panel that holds units and goods that represent Units and cargo that
-     * are on board the currently selected ship.
-     */
-    public final class CargoPanel extends JPanel {
-        private final EuropePanel europePanel;
 
 
-        /**
-         * Creates this CargoPanel.
-         * 
-         * @param europePanel The panel that holds this CargoPanel.
-         */
-        public CargoPanel(EuropePanel europePanel) {
-            this.europePanel = europePanel;
-        }
-
-        /**
-         * Adds a component to this CargoPanel and makes sure that the unit or
-         * good that the component represents gets modified so that it is on
-         * board the currently selected ship.
-         * 
-         * @param comp The component to add to this CargoPanel.
-         * @param editState Must be set to 'true' if the state of the component
-         *            that is added (which should be a dropped component
-         *            representing a Unit or good) should be changed so that the
-         *            underlying unit or goods are on board the currently
-         *            selected ship.
-         * @return The component argument.
-         */
-        public Component add(Component comp, boolean editState) {
-            if (selectedUnit == null) {
-                return null;
-            }
-
-            if (editState) {
-                Container oldParent = comp.getParent();
-
-                if (comp instanceof UnitLabel) {
-                    Unit unit = ((UnitLabel) comp).getUnit();
-                    if (selectedUnit.getUnit().getSpaceLeft() > 0) {
-                        inGameController.boardShip(unit, selectedUnit.getUnit());
-
-                        if (comp.getParent() != null) {
-                            comp.getParent().remove(comp);
-                        }
-                    } else {
-                        return null;
-                    }
-                } else if (comp instanceof GoodsLabel) {
-                    Goods g = ((GoodsLabel) comp).getGoods();
-
-                    Unit carrier = getSelectedUnit();
-                    int newAmount = g.getAmount();
-                    if (carrier.getSpaceLeft() == 0
-                            && carrier.getGoodsContainer().getGoodsCount(g.getType()) % 100 + g.getAmount() > 100) {
-                        newAmount = 100 - carrier.getGoodsContainer().getGoodsCount(g.getType()) % 100;
-                    } else if (g.getAmount() > 100) {
-                        newAmount = 100;
-                    }
-
-                    if (newAmount == 0) {
-                        return null;
-                    }
-
-                    if (g.getAmount() != newAmount) {
-                        g.setAmount(g.getAmount() - newAmount);
-                        g = new Goods(game, g.getLocation(), g.getType(), newAmount);
-                    } else {
-                        if (oldParent != null) {
-                            oldParent.remove(comp);
-                        }
-                    }
-
-                    if (!selectedUnit.getUnit().canAdd(g)) {
-                        return null;
-                    }
-
-                    inGameController.loadCargo(g, selectedUnit.getUnit());
-
-                    // TODO: Make this look prettier :-)
-                    UnitLabel t = selectedUnit;
-                    selectedUnit = null;
-                    setSelectedUnitLabel(t);
-                    // reinitialize();
-
-                    return comp;
-                } else if (comp instanceof MarketLabel) {
-                    MarketLabel label = (MarketLabel) comp;
-                    Player player = freeColClient.getMyPlayer();
-                    if (player.canTrade(label.getType())) {
-                        inGameController.buyGoods(label.getType(), label.getAmount(), selectedUnit.getUnit());
-                        inGameController.nextModelMessage();
-                        updateCargoLabel();
-
-                        // TODO: Make this look prettier :-)
-                        UnitLabel t = selectedUnit;
-                        selectedUnit = null;
-                        setSelectedUnitLabel(t);
-
-                        europePanel.getMarketPanel().revalidate();
-                        revalidate();
-                        europePanel.refresh();
-                        return comp;
-                    }
-
-                    inGameController.payArrears(label.getType());
-                    return null;
-                } else {
-                    logger.warning("An invalid component got dropped on this CargoPanel.");
-                    return null;
-                }
-            }
-
-            updateCargoLabel();
-            Component c = add(comp);
-            europePanel.refresh();
-            return c;
-        }
-
-        public boolean isActive() {
-            return (getSelectedUnit() != null);
-        }
-
-        public String getUIClassID() {
-            return "EuropeCargoPanelUI";
-        }
-    }
-
     /**
      * A panel that shows goods available for purchase in Europe.
      */
@@ -1092,9 +947,17 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
                     Player player = freeColClient.getMyPlayer();
                     if (player.canTrade(goods)) {
                         inGameController.sellGoods(goods);
-                        updateCargoLabel();
                     } else {
-                        inGameController.payArrears(goods);
+                        switch (parent.showBoycottedGoodsDialog(goods, europe)) {
+                        case PAY_ARREARS:
+                            inGameController.payArrears(goods);
+                            break;
+                        case DUMP_CARGO:
+                            inGameController.unloadCargo(goods);
+                            break;
+                        case CANCEL:
+                        default:
+                        }
                     }
                     europePanel.getCargoPanel().revalidate();
                     revalidate();
@@ -1145,10 +1008,10 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
             }
         }
         
-        public void logPurchase(int goodsType, int amount, int price) {
+        public void logPurchase(GoodsType goodsType, int amount, int price) {
             int total = amount * price;
             String text = Messages.message("transaction.purchase",
-                    "%goods%", Goods.getName(goodsType),
+                    "%goods%", goodsType.getName(),
                     "%amount%", String.valueOf(amount),
                     "%gold%", String.valueOf(price))
                 + "\n" + Messages.message("transaction.price",
@@ -1156,13 +1019,13 @@ public final class EuropePanel extends FreeColPanel implements ActionListener {
             add(text);
         }
 
-        public void logSale(int goodsType, int amount, int price, int tax) {
+        public void logSale(GoodsType goodsType, int amount, int price, int tax) {
             int totalBeforeTax = amount * price;
             int totalTax = totalBeforeTax * tax / 100;
             int totalAfterTax = totalBeforeTax - totalTax;
             
             String text = Messages.message("transaction.sale",
-                    "%goods%", Goods.getName(goodsType),
+                    "%goods%", goodsType.getName(),
                     "%amount%", String.valueOf(amount),
                     "%gold%", String.valueOf(price))
                 + "\n" + Messages.message("transaction.price",
