@@ -23,41 +23,60 @@ import java.awt.Color;
 import java.awt.FlowLayout;
 import java.awt.Font;
 import java.awt.Image;
+import java.awt.Insets;
 import java.awt.LayoutManager;
 import java.awt.event.KeyEvent;
 import java.awt.event.MouseAdapter;
+import java.text.DecimalFormat;
+import java.util.ArrayList;
+import java.util.EnumMap;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Set;
 import java.util.logging.Logger;
 
 import javax.swing.AbstractButton;
 import javax.swing.Action;
 import javax.swing.BorderFactory;
+import javax.swing.Icon;
 import javax.swing.InputMap;
 import javax.swing.JButton;
 import javax.swing.JComponent;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
 import javax.swing.JTextArea;
+import javax.swing.JTextPane;
 import javax.swing.KeyStroke;
 import javax.swing.UIManager;
 import javax.swing.border.Border;
+import javax.swing.text.Style;
+import javax.swing.text.StyleConstants;
+import javax.swing.text.StyleContext;
+import javax.swing.text.StyledDocument;
 
 import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.common.resources.ResourceManager;
+import net.sf.freecol.common.model.Modifier;
 import cz.autel.dmi.HIGConstraints;
 
 /**
  * Superclass for all panels in FreeCol.
  */
 public class FreeColPanel extends JPanel {
+
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(FreeColPanel.class.getName());
 
-
-
+    public static final Insets emptyMargin = new Insets(0,0,0,0);
 
     protected static final HIGConstraints higConst = new HIGConstraints();
 
     private static final int cancelKeyCode = KeyEvent.VK_ESCAPE;
 
+    // The decimal format to use for Modifiers
+    private static final DecimalFormat modifierFormat = new DecimalFormat("0.00");
+
+
     /**
      * The canvas all panels belong to.
      */
@@ -134,14 +153,14 @@ public class FreeColPanel extends JPanel {
         this.canvas = parent;
         setFocusCycleRoot(true);
 
-        Image menuborderN = (Image) UIManager.get("menuborder.n.image");
-        Image menuborderNW = (Image) UIManager.get("menuborder.nw.image");
-        Image menuborderNE = (Image) UIManager.get("menuborder.ne.image");
-        Image menuborderW = (Image) UIManager.get("menuborder.w.image");
-        Image menuborderE = (Image) UIManager.get("menuborder.e.image");
-        Image menuborderS = (Image) UIManager.get("menuborder.s.image");
-        Image menuborderSW = (Image) UIManager.get("menuborder.sw.image");
-        Image menuborderSE = (Image) UIManager.get("menuborder.se.image");
+        Image menuborderN = ResourceManager.getImage("menuborder.n.image");
+        Image menuborderNW = ResourceManager.getImage("menuborder.nw.image");
+        Image menuborderNE = ResourceManager.getImage("menuborder.ne.image");
+        Image menuborderW = ResourceManager.getImage("menuborder.w.image");
+        Image menuborderE = ResourceManager.getImage("menuborder.e.image");
+        Image menuborderS = ResourceManager.getImage("menuborder.s.image");
+        Image menuborderSW = ResourceManager.getImage("menuborder.sw.image");
+        Image menuborderSE = ResourceManager.getImage("menuborder.se.image");
         final FreeColImageBorder imageBorder = new FreeColImageBorder(menuborderN, menuborderW, menuborderS,
                 menuborderE, menuborderNW, menuborderNE, menuborderSW, menuborderSE);
         setBorder(BorderFactory.createCompoundBorder(imageBorder,
@@ -172,6 +191,30 @@ public class FreeColPanel extends JPanel {
         return editable;
     }
 
+    public static JTextPane getDefaultTextPane() {
+        JTextPane textPane = new JTextPane();
+        textPane.setOpaque(false);
+        textPane.setEditable(false);
+
+        StyledDocument doc = textPane.getStyledDocument();
+        //Initialize some styles.
+        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
+        
+        Style regular = doc.addStyle("regular", def);
+        StyleConstants.setFontFamily(def, "Dialog");
+        StyleConstants.setBold(def, true);
+        StyleConstants.setFontSize(def, 12);
+
+        Style buttonStyle = doc.addStyle("button", regular);
+        StyleConstants.setForeground(buttonStyle, LINK_COLOR);
+
+        Style right = doc.addStyle("right", regular);
+        StyleConstants.setAlignment(right, StyleConstants.ALIGN_RIGHT);
+
+        return textPane;
+    }
+
+
     /**
      * Returns a text area with standard settings suitable for use in FreeCol
      * dialogs.
@@ -193,6 +236,18 @@ public class FreeColPanel extends JPanel {
         return textArea;
     }
 
+    public static JButton getLinkButton(String text, Icon icon, String action) {
+        JButton button = new JButton(text, icon);
+        button.setMargin(emptyMargin);
+        button.setOpaque(false);
+        button.setForeground(LINK_COLOR);
+        button.setAlignmentY(0.8f);
+        button.setBorder(BorderFactory.createEmptyBorder());
+        button.setActionCommand(action);
+        return button;
+    }
+
+
     /**
      * Returns the default header for panels.
      * 
@@ -210,13 +265,11 @@ public class FreeColPanel extends JPanel {
         if (cancelButton == null) {
             throw new NullPointerException();
         }
-
-        Action cancelAction = cancelButton.getAction(); 
         
         InputMap inputMap = getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
-
         inputMap.put(KeyStroke.getKeyStroke(cancelKeyCode, 0, true), "release");
-        
+                
+        Action cancelAction = cancelButton.getAction();
         getActionMap().put("release", cancelAction);
     }
 
@@ -234,4 +287,24 @@ public class FreeColPanel extends JPanel {
                 KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0, true), JComponent.WHEN_FOCUSED);
     }
 
+    public static final DecimalFormat getModifierFormat() {
+        return modifierFormat;
+    }
+
+    public Set<Modifier> sortModifiers(Set<Modifier> result) {
+        EnumMap<Modifier.Type, List<Modifier>> modifierMap =
+            new EnumMap<Modifier.Type, List<Modifier>>(Modifier.Type.class);
+        for (Modifier.Type type : Modifier.Type.values()) {
+            modifierMap.put(type, new ArrayList<Modifier>());
+        }
+        for (Modifier modifier : result) {
+            modifierMap.get(modifier.getType()).add(modifier);
+        }
+        Set<Modifier> sortedResult = new LinkedHashSet<Modifier>();
+        for (Modifier.Type type : Modifier.Type.values()) {
+            sortedResult.addAll(modifierMap.get(type));
+        }
+        return sortedResult;
+    }
+
 }
