@@ -28,10 +28,11 @@ import java.awt.Image;
 import javax.swing.BorderFactory;
 import javax.swing.ImageIcon;
 import javax.swing.JPanel;
-import javax.swing.UIManager;
 
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
+import net.sf.freecol.common.model.GoodsType;
+import net.sf.freecol.common.resources.ResourceManager;
 
 /**
  * Implements a simple progress bar suitable for use with FreeCol. Unlike
@@ -66,10 +67,10 @@ public class FreeColProgressBar extends JPanel {
     private int iconHeight = 16;
 
     /**
-     * The type of goods this progress bar is for. The default value of -1
+     * The type of goods this progress bar is for. The default value of null
      * indicates no goods type.
      */
-    private int goodsType = -1;
+    private GoodsType goodsType = null;
 
     @SuppressWarnings("unused")
     private final Canvas parent;
@@ -83,7 +84,7 @@ public class FreeColProgressBar extends JPanel {
      * @param parent a <code>Canvas</code> value
      * @param goodsType the type of goods produced
      */
-    public FreeColProgressBar(Canvas parent, int goodsType) {
+    public FreeColProgressBar(Canvas parent, GoodsType goodsType) {
         this(parent, goodsType, 0, 100, 0, 0);
     }
 
@@ -95,7 +96,7 @@ public class FreeColProgressBar extends JPanel {
      * @param min the minimum value of the progress bar
      * @param max the maximum value of the progress bar
      */
-    public FreeColProgressBar(Canvas parent, int goodsType, int min, int max) {
+    public FreeColProgressBar(Canvas parent, GoodsType goodsType, int min, int max) {
         this(parent, goodsType, min, max, 0, 0);
     }
 
@@ -109,7 +110,7 @@ public class FreeColProgressBar extends JPanel {
      * @param value the current value of the progress bar
      * @param step the expected increase next turn
      */
-    public FreeColProgressBar(Canvas parent, int goodsType, int min, int max, int value, int step) {
+    public FreeColProgressBar(Canvas parent, GoodsType goodsType, int min, int max, int value, int step) {
         this.parent = parent;
         this.goodsType = goodsType;
         this.min = min;
@@ -162,7 +163,7 @@ public class FreeColProgressBar extends JPanel {
         }
 
         if (isOpaque()) {
-            Image tempImage = (Image) UIManager.get("BackgroundImage");
+            Image tempImage = ResourceManager.getImage("BackgroundImage");
 
             if (tempImage != null) {
                 for (int x = getInsets().left; x < width + getInsets().left; x += tempImage.getWidth(null)) {
@@ -204,21 +205,30 @@ public class FreeColProgressBar extends JPanel {
             }
         }
 
-        String progressString = String.valueOf(value) + "+" + step + "/" + max;
-        if (step > 0 && max > value) {
-            int turns = (max - value) / step;
-            if ((max - value) % step > 0) {
-                turns++;
+        String stepSignal = (step < 0) ? "-" : "+"; 
+        String progressString = String.valueOf(value) + stepSignal + Math.abs(step) + "/" + max;
+        String turnsString = "";
+        if (max > value) {
+            if (step > 0) {
+                // There is progress, find how many turns necessary with current production
+                int turns = (max - value) / step;
+                if ((max - value) % step > 0) {
+                    turns++;
+                }
+                turnsString = " (" + turns + " " + Messages.message("turns") + ")";
+            }
+            else {
+                // No progress, or even negative progress, impossible to determine
+                turnsString = " (" + Messages.message("turns") + ": N/A)";
             }
-            progressString += " (" + turns + " " + Messages.message("turns") + ")";
         }
-
+        progressString += turnsString;
+        
         int stringWidth = g2d.getFontMetrics().stringWidth(progressString);
         int stringHeight = g2d.getFontMetrics().getAscent() + g2d.getFontMetrics().getDescent();
         int restWidth = getWidth() - stringWidth;
 
-        if (goodsType >= 0) {
-
+        if (goodsType != null) {
             restWidth -= iconWidth;
             g2d.drawImage(image, restWidth / 2, (getHeight() - iconHeight) / 2, null);
         }
