@@ -51,11 +51,9 @@ import net.sf.freecol.server.generator.MapGeneratorOptions;
  * {@link net.sf.freecol.server.generator.MapGeneratorOptions}.
  */
 public final class MapGeneratorOptionsDialog extends FreeColDialog implements ActionListener {
+
     private static final Logger logger = Logger.getLogger(MapGeneratorOptionsDialog.class.getName());
 
-
-
-
     private static final int OK = 0, CANCEL = 1, RESET = 2;
 
     private final Canvas parent;
@@ -144,6 +142,7 @@ public final class MapGeneratorOptionsDialog extends FreeColDialog implements Ac
          *       The update should be solved by PropertyEvent.
          */
         JPanel shortcutsPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
+        shortcutsPanel.setOpaque(false);
         shortcutsPanel.add(new JLabel(Messages.message("shortcuts")));
         File mapDirectory = new File(FreeCol.getDataDirectory(), "maps");
         if (mapDirectory.isDirectory()) {
@@ -177,7 +176,6 @@ public final class MapGeneratorOptionsDialog extends FreeColDialog implements Ac
                             FileOptionUI fou = (FileOptionUI) theUI.getOptionUI(MapGeneratorOptions.IMPORT_FILE);
                             fou.setValue(file);
                     
-                            ((BooleanOptionUI) theUI.getOptionUI(MapGeneratorOptions.IMPORT_LAND_MAP)).setValue(true);
                             ((BooleanOptionUI) theUI.getOptionUI(MapGeneratorOptions.IMPORT_RUMOURS)).setValue(false);
                             ((BooleanOptionUI) theUI.getOptionUI(MapGeneratorOptions.IMPORT_TERRAIN)).setValue(true);
                             ((BooleanOptionUI) theUI.getOptionUI(MapGeneratorOptions.IMPORT_BONUSES)).setValue(false);
@@ -218,6 +216,7 @@ public final class MapGeneratorOptionsDialog extends FreeColDialog implements Ac
                 setResponse(new Boolean(true));
                 break;
             case CANCEL:
+                ui.rollback();
                 ui.unregister();
                 parent.remove(this);
                 setResponse(new Boolean(false));
