@@ -20,9 +20,9 @@
 package net.sf.freecol.client.gui.panel;
 
 import java.awt.Color;
-import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
@@ -51,6 +51,7 @@ import net.sf.freecol.common.model.GoldTradeItem;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsTradeItem;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.StanceTradeItem;
 import net.sf.freecol.common.model.TradeItem;
@@ -133,9 +134,9 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             this.otherPlayer = sender;
         }
 
-        if (player.getStance(otherPlayer) == Player.WAR) {
+        if (player.getStance(otherPlayer) == Stance.WAR) {
             if (!hasPeaceOffer()) {
-                int stance = Player.PEACE;
+                Stance stance = Stance.PEACE;
                 this.agreement.add(new StanceTradeItem(freeColClient.getGame(), player, otherPlayer, stance));
             }
         }
@@ -162,19 +163,7 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
      *
      */
     public void initialize() {
-
-        int foreignGold = 0;
-        Element report = getCanvas().getClient().getInGameController().getForeignAffairsReport();
-        int number = report.getChildNodes().getLength();
-        for (int i = 0; i < number; i++) {
-            Element enemyElement = (Element) report.getChildNodes().item(i);
-            int nationID = Integer.parseInt(enemyElement.getAttribute("nation"));
-            if (nationID == otherPlayer.getNation()) {
-                foreignGold = Integer.parseInt(enemyElement.getAttribute("gold"));
-                break;
-            }
-        }
-
+    	
         sendButton = new JButton(Messages.message("negotiationDialog.send"));
         sendButton.addActionListener(this);
         sendButton.setActionCommand(SEND);
@@ -192,11 +181,9 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
         setCancelComponent(cancelButton);
         FreeColPanel.enterPressesWhenFocused(cancelButton);
 
-        updateSummary();
-
         stance = new StanceTradeItemPanel(this, player, otherPlayer);
-        goldDemand = new GoldTradeItemPanel(this, otherPlayer, foreignGold);
-        goldOffer = new GoldTradeItemPanel(this, player, player.getGold());
+        goldDemand = new GoldTradeItemPanel(this, otherPlayer, 0);
+        goldOffer = new GoldTradeItemPanel(this, player, 0);
         colonyDemand = new ColonyTradeItemPanel(this, otherPlayer);
         colonyOffer = new ColonyTradeItemPanel(this, player);
         /** TODO: UnitTrade
@@ -231,9 +218,9 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
         add(summary, higConst.rcwh(row, summaryColumn, 1, 5));
         row += 2;
         if (unit.isCarrier()) {
-            goodsDemand = new GoodsTradeItemPanel(this, otherPlayer, settlement.getGoodsContainer().getGoods());
+            goodsDemand = new GoodsTradeItemPanel(this, otherPlayer, null);
             add(goodsDemand, higConst.rc(row, demandColumn));
-            goodsOffer = new GoodsTradeItemPanel(this, player, unit.getGoodsContainer().getGoods());
+            goodsOffer = new GoodsTradeItemPanel(this, player, null);
             add(goodsOffer, higConst.rc(row, offerColumn));
         } else {
             add(colonyDemand, higConst.rc(row, demandColumn));
@@ -249,9 +236,9 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
         add(acceptButton, higConst.rc(row, summaryColumn, ""));
         add(cancelButton, higConst.rc(row, offerColumn, ""));
             
+        updateDialog();
     }
 
-
     private void updateSummary() {
         try {
             StyledDocument document = summary.getStyledDocument();
@@ -299,6 +286,89 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             logger.warning("Failed to update summary: " + e.toString());
         }
     }
+    
+    private void updateOfferItems(){
+    	// Update stance options
+    	stance.updateStanceBox();
+    	
+    	// Update the gold amount that can be demanded
+    	goldOffer.setAvailableGold(player.getGold());
+    	
+    	if(unit.isCarrier()){
+        	Iterator<Goods> goodsInAgreement = agreement.getGoodsGivenBy(player).iterator();
+        	List<Goods> goodsAvail = new ArrayList<Goods>();
+        	goodsAvail.addAll(unit.getGoodsContainer().getGoods());
+        	
+        	//remove the ones already on the table
+        	while(goodsInAgreement.hasNext()){
+        		Goods goods = goodsInAgreement.next();
+        		for(int i=0;i<goodsAvail.size();i++){
+        			Goods goodAvail = goodsAvail.get(i);
+        			if(goodAvail.getType() == goods.getType() &&
+        			    goodAvail.getAmount() == goods.getAmount()){
+        					// this good is already on the agreement, remove it
+        					goodsAvail.remove(i);
+        					break;
+        			}
+        		}
+        	}
+        	
+    		// Update the list of goods available to add to agreement
+    		goodsOffer.updateGoodsBox(goodsAvail);
+    	} else{
+        	// Update the list of colonies available to add to agreement
+        	colonyOffer.updateColonyBox();
+    	}
+    }
+    
+    private void updateDemandItems(){
+    	// Update the gold amount that can be demanded
+    	int foreignGold = 0;
+        Element report = getCanvas().getClient().getInGameController().getForeignAffairsReport();
+        int number = report.getChildNodes().getLength();
+        for (int i = 0; i < number; i++) {
+            Element enemyElement = (Element) report.getChildNodes().item(i);
+            Player enemy = (Player) getCanvas().getClient().getGame().getFreeColGameObject(enemyElement.getAttribute("player"));
+            if (enemy == otherPlayer) {
+                foreignGold = Integer.parseInt(enemyElement.getAttribute("gold"));
+                break;
+            }
+        }
+        goldDemand.setAvailableGold(foreignGold);
+        
+        if(unit.isCarrier()){
+        	Iterator<Goods> goodsInAgreement = agreement.getGoodsGivenBy(otherPlayer).iterator();
+        	List<Goods> goodsAvail = new ArrayList<Goods>();
+        	goodsAvail.addAll(settlement.getGoodsContainer().getGoods());
+        	
+        	//remove the ones already on the table
+        	while(goodsInAgreement.hasNext()){
+        		Goods goods = goodsInAgreement.next();
+        		for(int i=0;i<goodsAvail.size();i++){
+        			Goods goodAvail = goodsAvail.get(i);
+        			if(goodAvail.getType() == goods.getType() &&
+        			    goodAvail.getAmount() == goods.getAmount()){
+        					// this good is already on the agreement, remove it
+        					goodsAvail.remove(i);
+        					break;
+        			}
+        		}
+        	}
+        	
+        	// Update the list of goods available to add to agreement
+        	goodsDemand.updateGoodsBox(goodsAvail);
+        }
+        else{
+        	// Update the list of colonies available to add to agreement
+        	colonyDemand.updateColonyBox();
+        }
+    }
+    
+    public void updateDialog(){
+    	updateOfferItems();
+    	updateDemandItems();
+    	updateSummary();
+    }
 
     private void insertText(String text) throws Exception {
         StyledDocument document = summary.getStyledDocument();
@@ -338,14 +408,8 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
                     description = ((UnitTradeItem) item).getUnit().getName();
                 }
                 try {
-                    JButton button = new JButton(description);
-                    button.setMargin(new Insets(0,0,0,0));
-                    button.setOpaque(false);
-                    button.setForeground(LINK_COLOR);
-                    button.setAlignmentY(0.8f);
-                    button.setBorder(BorderFactory.createEmptyBorder());
+                    JButton button = getLinkButton(description, null, String.valueOf(index));
                     button.addActionListener(this);
-                    button.setActionCommand(String.valueOf(index));
                     StyleConstants.setComponent(document.getStyle("button"), button);
                     document.insertString(document.getLength(), " ", document.getStyle("button"));
                     if (index < items.size() - 1) {
@@ -372,7 +436,7 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
 
 
     private boolean hasPeaceOffer() {
-        return (getStance() > Integer.MIN_VALUE);
+        return (getStance() != null);
     }
 
 
@@ -429,9 +493,9 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
     /**
      * Sets the <code>stance</code> between the players.
      *
-     * @param stance an <code>int</code> value
+     * @param stance a <code>Stance</code> value
      */
-    public void setStance(int stance) {
+    public void setStance(Stance stance) {
         agreement.add(new StanceTradeItem(freeColClient.getGame(), otherPlayer, player, stance));
     }
 
@@ -440,9 +504,9 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
      * Returns the stance being offered, or Integer.MIN_VALUE if none
      * is being offered.
      *
-     * @return an <code>int</code> value
+     * @return a <code>Stance</code> value
      */
-    public int getStance() {
+    public Stance getStance() {
         return agreement.getStance();
     }
 
@@ -465,18 +529,18 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
         } else {
             int index = Integer.parseInt(command);
             agreement.remove(index);
-            initialize();
+            updateDialog();
         }
     }
 
-
     public class ColonyTradeItemPanel extends JPanel implements ActionListener {
 
         private JComboBox colonyBox;
         private JButton addButton;
         private Player player;
         private NegotiationDialog negotiationDialog;
-
+        private JLabel textLabel;
+        
         /**
          * Creates a new <code>ColonyTradeItemPanel</code> instance.
          *
@@ -495,17 +559,17 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             setLayout(new HIGLayout(new int[] {0}, new int[] {0, 0, 0}));
             setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK),
                                                          BorderFactory.createEmptyBorder(5, 5, 5, 5)));
-            add(new JLabel(Messages.message("tradeItem.colony")),
-                higConst.rc(1, 1));
-            add(colonyBox, higConst.rc(2, 1));
-            add(addButton, higConst.rc(3, 1));
+            this.textLabel = new JLabel(Messages.message("tradeItem.colony"));
+            add(this.textLabel, higConst.rc(1, 1));
+            add(colonyBox,  higConst.rc(2, 1));
+            add(addButton,  higConst.rc(3, 1));
             
         }
-
+        
         private void updateColonyBox() {
 
             if (!player.isEuropean()) {
-                return;
+        		return;
             }
 
             // Remove all action listeners, so the update has no effect (except
@@ -515,15 +579,40 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
                 colonyBox.removeActionListener(al);
             }
             colonyBox.removeAllItems();
-            List<Colony> colonies = player.getColonies();
-            Collections.sort(colonies, freeColClient.getClientOptions().getColonyComparator());
-            Iterator<Colony> colonyIterator = colonies.iterator();
-            while (colonyIterator.hasNext()) {
-                colonyBox.addItem(colonyIterator.next());
-            }
-            for(ActionListener al : listeners) {
-                colonyBox.addActionListener(al);
-            }
+        	
+            Iterator<Colony> coloniesInAgreement = agreement.getColoniesGivenBy(player).iterator();
+        	List<Colony> coloniesAvail = new ArrayList<Colony>();
+        	coloniesAvail.addAll(player.getColonies());
+        	
+        	//remove the ones already on the table
+        	while(coloniesInAgreement.hasNext()){
+        		Colony colony = coloniesInAgreement.next();
+        		for(int i=0;i<coloniesAvail.size();i++){
+        			Colony colonyAvail = coloniesAvail.get(i);
+        			if(colonyAvail == colony){
+        					// this good is already on the agreement, remove it
+        					coloniesAvail.remove(i);
+        					break;
+        			}
+        		}
+        	}
+            
+        	if(coloniesAvail.isEmpty()){
+        		addButton.setEnabled(false);
+        		colonyBox.setEnabled(false);
+        	}
+        	else{
+        		Collections.sort(coloniesAvail, freeColClient.getClientOptions().getColonyComparator());
+        		Iterator<Colony> colonyIterator = coloniesAvail.iterator();
+        		while (colonyIterator.hasNext()) {
+        			colonyBox.addItem(colonyIterator.next());
+        		}
+        		for(ActionListener al : listeners) {
+        			colonyBox.addActionListener(al);
+        		}
+        		addButton.setEnabled(true);
+        		colonyBox.setEnabled(true);
+        	}
         }
 
         /**
@@ -536,7 +625,7 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             String command = event.getActionCommand();
             if (command.equals("add")) {
                 negotiationDialog.addColonyTradeItem(player, (Colony) colonyBox.getSelectedItem());
-                updateSummary();
+                updateDialog();
             }
 
         }
@@ -548,7 +637,8 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
         private JButton addButton;
         private Player player;
         private NegotiationDialog negotiationDialog;
-
+        private JLabel label;
+        
         /**
          * Creates a new <code>GoodsTradeItemPanel</code> instance.
          *
@@ -563,14 +653,9 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             addButton.addActionListener(this);
             addButton.setActionCommand("add");
             goodsBox = new JComboBox();
-            JLabel label = new JLabel(Messages.message("tradeItem.goods"));
-            if (allGoods == null) {
-                label.setEnabled(false);
-                addButton.setEnabled(false);
-                goodsBox.setEnabled(false);
-            } else {
-                updateGoodsBox(allGoods);
-            }
+            this.label = new JLabel(Messages.message("tradeItem.goods"));
+
+            updateGoodsBox(allGoods);
 
             setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK),
                                                          BorderFactory.createEmptyBorder(5, 5, 5, 5)));
@@ -579,7 +664,6 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             add(goodsBox, higConst.rc(2, 1));
             add(addButton, higConst.rc(3, 1));
             setSize(getPreferredSize());
-            
         }
 
         private void updateGoodsBox(List<Goods> allGoods) {
@@ -591,13 +675,27 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
                 goodsBox.removeActionListener(al);
             }
             goodsBox.removeAllItems();
-            Iterator<Goods> goodsIterator = allGoods.iterator();
-            while (goodsIterator.hasNext()) {
-                goodsBox.addItem(goodsIterator.next());
-            }
-            for(ActionListener al : listeners) {
-                goodsBox.addActionListener(al);
-            }
+            
+            if(allGoods != null && !allGoods.isEmpty()){
+            	Iterator<Goods> goodsIterator = allGoods.iterator();
+            	while (goodsIterator.hasNext()) {
+            		Goods goods = goodsIterator.next();
+            		if(goods.getType().isStorable()){
+            			goodsBox.addItem(goods);
+            		}
+            	}
+            	for(ActionListener al : listeners) {
+                    goodsBox.addActionListener(al);
+                }
+            	
+            	this.label.setEnabled(true);
+                addButton.setEnabled(true);
+                goodsBox.setEnabled(true);
+            } else{
+            	this.label.setEnabled(false);
+                addButton.setEnabled(false);
+                goodsBox.setEnabled(false);
+            }            
         }
 
         /**
@@ -610,7 +708,7 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             String command = event.getActionCommand();
             if (command.equals("add")) {
                 negotiationDialog.addGoodsTradeItem(player, (Goods) goodsBox.getSelectedItem());
-                updateSummary();
+                updateDialog();
             }
 
         }
@@ -619,8 +717,8 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
     public class StanceTradeItemPanel extends JPanel implements ActionListener {
 
         class StanceItem {
-            private int value;
-            StanceItem(int value) {
+            private Stance value;
+            StanceItem(Stance value) {
                 this.value = value;
             }
             
@@ -628,7 +726,7 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
                 return Player.getStanceAsString(value);
             }
             
-            int getValue() {
+            Stance getValue() {
                 return value;
             }
             
@@ -636,14 +734,16 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
                 if (other == null || !(other instanceof StanceItem)) {
                     return false;
                 }
-                return value == ((StanceItem) other).value;
+                return value.equals(((StanceItem) other).value);
             }
         }
         
         private JComboBox stanceBox;
         private JButton addButton;
         private NegotiationDialog negotiationDialog;
-
+        private Player source;
+        private Player target;
+        
         /**
          * Creates a new <code>StanceTradeItemPanel</code> instance.
          *
@@ -652,19 +752,15 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
          */
         public StanceTradeItemPanel(NegotiationDialog parent, Player source, Player target) {
             this.negotiationDialog = parent;
+            this.source = source;
+            this.target = target;
+            
             addButton = new JButton(Messages.message("negotiationDialog.add"));
             addButton.addActionListener(this);
             addButton.setActionCommand("add");
             stanceBox = new JComboBox();
             
-            int stance = source.getStance(target);
-            if (stance != Player.WAR) stanceBox.addItem(new StanceItem(Player.WAR));
-            if (stance == Player.WAR) stanceBox.addItem(new StanceItem(Player.CEASE_FIRE));
-            if (stance != Player.PEACE) stanceBox.addItem(new StanceItem(Player.PEACE));
-            if (stance != Player.ALLIANCE) stanceBox.addItem(new StanceItem(Player.ALLIANCE));
-            if (parent.hasPeaceOffer()) {
-                stanceBox.setSelectedItem(new StanceItem(parent.getStance()));
-            }
+            updateStanceBox();
 
             setBorder(BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK),
                                                          BorderFactory.createEmptyBorder(5, 5, 5, 5)));
@@ -673,7 +769,6 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
                 higConst.rc(1, 1));
             add(stanceBox, higConst.rc(2, 1));
             add(addButton, higConst.rc(3, 1));
-            
         }
 
         /**
@@ -691,6 +786,18 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             }
 
         }
+        
+        public void updateStanceBox(){
+        	stanceBox.removeAllItems();
+        	Stance stance = source.getStance(target);
+            if (stance != Stance.WAR) stanceBox.addItem(new StanceItem(Stance.WAR));
+            if (stance == Stance.WAR) stanceBox.addItem(new StanceItem(Stance.CEASE_FIRE));
+            if (stance != Stance.PEACE) stanceBox.addItem(new StanceItem(Stance.PEACE));
+            if (stance != Stance.ALLIANCE) stanceBox.addItem(new StanceItem(Stance.ALLIANCE));
+            if (negotiationDialog.hasPeaceOffer()) {
+                stanceBox.setSelectedItem(new StanceItem(negotiationDialog.getStance()));
+            }
+        }
     }
 
     public class GoldTradeItemPanel extends JPanel implements ActionListener {
@@ -734,10 +841,15 @@ public final class NegotiationDialog extends FreeColDialog implements ActionList
             if (command.equals("add")) {
                 int amount = ((Integer) spinner.getValue()).intValue();
                 negotiationDialog.addGoldTradeItem(player, amount);
-                updateSummary();
+                updateDialog();
             }
 
         }
+        
+        public void setAvailableGold(int gold) {
+        	SpinnerNumberModel model = (SpinnerNumberModel) spinner.getModel();
+        	model.setMaximum(new Integer(gold));
+        }
     }
 
 }
