@@ -26,6 +26,7 @@ import javax.swing.BorderFactory;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.FoundingFather;
@@ -36,7 +37,7 @@ import cz.autel.dmi.HIGLayout;
 /**
  * This panel displays the ContinentalCongress Report.
  */
-public final class ReportContinentalCongressPanel extends ReportPanel implements ActionListener {
+public final class ReportContinentalCongressPanel extends ReportPanel {
 
 
 
@@ -84,12 +85,12 @@ public final class ReportContinentalCongressPanel extends ReportPanel implements
 
         // summary
         summaryPanel.add(new JLabel(Messages.message("report.continentalCongress.recruiting")), higConst.rc(1, 1));
-        if (player.getCurrentFather() == FoundingFather.NONE) {
+        if (player.getCurrentFather() == null) {
             summaryPanel.add(new JLabel(none), higConst.rc(1, 3));
         } else {
-            JLabel currentFatherLabel = new JLabel(Messages.message(FoundingFather.getName(player.getCurrentFather())));
-            currentFatherLabel.setToolTipText(Messages
-                    .message(FoundingFather.getDescription(player.getCurrentFather())));
+            FoundingFather father = player.getCurrentFather();
+            JLabel currentFatherLabel = new JLabel(father.getName());
+            currentFatherLabel.setToolTipText(father.getDescription());
             summaryPanel.add(currentFatherLabel, higConst.rc(1, 3));
             int bells = player.getBells();
             int required = player.getTotalFoundingFatherCost();
@@ -106,10 +107,10 @@ public final class ReportContinentalCongressPanel extends ReportPanel implements
             fatherLabel.setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 10));
             fatherPanel.add(fatherLabel);
         } else {
-            for (int fatherId = 0; fatherId < FoundingFather.FATHER_COUNT; fatherId++) {
-                if (player.hasFather(fatherId)) {
-                    JLabel fatherLabel = new JLabel(Messages.message(FoundingFather.getName(fatherId)));
-                    fatherLabel.setToolTipText(Messages.message(FoundingFather.getDescription(fatherId)));
+            for (FoundingFather father : FreeCol.getSpecification().getFoundingFathers()) {
+                if (player.hasFather(father)) {
+                    JLabel fatherLabel = new JLabel(Messages.message(father.getName()));
+                    fatherLabel.setToolTipText(Messages.message(father.getDescription()));
                     fatherLabel.setBorder(BorderFactory.createEmptyBorder(5, 10, 5, 10));
                     fatherPanel.add(fatherLabel);
                 }
