@@ -20,29 +20,27 @@
 package net.sf.freecol.client.gui.panel;
 
 import java.awt.GridLayout;
-import java.awt.Image;
 import java.awt.event.ActionListener;
 
 import javax.swing.ImageIcon;
 import javax.swing.JLabel;
 import javax.swing.JPanel;
-import javax.swing.UIManager;
 
+import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
+import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Player.Stance;
 
 import org.w3c.dom.Element;
 
-import cz.autel.dmi.HIGConstraints;
 import cz.autel.dmi.HIGLayout;
 
 /**
  * This panel displays the Foreign Affairs Report.
  */
-public final class ReportForeignAffairPanel extends ReportPanel implements ActionListener {
-
-
+public final class ReportForeignAffairPanel extends ReportPanel {
 
 
     /**
@@ -68,24 +66,26 @@ public final class ReportForeignAffairPanel extends ReportPanel implements Actio
         int labelColumn = 3;
         int valueColumn = 5;
         int percentColumn = 6;
-        HIGConstraints higConst = new HIGConstraints();
 
-        Element report = getCanvas().getClient().getInGameController().getForeignAffairsReport();
+        FreeColClient client = getCanvas().getClient();
+        ImageLibrary imageLibrary = getCanvas().getGUI().getImageLibrary();
+        Element report = client.getInGameController().getForeignAffairsReport();
         int number = report.getChildNodes().getLength();
         for (int i = 0; i < number; i++) {
             Element enemyElement = (Element) report.getChildNodes().item(i);
             JPanel enemyPanel = new JPanel(new HIGLayout(widths, heights));
             enemyPanel.setOpaque(false);
             int row = 1;
-            int nationID = Integer.parseInt(enemyElement.getAttribute("nation"));
-            final String currentNation = Player.getNationIdentifier(nationID);
-            final Image coatOfArms = (Image) UIManager.get("coatOfArms." + currentNation + ".image");
-            enemyPanel.add(new JLabel(new ImageIcon(coatOfArms)),
-                           higConst.rcwh(row, coatColumn, 1, heights.length, "t"));
-            enemyPanel.add(new JLabel(Player.getNationAsString(nationID)), higConst.rc(row, labelColumn));
+            Player enemy = (Player) client.getGame().getFreeColGameObject(enemyElement.getAttribute("player"));
+            final ImageIcon coatOfArms = imageLibrary.getCoatOfArmsImageIcon(enemy.getNation());
+            if (coatOfArms != null) {
+                enemyPanel.add(new JLabel(coatOfArms),
+                               higConst.rcwh(row, coatColumn, 1, heights.length, "t"));
+            }
+            enemyPanel.add(new JLabel(enemy.getNationAsString()), higConst.rc(row, labelColumn));
             row += 2;
             enemyPanel.add(new JLabel(Messages.message("report.stance")), higConst.rc(row, labelColumn));
-            int stance = Integer.parseInt(enemyElement.getAttribute("stance"));
+            Stance stance = Enum.valueOf(Stance.class, enemyElement.getAttribute("stance"));
             enemyPanel.add(new JLabel(Player.getStanceAsString(stance)),
                            higConst.rc(row, valueColumn, "r"));
             row++;
