@@ -20,7 +20,6 @@
 package net.sf.freecol.client.gui.panel;
 
 import cz.autel.dmi.HIGLayout;
-import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.ImageLibrary;
 import net.sf.freecol.client.gui.i18n.Messages;
@@ -43,24 +42,14 @@ import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.Collections;
+import java.util.LinkedHashSet;
 import java.util.List;
+import java.util.Set;
 
 /**
  * This panel displays the Labour Report.
  */
-public final class ReportLabourPanel extends ReportPanel implements ActionListener {
-
-    public static final int[] UNIT_TYPES = new int[]{
-        Unit.FREE_COLONIST, Unit.INDENTURED_SERVANT, Unit.PETTY_CRIMINAL,
-        Unit.INDIAN_CONVERT, Unit.EXPERT_FARMER, Unit.EXPERT_FISHERMAN,
-        Unit.MASTER_SUGAR_PLANTER, Unit.MASTER_DISTILLER, Unit.EXPERT_LUMBER_JACK,
-        Unit.MASTER_CARPENTER, Unit.MASTER_TOBACCO_PLANTER, Unit.MASTER_TOBACCONIST,
-        Unit.EXPERT_FUR_TRAPPER, Unit.MASTER_FUR_TRADER, Unit.MASTER_COTTON_PLANTER,
-        Unit.MASTER_WEAVER, Unit.EXPERT_ORE_MINER, Unit.MASTER_BLACKSMITH,
-        Unit.MASTER_GUNSMITH, Unit.EXPERT_SILVER_MINER, Unit.HARDY_PIONEER,
-        Unit.VETERAN_SOLDIER, Unit.SEASONED_SCOUT, Unit.JESUIT_MISSIONARY,
-        Unit.ELDER_STATESMAN, Unit.FIREBRAND_PREACHER
-    };
+public final class ReportLabourPanel extends ReportPanel {
 
     private static final int HEADER_ROWS = 1;
 
@@ -137,10 +126,14 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
     }
 
     private ImageIcon getUnitIcon(UnitType unit) {
-        int unitIndex = unit.getIndex();
-        int imageType = ImageLibrary.getUnitGraphicsType(unitIndex, false, false,
-                                                         unitIndex == Unit.HARDY_PIONEER ? 20 : 0, false);
-        return getLibrary().getUnitImageIcon(imageType);
+        Unit.Role role = Unit.Role.DEFAULT;
+        if (unit.hasAbility("model.ability.expertPioneer")) {
+            role = Unit.Role.PIONEER;
+        } else if (unit.hasAbility("model.ability.expertMissionary")) {
+            role = Unit.Role.MISSIONARY;
+        }
+
+        return getLibrary().getUnitImageIcon(unit, role);
     }
 
     @Override
@@ -179,9 +172,7 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
         if (isOverview()) {
             int rows = labourData.getSummary().getTotal().getRowCount();
 
-            for (int unitIndex : UNIT_TYPES) {
-                UnitType unitType = FreeCol.getSpecification().unitType(unitIndex);
-
+            for (UnitType unitType : LabourData.getLabourTypes(player)) {
                 rows += labourData.getUnitData(unitType).getUnitSummaryRowCount();
             }
 
@@ -218,8 +209,7 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
         int minHeight = allColonistsButton.getPreferredSize().height;
         HIGLayout higLayout = (HIGLayout) reportPanel.getLayout();
 
-        for (int unitIndex : UNIT_TYPES) {
-            UnitType unitType = FreeCol.getSpecification().unitType(unitIndex);
+        for (UnitType unitType : LabourData.getLabourTypes(player)) {
             LabourData.UnitData unitData = labourData.getUnitData(unitType);
 
             JButton unitButton = createUnitNameButton(unitData.getUnitName(), unitData);
@@ -261,7 +251,7 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
      *
      * @param value
      * @param pocketCount
-     * @return
+     * @return distribution of {@code value}
      */
     private int[] distribute(int value, int pocketCount) {
         int[] pockets = new int[pocketCount];
@@ -326,8 +316,6 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
 
     /**
      * adds the header rows
-     *
-     * @return next row index
      */
     private void addHeader() {
         int row = 1;
@@ -365,15 +353,13 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
             GoodsType goods = unit.getExpertProduction();
 
             ImageLibrary imageLibrary = getCanvas().getGUI().getImageLibrary();
-            JLabel production = new JLabel(imageLibrary.getGoodsImageIcon(goods.index));
+            JLabel production = new JLabel(imageLibrary.getGoodsImageIcon(goods));
             production.setBorder(TOPCELLBORDER);
 
-            int storedAs = GoodsType.getStoredAs(goods.index);
+            headerRow.add(production, higConst.rcwh(row, PRODUCTION_SYMBOL_COLUMN, !goods.isStoredAs() ? 4 : 3, 1));
 
-            headerRow.add(production, higConst.rcwh(row, PRODUCTION_SYMBOL_COLUMN, goods.index == storedAs ? 4 : 3, 1));
-
-            if (showNetProduction && goods.index != storedAs) {
-                JLabel netProduction = new JLabel(imageLibrary.getGoodsImageIcon(storedAs));
+            if (showNetProduction && goods.isStoredAs()) {
+                JLabel netProduction = new JLabel(imageLibrary.getGoodsImageIcon(goods.getStoredAs()));
                 netProduction.setBorder(TOPCELLBORDER);
                 headerRow.add(netProduction, higConst.rc(row, NETPRODUCTION_SUMMARY_COLUMN));
             }
@@ -416,9 +402,12 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
         Building productionBuilding = null;
 
         if (!allColonists) {
-            workingAs = Messages.message(unitType.getName() + ".workingAs");
+            workingAs = Messages.message(unitType.getId() + ".workingAs");
             if (colony != null) {
-                productionBuilding = LabourData.getBuilding(colony, unitType);
+                GoodsType expert = unitType.getExpertProduction();
+                if (expert != null) {
+                    productionBuilding = colony.getBuildingForProducing(expert);
+                }
             }
         }
         addLocationSummary(data, row);
@@ -426,16 +415,6 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
         int buildingStartRow = row;
 
         //details
-        int professionals = data.getWorkingProfessionals().getColonists();
-        if (professionals > 0 || data.alwaysShowProfessionals()) {
-            addRow(data,
-                   unitName,
-                   allColonists ? Messages.message("report.labour.expertsWorking") : workingAs,
-                   createNumberLabel(professionals, null),
-                   data.getWorkingProfessionals().getProduction(),
-                   row);
-            row++;
-        }
 
         int otherAmateurs = data.getOtherWorkingAmateurs().getColonists();
         if (!allColonists && otherAmateurs > 0) {
@@ -448,6 +427,13 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
             row++;
         }
 
+        row = addRow(data,
+                     unitName,
+                     allColonists ? Messages.message("report.labour.expertsWorking") : workingAs,
+                     data.getWorkingProfessionals().getColonists(),
+                     data.getWorkingProfessionals().getProduction(),
+                     row);
+
         int notProducingStartRow = row;
 
         if (showBuildings && productionBuilding != null && row > buildingStartRow) {
@@ -463,20 +449,22 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
                          "report.labour.amateursWorking" :
                          "report.labour.workingAsOther"),
                      data.getWorkingAmateurs(),
-                     -1, row);
+                     0, row);
         if (data.getNotWorking() > 0) {
             addRow(data,
                    unitName,
                    Messages.message("report.labour.notWorking"),
                    createNumberLabel(data.getNotWorking(), "report.labour.notWorking.tooltip"),
-                   -1, row);
+                   0, row);
             row++;
         }
 
-        Building schoolhouse = colony != null && data.isTraining() ? colony.getBuilding(Building.SCHOOLHOUSE) : null;
+        Building schoolhouse = colony != null && data.isTraining() ?
+            colony.getBuildingWithAbility("model.ability.teach") : null;
 
         if (showBuildings && schoolhouse != null && row > buildingStartRow) {
-            reportPanel.add(createEmptyLabel(), higConst.rcwh(buildingStartRow, BUILDING_COLUMN, 1, row - buildingStartRow));
+            reportPanel.add(createEmptyLabel(),
+                            higConst.rcwh(buildingStartRow, BUILDING_COLUMN, 1, row - buildingStartRow));
             buildingStartRow = row;
         }
 
@@ -484,7 +472,7 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
                      unitName,
                      Messages.message("report.labour.teacher"),
                      data.getTeachers(),
-                     -1, row);
+                     0, row);
         if (!allColonists) {
             row = addRow(data,
                          data.getOtherStudentsName(),
@@ -492,24 +480,34 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
                                           "%unit%",
                                           data.getUnitData().getUnitName()),
                          data.getOtherStudents(),
-                         -1, row);
+                         0, row);
         }
 
-        int students = data.getStudents();
-        if (students > 0) {
-            if (allColonists || data.getUnitData().isSummary()) {
-                addRow(data, null, Messages.message("report.labour.sutdent"), createNonCountedLabel(students), -1, row);
+        int studentCount = data.getStudents();
+        if (studentCount > 0) {
+            if (allColonists) {
+                addRow(data, null, Messages.message("report.labour.sutdent"), createNonCountedLabel(studentCount), 0, row);
             } else {
-                String student = unitType.getSkill() < 0 ?
+                Set<UnitType> resultOfTraining = new LinkedHashSet<UnitType>();
+                if (colony != null) {
+                    for (Unit teacher : colony.getTeachers()) {
+                        Unit student = teacher.getStudent();
+                        if (student != null && student.getType() == unitType) {
+                            resultOfTraining.add(Unit.getUnitTypeTeaching(teacher.getType(), student.getType()));
+                        }
+                    }
+                }
+
+                String student = resultOfTraining.size() == 1 ?
                     Messages.message("report.labour.learning",
                                      "%unit%",
-                                     Unit.getName(Unit.getTrainingResult(unitType.getIndex(), 0))) :
+                                     resultOfTraining.iterator().next().getName()) :
                     Messages.message("report.labour.learningOther");
                 addRow(data,
                        data.getUnitData().getUnitName(),
                        student,
-                       createNumberLabel(-students, "report.labour.subtracted.tooltip"),
-                       -1, row);
+                       createNumberLabel(-studentCount, "report.labour.subtracted.tooltip"),
+                       0, row);
             }
             row++;
         }
@@ -573,11 +571,10 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
         if (showProductionSymbols) {
             JLabel icon = new JLabel();
             icon.setBorder(CELLBORDER);
-            icon.setHorizontalAlignment(SwingConstants.CENTER);
             GoodsType goods = data.getUnitData().getExpertProduction();
             if (goods != null) {
                 ImageLibrary imageLibrary = getCanvas().getGUI().getImageLibrary();
-                icon.setIcon(imageLibrary.getGoodsImageIcon(goods.index));
+                icon.setIcon(imageLibrary.getGoodsImageIcon(goods));
             }
             reportPanel.add(icon, higConst.rcwh(row, PRODUCTION_SYMBOL_COLUMN, 1, rows));
         }
@@ -620,7 +617,7 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
 
         reportPanel.add(colonistLabel, higConst.rc(row, COLONIST_COLUMN));
 
-        if (data.getUnitData().showProduction() && production >= 0) {
+        if (data.getUnitData().showProduction() && production > 0) {
             reportPanel.add(createNumberLabel(production, "report.labour.potentialProduction.tooltip"),
                             higConst.rc(row, PRODUCTION_COLUMN));
         }
@@ -629,15 +626,15 @@ public final class ReportLabourPanel extends ReportPanel implements ActionListen
     private JButton createColonyButton(final Colony colony) {
         String text = colony.getName();
         if (!unitData.isSummary()) {
-            int unitIndex = unitData.getUnitType().getIndex();
+//            int unitIndex = unitData.getUnitType().getIndex();
+//
+//            int skillLevel = Unit.getSkillLevel(unitIndex);
+//            if (skillLevel <= 0 && skillLevel > -2) {
+//                //settlers and servants can be trained anywwhere a farmer can
+//                unitIndex = Unit.EXPERT_FARMER;
+//            }
 
-            int skillLevel = Unit.getSkillLevel(unitIndex);
-            if (skillLevel <= 0 && skillLevel > -2) {
-                //settlers and servants can be trained anywwhere a farmer can
-                unitIndex = Unit.EXPERT_FARMER;
-            }
-
-            if (colony.canTrain(unitIndex)) {
+            if (colony.canTrain(unitData.getUnitType())) {
                 text = text + "*";
             }
         }
