@@ -44,6 +44,8 @@ import net.sf.freecol.common.model.Unit;
  */
 public class ReportPanel extends FreeColPanel implements ActionListener {
 
+
+
     protected static final Logger logger = Logger.getLogger(ReportPanel.class.getName());
 
     protected static final int OK = -1;
@@ -52,7 +54,7 @@ public class ReportPanel extends FreeColPanel implements ActionListener {
 
     protected JLabel header;
 
-    private JButton ok;
+    protected JButton ok;
 
     protected JScrollPane scrollPane;
 
@@ -60,7 +62,12 @@ public class ReportPanel extends FreeColPanel implements ActionListener {
 
     private static final Comparator<Unit> unitTypeComparator = new Comparator<Unit>() {
         public int compare(Unit unit1, Unit unit2) {
-            return unit2.getType() - unit1.getType();
+            int deltaType = unit2.getType().getIndex() - unit1.getType().getIndex();
+            if (deltaType == 0) {
+                return unit2.getRole().ordinal() - unit1.getRole().ordinal();
+            } else {
+                return deltaType;
+            }
         }
     };
 
@@ -74,7 +81,7 @@ public class ReportPanel extends FreeColPanel implements ActionListener {
     public ReportPanel(Canvas parent, String title) {
         super(parent, new FlowLayout(FlowLayout.CENTER, 1000, 10));
 
-        ReportPanel.library = (ImageLibrary) getCanvas().getImageProvider();
+        ReportPanel.library = getCanvas().getGUI().getImageLibrary();
 
         setLayout(new BorderLayout());
 
@@ -87,6 +94,7 @@ public class ReportPanel extends FreeColPanel implements ActionListener {
 
         scrollPane = new JScrollPane(reportPanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
+        scrollPane.getVerticalScrollBar().setUnitIncrement( 16 );
         add(scrollPane, BorderLayout.CENTER);
 
         ok = new JButton(Messages.message("ok"));
@@ -98,7 +106,7 @@ public class ReportPanel extends FreeColPanel implements ActionListener {
 
         setSize(850, 600);
     }
-
+    
     protected Border createBorder() {
         return new EmptyBorder(20, 20, 20, 20);
     }
@@ -146,16 +154,6 @@ public class ReportPanel extends FreeColPanel implements ActionListener {
         return unitTypeComparator;
     }
 
-    /**
-     * Builds the button for the given unit.
-     * 
-     * @param unitIcon
-     * @param scale
-     */
-    public JLabel buildUnitLabel(int unitIcon, float scale) {
-        return new JLabel(library.getScaledUnitImageIcon(unitIcon, scale));
-    }
-
     /**
      * This function analyses an event and calls the right methods to take care
      * of the user's requests.
