@@ -20,9 +20,10 @@
 package net.sf.freecol.client.gui.panel;
 
 import java.awt.Color;
-import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.List;
@@ -32,11 +33,14 @@ import javax.swing.JLabel;
 import javax.swing.JPanel;
 import javax.swing.SwingConstants;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Building;
+import net.sf.freecol.common.model.BuildingType;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Unit;
 import cz.autel.dmi.HIGConstraints;
@@ -47,8 +51,6 @@ import cz.autel.dmi.HIGLayout;
  */
 public final class ReportProductionPanel extends JPanel implements ActionListener {
 
-
-
     /** How many additional rows are defined. */
     private final int extraRows = 2;
 
@@ -58,18 +60,11 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
     /** How wide the margins should be. */
     private final int marginWidth = 12;
 
-    /** The widths of the columns. */
-    private final int[] widths;
-
-    /** The heights of the rows. */
-    // private final int[] heights;
-    private static final HIGConstraints higConst = new HIGConstraints();
-
     private Canvas parent;
 
     private List<Colony> colonies;
 
-    private final int goodsType;
+    private final GoodsType goodsType;
 
     private final ReportPanel reportPanel;
 
@@ -81,31 +76,11 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
      * 
      * @param parent The parent of this panel.
      */
-    public ReportProductionPanel(int goodsType, Canvas parent, ReportPanel reportPanel) {
+    public ReportProductionPanel(GoodsType goodsType, Canvas parent, ReportPanel reportPanel) {
         this.goodsType = goodsType;
         this.parent = parent;
         this.reportPanel = reportPanel;
 
-        if (goodsType == Goods.BELLS) {
-            /*
-            widths = new int[] { 0, columnSeparatorWidth, 0, columnSeparatorWidth, 0, columnSeparatorWidth, 0,
-                    columnSeparatorWidth, 0, columnSeparatorWidth, 0, columnSeparatorWidth, 0, columnSeparatorWidth, 0 };
-            */
-            widths = new int[8];
-        } else if (goodsType == Goods.CROSSES) {
-            /*
-            widths = new int[] { 0, columnSeparatorWidth, 0, columnSeparatorWidth, 0, columnSeparatorWidth, 0,
-                    columnSeparatorWidth, 0 };
-            */
-            widths = new int[5];
-        } else {
-            /*
-            widths = new int[] { 0, columnSeparatorWidth, 0, columnSeparatorWidth, 0 };
-            */
-            widths = new int[3];
-        }
-        // heights = null;
-        totalProduction = 0;
         setOpaque(false);
     }
 
@@ -114,37 +89,57 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
      */
     public void initialize() {
         Player player = parent.getClient().getMyPlayer();
+        
+        int columns = 3;
+        if (goodsType == Goods.BELLS) {
+            // percentage, sol, tory, percent50, percent100
+            columns += 5;
+        }
+
+        int buildingCount[] = new int[FreeCol.getSpecification().numberOfBuildingTypes()];
+        List<BuildingType> buildingTypes = new ArrayList<BuildingType>();
+        for (BuildingType buildingType : FreeCol.getSpecification().getBuildingTypeList()) {
+            if ((buildingType.getProducedGoodsType() == goodsType ||
+                 !buildingType.getModifierSet(goodsType.getId()).isEmpty()) &&
+                !buildingType.getGoodsRequired().isEmpty()) {
+                buildingTypes.add(buildingType);
+                buildingCount[buildingType.getIndex()] = 0;
+            }
+        }
+
+        // add a column for each building
+        columns += buildingTypes.size();
 
         int colonyColumn = 1;
         int productionColumn = 2;
         int unitColumn = 3;
-        int level1Column = 4;
-        int level2Column = 5;
-        int percentageColumn = 6;
-        int solColumn = 7;
-        int toryColumn = 8;
+        int buildingColumn = 4;
+        int percentageColumn = buildingColumn + buildingTypes.size();
+        int solColumn = percentageColumn + 1;
+        int toryColumn = solColumn + 1;
+        int percent50Column = toryColumn + 1;
+        int percent100Column = percent50Column + 1;
 
         // Display Panel
         removeAll();
 
         colonies = player.getColonies();
         int numberOfColonies = colonies.size();
+        int[] widths = new int[columns];
         int[] heights = new int[numberOfColonies + extraRows];
 
-        for (int index = extraRows; index < heights.length; index++) {
-            /** TODO: replace this magic number by some value from the
-             * ImageLibrary. At the moment this is difficult, as the
-             * unit images in the library are not sorted according to
-             * type. For this purpose, however, one would need to
-             * consider only ships for the naval report and only units
-             * able to defend colonies for the military report. The
-             * value of 64 is large enough to accommodate the 2/3
-             * scale version of all unit graphics.
-             */
-            heights[index] = 64;
-        }
-        
+        /** TODO: replace this magic number by some value from the
+         * ImageLibrary. At the moment this is difficult, as the
+         * unit images in the library are not sorted according to
+         * type. For this purpose, however, one would need to
+         * consider only ships for the naval report and only units
+         * able to defend colonies for the military report. The
+         * value of 64 is large enough to accommodate the 2/3
+         * scale version of all unit graphics.
+         */
+        Arrays.fill(heights, 0, heights.length, 64);        
 
+        HIGConstraints higConst = new HIGConstraints();
         setLayout(new HIGLayout(widths, heights));
 
         JLabel newLabel;
@@ -161,16 +156,15 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
         newLabel.setBorder(FreeColPanel.TOPCELLBORDER);
         add(newLabel, higConst.rc(1, unitColumn));
 
+        int column = buildingColumn;
+        for (BuildingType buildingType : buildingTypes) {
+            newLabel = new JLabel(buildingType.getName());
+            newLabel.setBorder(FreeColPanel.TOPCELLBORDER);
+            add(newLabel, higConst.rc(1, column));
+            column++;
+        }
+        
         if (goodsType == Goods.BELLS) {
-
-            newLabel = new JLabel(Messages.message("colopedia.buildings.name.PrintingPress"));
-            newLabel.setBorder(FreeColPanel.TOPCELLBORDER);
-            add(newLabel, higConst.rc(1, level1Column));
-
-            newLabel = new JLabel(Messages.message("colopedia.buildings.name.Newspaper"));
-            newLabel.setBorder(FreeColPanel.TOPCELLBORDER);
-            add(newLabel, higConst.rc(1, level2Column));
-
             newLabel = new JLabel(Messages.message("sonsOfLiberty") + "%");
             newLabel.setBorder(FreeColPanel.TOPCELLBORDER);
             add(newLabel, higConst.rc(1, percentageColumn));
@@ -183,16 +177,15 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
             newLabel.setBorder(FreeColPanel.TOPCELLBORDER);
             add(newLabel, higConst.rc(1, toryColumn));
 
-        } else if (goodsType == Goods.CROSSES) {
-
-            newLabel = new JLabel(Messages.message("colopedia.buildings.name.Church"));
+            newLabel = new JLabel("50%");
+            newLabel.setToolTipText(Messages.message("report.50percent"));
             newLabel.setBorder(FreeColPanel.TOPCELLBORDER);
-            add(newLabel, higConst.rc(1, level1Column));
+            add(newLabel, higConst.rc(1, percent50Column));
 
-            newLabel = new JLabel(Messages.message("colopedia.buildings.name.Cathedral"));
+            newLabel = new JLabel("100%");
+            newLabel.setToolTipText(Messages.message("report.100percent"));
             newLabel.setBorder(FreeColPanel.TOPCELLBORDER);
-            add(newLabel, higConst.rc(1, level2Column));
-
+            add(newLabel, higConst.rc(1, percent100Column));
         }
             
 
@@ -200,10 +193,6 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
         totalProduction = 0;
         int totalUnits = 0;
 
-        int pressCount = 0;
-        int paperCount = 0;
-        int churchCount = 0;
-        int cathedralCount = 0;
         int percentageCount = 0;
         int solCount = 0;
         int toryCount = 0;
@@ -218,10 +207,12 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
             add(createColonyButton(colonyIndex), higConst.rc(row, colonyColumn));
 
             // production
-            Goods goods = new Goods(goodsType);
             int newValue = colony.getProductionOf(goodsType);
-            goods.setAmount(newValue);
+            if (goodsType == Goods.BELLS) {
+                newValue -= colony.getBellUpkeep();
+            }
             totalProduction += newValue;
+            Goods goods = new Goods(colony.getGame(), colony, goodsType, newValue);
             GoodsLabel goodsLabel = new GoodsLabel(goods, parent);
             goodsLabel.setHorizontalAlignment(JLabel.LEADING);
             goodsLabel.setBorder(FreeColPanel.CELLBORDER);
@@ -243,25 +234,25 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
             }
             add(unitPanel, higConst.rc(row, unitColumn));
 
+            
+            column = buildingColumn;
+            for (BuildingType buildingType : buildingTypes) {
+                Building b = colony.getBuilding(buildingType);
+                int level = 0;
+                if (b!=null)
+                    level = b.getLevel();
+                newLabel = new JLabel();
+                newLabel.setBorder(FreeColPanel.CELLBORDER);
+                if (level == buildingType.getLevel()) {
+                    newLabel.setText("X");
+                    buildingCount[buildingType.getIndex()]++;
+                }
+                add(newLabel, higConst.rc(row, column));
+                column++;
+            }
+
             // special
             if (goodsType == Goods.BELLS) {
-
-                int level = colony.getBuilding(Building.PRINTING_PRESS).getLevel();
-                newLabel = new JLabel();
-                newLabel.setBorder(FreeColPanel.CELLBORDER);
-                if (level == Building.HOUSE) {
-                    newLabel.setText("X");
-                    pressCount++;
-                }
-                add(newLabel, higConst.rc(row, level1Column));
-                newLabel = new JLabel();
-                newLabel.setBorder(FreeColPanel.CELLBORDER);
-                if (level == Building.SHOP) {
-                    newLabel.setText("X");
-                    paperCount++;
-                }
-                add(newLabel, higConst.rc(row, level2Column));
-
                 int percentage = colony.getSoL();
                 percentageCount += percentage;
                 int count = colony.getUnitCount();
@@ -280,23 +271,34 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
                 newLabel.setBorder(FreeColPanel.CELLBORDER);
                 add(newLabel, higConst.rc(row, toryColumn));
 
-            } else if (goodsType == Goods.CROSSES) {
-
-                int level = colony.getBuilding(Building.CHURCH).getLevel();
-                newLabel = new JLabel();
-                newLabel.setBorder(FreeColPanel.CELLBORDER);
-                if (level == Building.HOUSE) {
-                    newLabel.setText("X");
-                    churchCount++;
+                if (newValue > 0 && percentage < 50) {
+                    int required = (Colony.BELLS_PER_REBEL * count) / 2;
+                    int turns = required / newValue;
+                    if (required % newValue > 0) {
+                        turns++;
+                    }
+                    newLabel = new JLabel(String.valueOf(turns), JLabel.TRAILING);
+                    newLabel.setBorder(FreeColPanel.CELLBORDER);
+                    add(newLabel, higConst.rc(row, percent50Column));
+                } else {
+                    newLabel = new JLabel();
+                    newLabel.setBorder(FreeColPanel.CELLBORDER);
+                    add(newLabel, higConst.rc(row, percent50Column));
                 }
-                add(newLabel, higConst.rc(row, level1Column));
-                newLabel = new JLabel();
-                newLabel.setBorder(FreeColPanel.CELLBORDER);
-                if (level == Building.SHOP) {
-                    newLabel.setText("X");
-                    cathedralCount++;
+                if (newValue > 0 && percentage < 100) {
+                    int required = Colony.BELLS_PER_REBEL * count;
+                    int turns = required / newValue;
+                    if (required % newValue > 0) {
+                        turns++;
+                    }
+                    newLabel = new JLabel(String.valueOf(turns), JLabel.TRAILING);
+                    newLabel.setBorder(FreeColPanel.CELLBORDER);
+                    add(newLabel, higConst.rc(row, percent100Column));
+                } else {
+                    newLabel = new JLabel();
+                    newLabel.setBorder(FreeColPanel.CELLBORDER);
+                    add(newLabel, higConst.rc(row, percent100Column));
                 }
-                add(newLabel, higConst.rc(row, level2Column));
             }
 
 
@@ -306,13 +308,12 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
         row = 2;
         // summary
         JLabel allColonies = new JLabel(Messages.message("report.allColonies", "%number%",
-                String.valueOf(numberOfColonies)));
+                                                         String.valueOf(numberOfColonies)));
         allColonies.setForeground(Color.BLUE);
         allColonies.setBorder(FreeColPanel.LEFTCELLBORDER);
         add(allColonies, higConst.rc(row, colonyColumn));
 
-        Goods allGoods = new Goods(goodsType);
-        allGoods.setAmount(totalProduction);
+        Goods allGoods = new Goods(parent.getClient().getGame(), null, goodsType, totalProduction);
         GoodsLabel allGoodsLabel = new GoodsLabel(allGoods, parent);
         allGoodsLabel.setHorizontalAlignment(JLabel.LEADING);
         allGoodsLabel.setBorder(FreeColPanel.CELLBORDER);
@@ -322,13 +323,16 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
         newLabel.setBorder(FreeColPanel.CELLBORDER);
         add(newLabel, higConst.rc(row, unitColumn));
 
+        column = buildingColumn;
+        for (BuildingType buildingType : buildingTypes) {
+            int count = buildingCount[buildingType.getIndex()];
+            newLabel = new JLabel(String.valueOf(count), JLabel.TRAILING);
+            newLabel.setBorder(FreeColPanel.CELLBORDER);
+            add(newLabel, higConst.rc(row, column));
+            column++;
+        }
+        
         if (goodsType == Goods.BELLS) {
-            newLabel = new JLabel(String.valueOf(pressCount), JLabel.TRAILING);
-            newLabel.setBorder(FreeColPanel.CELLBORDER);
-            add(newLabel, higConst.rc(row, level1Column));
-            newLabel = new JLabel(String.valueOf(paperCount), JLabel.TRAILING);
-            newLabel.setBorder(FreeColPanel.CELLBORDER);
-            add(newLabel, higConst.rc(row, level2Column));
             int percentage = 0;
             if (numberOfColonies > 0) {
                 percentage = percentageCount / numberOfColonies;
@@ -342,27 +346,20 @@ public final class ReportProductionPanel extends JPanel implements ActionListene
             newLabel = new JLabel(String.valueOf(toryCount), JLabel.TRAILING);
             newLabel.setBorder(FreeColPanel.CELLBORDER);
             add(newLabel, higConst.rc(row, toryColumn));
-        } else if (goodsType == Goods.CROSSES) {
-            newLabel = new JLabel(String.valueOf(churchCount), JLabel.TRAILING);
+            newLabel = new JLabel();
             newLabel.setBorder(FreeColPanel.CELLBORDER);
-            add(newLabel, higConst.rc(row, level1Column));
-            newLabel = new JLabel(String.valueOf(cathedralCount), JLabel.TRAILING);
+            add(newLabel, higConst.rc(row, percent50Column));
+            newLabel = new JLabel();
             newLabel.setBorder(FreeColPanel.CELLBORDER);
-            add(newLabel, higConst.rc(row, level2Column));
+            add(newLabel, higConst.rc(row, percent100Column));
         }
     }
 
 
     private JButton createColonyButton(int index) {
-
-        JButton button = new JButton(colonies.get(index).getName());
-        button.setMargin(new Insets(0,0,0,0));
-        button.setOpaque(false);
-        button.setForeground(FreeColPanel.LINK_COLOR);
-        button.setAlignmentY(0.8f);
+        JButton button = FreeColPanel.getLinkButton(colonies.get(index).getName(), null, String.valueOf(index));
         button.setHorizontalAlignment(SwingConstants.LEADING);
         button.setBorder(FreeColPanel.LEFTCELLBORDER);
-        button.setActionCommand(String.valueOf(index));
         button.addActionListener(this);
         return button;
     }
