@@ -20,11 +20,9 @@
 package net.sf.freecol.client.gui.panel;
 
 import java.awt.Image;
-import java.awt.Insets;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.util.ArrayList;
-import java.util.Collections;
+import java.util.Arrays;
 import java.util.Comparator;
 
 import javax.swing.BorderFactory;
@@ -33,13 +31,10 @@ import javax.swing.JButton;
 import javax.swing.JComponent;
 import javax.swing.JLabel;
 import javax.swing.JTextPane;
-import javax.swing.text.Style;
 import javax.swing.text.StyleConstants;
-import javax.swing.text.StyleContext;
 import javax.swing.text.StyledDocument;
 
 import net.sf.freecol.client.ClientOptions;
-import net.sf.freecol.client.FreeColClient;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
 import net.sf.freecol.common.model.Colony;
@@ -57,41 +52,26 @@ import cz.autel.dmi.HIGLayout;
 /**
  * This panel displays the Turn Report.
  */
-public final class ReportTurnPanel extends ReportPanel implements ActionListener {
-
-
-
-    private final FreeColClient freeColClient;
-
-    private StyledDocument document;
+public final class ReportTurnPanel extends ReportPanel {
 
     /**
      * The constructor that will add the items to this panel.
      * 
      * @param parent The parent of this panel.
      */
-    public ReportTurnPanel(Canvas parent) {
+    public ReportTurnPanel(Canvas parent, ModelMessage... messages) {
         super(parent, Messages.message("menuBar.report.turn"));
-        this.freeColClient = parent.getClient();
-    }
-
-    /**
-     * Prepares this panel to be displayed.
-     *
-     * @param messages The messages to be displayed
-     */
-    public void initialize(ArrayList<ModelMessage> messages) {
 
         Comparator<ModelMessage> comparator = getCanvas().getClient().getClientOptions().getModelMessageComparator();
         if (comparator != null) {
-            Collections.sort(messages, comparator);
+            Arrays.sort(messages, comparator);
         }
 
         ClientOptions options = getCanvas().getClient().getClientOptions();
         int groupBy = options.getInteger(ClientOptions.MESSAGES_GROUP_BY);
 
         Object source = this;
-        int type = -1;
+        ModelMessage.MessageType type = null;
         int headlines = 0;
 
         // count number of headlines
@@ -109,7 +89,7 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
         reportPanel.removeAll();
 
         int[] widths = new int[] { 0, margin, 500, margin, 0, margin, 0 };
-        int[] heights = new int[2 * (messages.size() + headlines) - 1];
+        int[] heights = new int[2 * (messages.length + headlines) - 1];
         int imageColumn = 1;
         int textColumn = 3;
         int button1Column = 5;
@@ -124,7 +104,7 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
         reportPanel.setLayout(layout);
 
         source = this;
-        type = -1;
+        type = null;
 
         int row = 1;
         for (final ModelMessage message : messages) {
@@ -141,10 +121,10 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
                 row += 2;
             }
 
-            final JLabel label = new JLabel();
+            JComponent component = new JLabel();
             if (message.getDisplay() != null) {
 
-                // TODO Scale icons relative to fond size.
+                // TODO: Scale icons relative to font size.
                 ImageIcon icon = getCanvas().getImageIcon(message.getDisplay(), false);
                 if (icon != null && icon.getIconHeight() > 40) {
                     Image image = icon.getImage();
@@ -154,31 +134,40 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
                 }
 
                 if (message.getDisplay() instanceof Colony) {
-                    final JButton button = new JButton();
+                    JButton button = new JButton();
                     button.setIcon(icon);
                     button.setActionCommand(((Colony) message.getDisplay()).getId());
                     button.addActionListener(this);
                     button.setBorder(BorderFactory.createEmptyBorder());
+                    component = button;
                 } else if (message.getDisplay() instanceof Unit) {
-                    label.setIcon(icon);
+                    JButton button = new JButton();
+                    button.setIcon(icon);
+                    button.setActionCommand(((Unit) message.getDisplay()).getLocation().getId());
+                    button.addActionListener(this);
+                    button.setBorder(BorderFactory.createEmptyBorder());
+                    component = button;
                 } else if (message.getDisplay() instanceof Player) {
-                    label.setIcon(icon);
+                    component = new JLabel(icon);
                 } else {
-                    label.setIcon(icon);
+                    component = new JLabel(icon);
                 }
 
-                reportPanel.add(label, higConst.rc(row, imageColumn, ""));
+                reportPanel.add(component, higConst.rc(row, imageColumn, ""));
             }
 
-            final JTextPane textPane = getTextPane(message);
+            final JTextPane textPane = getDefaultTextPane();
+            insertMessage(textPane.getStyledDocument(), message, getCanvas().getClient().getMyPlayer());
+
+            final JComponent label = component;
             reportPanel.add(textPane, higConst.rc(row, textColumn));
-            if (message.getType() == ModelMessage.WAREHOUSE_CAPACITY) {
+            if (message.getType() == ModelMessage.MessageType.WAREHOUSE_CAPACITY) {
                 JButton ignoreButton = new JButton("x");
                 ignoreButton.setToolTipText(Messages.message("model.message.ignore", message.getData()));
                 ignoreButton.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
                         boolean flag = label.isEnabled();
-                        freeColClient.getInGameController().ignoreMessage(message, flag);
+                        getCanvas().getClient().getInGameController().ignoreMessage(message, flag);
                         textPane.setEnabled(!flag);
                         label.setEnabled(!flag);
                     }
@@ -191,7 +180,6 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
                 JButton filterButton = new JButton("X");
                 filterButton.setToolTipText(Messages.message("model.message.filter", 
                     "%type%", message.getTypeName()));
-            
                 filterButton.addActionListener(new ActionListener() {
                     public void actionPerformed(ActionEvent event) {
                         boolean flag = filterOption.getValue();
@@ -204,7 +192,6 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
             }
             row += 2;
         }
-
     }
 
     private JComponent getHeadline(Object source) {
@@ -225,7 +212,7 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
         } else if (source instanceof Market) {
             JButton button = new JButton(Messages.message("model.message.marketPrices"));
             button.addActionListener(this);
-            button.setActionCommand(freeColClient.getMyPlayer().getEurope().getId());
+            button.setActionCommand(getCanvas().getClient().getMyPlayer().getEurope().getId());
             headline = button;
         } else if (source instanceof Colony) {
             final Colony colony = (Colony) source;
@@ -237,7 +224,7 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
             final Unit unit = (Unit) source;
             JButton button = new JButton(unit.getName());
             button.addActionListener(this);
-            button.setActionCommand(unit.getTile().getId());
+            button.setActionCommand(unit.getLocation().getId());
             headline = button;
         } else if (source instanceof Nameable) {
             headline = new JLabel(((Nameable) source).getName());
@@ -252,121 +239,105 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
         return headline;
     }
 
-    //Create a text pane.
-    private JTextPane getTextPane(ModelMessage message) {
-        JTextPane textPane = new JTextPane();
-        textPane.setOpaque(false);
-        textPane.setEditable(false);
+    private void insertMessage(StyledDocument document, ModelMessage message, Player player) {
 
-        document = textPane.getStyledDocument();
-        //Initialize some styles.
-        Style def = StyleContext.getDefaultStyleContext().getStyle(StyleContext.DEFAULT_STYLE);
-        
-        Style regular = document.addStyle("regular", def);
-        StyleConstants.setFontFamily(def, "Dialog");
-        StyleConstants.setBold(def, true);
-        StyleConstants.setFontSize(def, 12);
-
-        Style buttonStyle = document.addStyle("button", regular);
-        StyleConstants.setForeground(buttonStyle, LINK_COLOR);
-
-        insertMessage(message);
-        return textPane;
-    }
-
-
-
-    private void insertMessage(ModelMessage message) {
-
-        Player player = freeColClient.getMyPlayer();
         try {
-            String input = Messages.message(message.getMessageID());
+            String input = Messages.message(message.getId());
             int start = input.indexOf('%');
             if (start == -1) {
                 // no variables present
-                insertText(input.substring(0));
+                insertText(document, input.substring(0));
                 return;
             } else if (start > 0) {
-                // output any string before the first occurence of '%'
-                insertText(input.substring(0, start));
+                // output any string before the first occurrence of '%'
+                insertText(document, input.substring(0, start));
             }
             int end;
 
-            loop: while ((end = input.indexOf('%', start + 1)) >= 0) {
+            while ((end = input.indexOf('%', start + 1)) >= 0) {
                 String var = input.substring(start, end + 1);
-                for (String[] item : message.getData()) {
-                    if (item == null || item.length != 2) {
-                        logger.warning("Data has a wrong format for message: " + message);
-                    } else if (item[0] == null || item[1] == null) {
-                        logger.warning("Data in model message is 'null': " + message + ", " +
-                                       item[0] + ", " + item[1]);
-                    } else if (var.equals(item[0])) {
-                        // found variable to replace
-                        if (var.equals("%colony%")) {
-                            Colony colony = player.getColony(item[1]);
-                            if (colony != null) {
-                                insertLinkButton(colony, colony.getName());
-                            } else {
-                                insertText(item[1]);
-                            }
-                        } else if (var.equals("%europe%")) {
-                            insertLinkButton(player.getEurope(), player.getEurope().getName());
-                        } else if ((var.equals("%unit%") ||
-                                    var.equals("%newName%")) &&
-                                   message.getSource() instanceof Unit) {
-                            Tile tile = ((Unit) message.getSource()).getTile();
-                            if (tile != null) {
-                                insertLinkButton(tile, item[1]);
-                            } else {
-                                insertText(item[1]);
-                            }
+                String[] item = findReplacementData(message, var);
+                if (item != null && var.equals(item[0])) {
+                    // found variable to replace
+                    if (var.equals("%colony%")) {
+                        Colony colony = player.getColony(item[1]);
+                        if (colony != null) {
+                            insertLinkButton(document, colony, item[1]);
+                        } else if (message.getSource() instanceof Tile) {
+                            insertLinkButton(document, message.getSource(), item[1]);
                         } else {
-                            insertText(item[1]);
+                            insertText(document, item[1]);
                         }
-                        start = end + 1;
-                        continue loop;
+                    } else if (var.equals("%europe%")) {
+                        insertLinkButton(document, player.getEurope(), player.getEurope().getName());
+                    } else if (var.equals("%unit%") || var.equals("%newName%")) {
+                        Tile tile = null;
+                        if (message.getSource() instanceof Unit) {
+                            tile = ((Unit) message.getSource()).getTile();
+                        } else if (message.getSource() instanceof Tile) {
+                            tile = (Tile)message.getSource();
+                        }
+                        if (tile != null) {
+                            insertLinkButton(document, tile, item[1]);
+                        } else {
+                            insertText(document, item[1]);
+                        }
+                    } else {
+                        insertText(document, item[1]);
                     }
+                    start = end + 1;
+                } else {
+                    // found no variable to replace: either a single '%', or
+                    // some unnecessary variable
+                    insertText(document, input.substring(start, end));
+                    start = end;
                 }
-
-                // found no variable to replace: either a single '%', or
-                // some unnecessary variable
-                insertText(input.substring(start, end));
-                start = end;
             }
 
-            // output any string after the last occurence of '%'
+            // output any string after the last occurrence of '%'
             if (start < input.length()) {
-                insertText(input.substring(start));
+                insertText(document, input.substring(start));
             }
+
         } catch(Exception e) {
             logger.warning(e.toString());
         }
-        
-
+    }
+    
+    private String[] findReplacementData(ModelMessage message, String variable) {
+        String[] data = message.getData();
+        if (data == null) {
+            // message with no variables
+            return null;
+        } else if (data.length % 2 == 0) {
+            for (int index = 0; index < data.length; index += 2) {
+                if (variable.equals(data[index])) {
+                    return new String[] { variable, data[index + 1] };
+                }
+            }
+        } else {
+            logger.warning("Data has a wrong format for message: " + message);
+        }
+        return null;
     }
 
-    private void insertText(String text) throws Exception {
+    private void insertText(StyledDocument document, String text) throws Exception {
         document.insertString(document.getLength(), text,
                               document.getStyle("regular"));
     }
 
 
-    private void insertLinkButton(FreeColGameObject object, String name) throws Exception {
-        JButton button = new JButton(name);
-        button.setMargin(new Insets(0,0,0,0));
-        button.setOpaque(false);
-        button.setForeground(LINK_COLOR);
-        button.setAlignmentY(0.8f);
-        button.setBorder(BorderFactory.createEmptyBorder());
+    private void insertLinkButton(StyledDocument document, FreeColGameObject object, String name)
+        throws Exception {
+        JButton button = getLinkButton(name, null, object.getId());
         button.addActionListener(this);
-        button.setActionCommand(object.getId());
         StyleConstants.setComponent(document.getStyle("button"), button);
         document.insertString(document.getLength(), " ", document.getStyle("button"));
     }
 
 
     /**
-     * This function analyses an event and calls the right methods to take care
+     * This function analyzes an event and calls the right methods to take care
      * of the user's requests.
      * 
      * @param event The incoming ActionEvent.
@@ -374,11 +345,10 @@ public final class ReportTurnPanel extends ReportPanel implements ActionListener
     @Override
     public void actionPerformed(ActionEvent event) {
         String command = event.getActionCommand();
-        //System.out.println(command);
         if (command.equals(String.valueOf(OK))) {
             super.actionPerformed(event);
         } else {
-            FreeColGameObject object = freeColClient.getGame().getFreeColGameObject(command);
+            FreeColGameObject object = getCanvas().getClient().getGame().getFreeColGameObject(command);
             if (object instanceof Europe) {
                 getCanvas().showEuropePanel();
             } else if (object instanceof Tile) {
