@@ -46,9 +46,10 @@ import javax.swing.TransferHandler;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.client.gui.Canvas;
 import net.sf.freecol.client.gui.i18n.Messages;
-import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Location;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Settlement;
@@ -60,11 +61,9 @@ import cz.autel.dmi.HIGLayout;
  * Allows the user to edit trade routes.
  */
 public final class TradeRouteInputDialog extends FreeColDialog implements ActionListener {
+
     private static final Logger logger = Logger.getLogger(TradeRouteInputDialog.class.getName());
 
-
-
-
     private static final int OK = 0, CANCEL = 1;
 
     private TradeRoute originalRoute;
@@ -285,7 +284,7 @@ public final class TradeRouteInputDialog extends FreeColDialog implements Action
     public void requestFocus() {
         ok.requestFocus();
     }
-
+    
     private boolean verifyNewTradeRoute(){
     	// Verify that it has at least two stops
     	if(listModel.getSize() < 2){
@@ -294,7 +293,7 @@ public final class TradeRouteInputDialog extends FreeColDialog implements Action
     	} 	
     	return true;
     }
-    
+
     /**
      * This function analyses an event and calls the right methods to take care
      * of the user's requests.
@@ -306,7 +305,7 @@ public final class TradeRouteInputDialog extends FreeColDialog implements Action
         try {
             switch (Integer.valueOf(command).intValue()) {
             case OK:
-            	// the
+            	
             	if(! verifyNewTradeRoute())
             		break;
             	
@@ -342,17 +341,17 @@ public final class TradeRouteInputDialog extends FreeColDialog implements Action
      * AbstractGoodsLabel and a CargoLabel (its current function).
      */
     public class CargoLabel extends JLabel {
-        private final int goodsType;
+        private final GoodsType goodsType;
 
 
-        public CargoLabel(int type) {
-            super(getCanvas().getImageProvider().getGoodsImageIcon(type));
+        public CargoLabel(GoodsType type) {
+            super(getCanvas().getGUI().getImageLibrary().getGoodsImageIcon(type));
             setTransferHandler(cargoHandler);
             addMouseListener(dragListener);
             this.goodsType = type;
         }
 
-        public int getType() {
+        public GoodsType getType() {
             return this.goodsType;
         }
 
@@ -364,10 +363,12 @@ public final class TradeRouteInputDialog extends FreeColDialog implements Action
     public class GoodsPanel extends JPanel {
 
         public GoodsPanel() {
-            super(new GridLayout(4, 4, margin, margin));
-            for (int goods = 0; goods < Goods.NUMBER_OF_TYPES; goods++) {
-                CargoLabel label = new CargoLabel(goods);
-                add(label);
+            super(new GridLayout(5, 5, margin, margin));
+            for (GoodsType goodsType : FreeCol.getSpecification().getGoodsTypeList()) {
+                if (goodsType.isStorable()) {
+                    CargoLabel label = new CargoLabel(goodsType);
+                    add(label);
+                }
             }
             setOpaque(false);
             setBorder(BorderFactory.createTitledBorder(Messages.message("goods")));
@@ -398,8 +399,8 @@ public final class TradeRouteInputDialog extends FreeColDialog implements Action
             removeAll();
             if (newStop != null) {
                 // stop = newStop;
-                for (Integer cargo : newStop.getCargo()) {
-                    add(new CargoLabel(cargo.intValue()));
+                for (GoodsType goodsType : newStop.getCargo()) {
+                    add(new CargoLabel(goodsType));
                 }
             }
             revalidate();
@@ -433,7 +434,7 @@ public final class TradeRouteInputDialog extends FreeColDialog implements Action
                         cargoPanel.revalidate();
                         Stop stop = (Stop) stopList.getSelectedValue();
                         if (stop != null) {
-                            stop.addCargo(new Integer(label.getType()));
+                            stop.addCargo(label.getType());
                             stop.setModified(true);
                         }
                     }
@@ -455,9 +456,9 @@ public final class TradeRouteInputDialog extends FreeColDialog implements Action
                     cargoPanel.remove(label);
                     Stop stop = (Stop) stopList.getSelectedValue();
                     if (stop != null) {
-                        ArrayList<Integer> cargo = stop.getCargo();
+                        ArrayList<GoodsType> cargo = stop.getCargo();
                         for (int index = 0; index < cargo.size(); index++) {
-                            if (cargo.get(index).intValue() == label.getType()) {
+                            if (cargo.get(index) == label.getType()) {
                                 cargo.remove(index);
                                 stop.setModified(true);
                                 break;
