@@ -19,10 +19,13 @@
 
 package net.sf.freecol.common.model;
 
+import java.beans.PropertyChangeEvent;
+import java.beans.PropertyChangeListener;
 import java.io.File;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
-import java.util.Vector;
+import java.util.List;
 import java.util.logging.Logger;
 
 import javax.xml.stream.XMLStreamConstants;
@@ -30,6 +33,9 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.client.gui.i18n.Messages;
+
 /**
  * The main component of the game model.
  * 
@@ -42,22 +48,21 @@ import javax.xml.stream.XMLStreamWriter;
  */
 public class Game extends FreeColGameObject {
 
-
-
     private static final Logger logger = Logger.getLogger(Game.class.getName());
 
-    /** A virtual player to use with enemy privateers */
-    // TODO: Using null here throws a warning. A Player should belong to a Game.
-    public static final Player unknownEnemy = new Player(null, "", false, Player.NO_NATION);
+    protected int playerIndex = 0;
+
+    /** 
+     * A virtual player to use with enemy privateers 
+     */
+    private Player unknownEnemy;
 
     /** Contains all the players in the game. */
-    private Vector<Player> players = new Vector<Player>();
+    private List<Player> players = new ArrayList<Player>();
 
     private Map map;
 
     private GameOptions gameOptions;
-    
-    private GameVariables gameVariables;
 
     /** The name of the player whose turn it is. */
     private Player currentPlayer = null;
@@ -75,12 +80,12 @@ public class Game extends FreeColGameObject {
     private HashMap<String, FreeColGameObject> freeColGameObjects = new HashMap<String, FreeColGameObject>(10000);
 
     /**
-     * The next availeble ID, that can be given to a new
+     * The next available ID, that can be given to a new
      * <code>FreeColGameObject</code>.
      */
     private int nextId = 1;
 
-    /** Indicates wether or not this object may give IDs. */
+    /** Indicates whether or not this object may give IDs. */
     private boolean canGiveID;
 
     private Turn turn = new Turn(1);
@@ -92,6 +97,21 @@ public class Game extends FreeColGameObject {
     /** The lost city rumour class. */
     private final static LostCityRumour lostCityRumour = new LostCityRumour();
 
+    /**
+     * The combat model this game uses. At the moment, the only combat
+     * model available is the SimpleCombatModel, which strives to
+     * implement the combat model of the original game. However, it is
+     * anticipated that other, more complex combat models will be
+     * implemented in future. As soon as that happens, we will also
+     * have to make the combat model selectable.
+     */
+    private CombatModel combatModel;
+
+    /**
+     * The nations that can still be selected in this Game.
+     */
+    private List<Nation> vacantNations = new ArrayList<Nation>();
+
 
     /**
      * Creates a new game model.
@@ -106,10 +126,10 @@ public class Game extends FreeColGameObject {
         super(null);
 
         this.modelController = modelController;
+        this.combatModel = new SimpleCombatModel(modelController.getPseudoRandom());
         this.viewOwner = null;
 
         gameOptions = new GameOptions();
-        gameVariables = new GameVariables();
 
         currentPlayer = null;
         canGiveID = true;
@@ -127,25 +147,29 @@ public class Game extends FreeColGameObject {
      * @param in The input stream containing the XML.
      * @param fcgos A list of <code>FreeColGameObject</code>s to be added to
      *            this <code>Game</code>.
-     * @throws XMLStreamException if an error occured during parsing.
+     * @throws XMLStreamException if an error occurred during parsing.
      * @see net.sf.freecol.server.FreeColServer#loadGame(File)
      */
     public Game(FreeColGameObjectListener freeColGameObjectListener, ModelController modelController,
-            XMLStreamReader in, FreeColGameObject[] fcgos) throws XMLStreamException {
+                XMLStreamReader in, FreeColGameObject[] fcgos) throws XMLStreamException {
         super(null, in);
 
         setFreeColGameObjectListener(freeColGameObjectListener);
         this.modelController = modelController;
+        if (modelController != null) {
+            // no model controller when using the map editor
+            this.combatModel = new SimpleCombatModel(modelController.getPseudoRandom());
+        }
         this.viewOwner = null;
 
         canGiveID = true;
 
-        for (int i = 0; i < fcgos.length; i++) {
-            fcgos[i].setGame(this);
-            fcgos[i].updateID();
+        for (FreeColGameObject object : fcgos) {
+            object.setGame(this);
+            object.updateID();
 
-            if (fcgos[i] instanceof Player) {
-                players.add((Player) fcgos[i]);
+            if (object instanceof Player) {
+                players.add((Player) object);
             }
         }
 
@@ -217,19 +241,63 @@ public class Game extends FreeColGameObject {
      *      String, int)
      */
     public Game(ModelController modelController, XMLStreamReader in, String viewOwnerUsername)
-            throws XMLStreamException {
+        throws XMLStreamException {
         super(null, in);
 
         this.modelController = modelController;
+        this.combatModel = new SimpleCombatModel(modelController.getPseudoRandom());
         canGiveID = false;
         readFromXML(in);
         this.viewOwner = getPlayerByName(viewOwnerUsername);
     }
 
+    /**
+     * Returns the Game's ModelController.
+     *
+     * @return a <code>ModelController</code> value
+     */
     public ModelController getModelController() {
         return modelController;
     }
 
+    /**
+     * Returns the "Unknown Enemy" Player, which is used for
+     * privateers.
+     *
+     * @return a <code>Player</code> value
+     */
+    public Player getUnknownEnemy() {
+    	return unknownEnemy;
+    }
+
+    /**
+     * Sets the "Unknown Enemy" Player, which is used for
+     * privateers.
+     *
+     * @param player a <code>Player</code> value
+     */
+    public void setUnknownEnemy(Player player) {
+        this.unknownEnemy = player;
+    }
+    
+    /**
+     * Get the <code>VacantNations</code> value.
+     *
+     * @return a <code>List<Nation></code> value
+     */
+    public final List<Nation> getVacantNations() {
+        return vacantNations;
+    }
+
+    /**
+     * Set the <code>VacantNations</code> value.
+     *
+     * @param newNations the new list of vacant nations
+     */
+    public final void setVacantNations(final List<Nation> newNations) {
+        this.vacantNations = newNations;
+    }
+
     /**
      * Check if the clients are trusted or if the server should keep secrets in
      * order to prevent cheating.
@@ -268,11 +336,10 @@ public class Game extends FreeColGameObject {
         Iterator<Player> pit = getPlayerIterator();
         while (pit.hasNext()) {
             Player p = pit.next();
-            Iterator<Settlement> it = p.getSettlementIterator();
-            while (it.hasNext()) {
-                Colony colony = (Colony) it.next();
-                if (colony.getName().equals(name)) {
-                    return colony;
+            if (p.isEuropean()) {
+                Colony result = p.getColony(name);
+                if (result != null) {
+                    return result;
                 }
             }
         }
@@ -283,6 +350,28 @@ public class Game extends FreeColGameObject {
         return turn;
     }
 
+    void setTurn(Turn newTurn) {
+        turn = newTurn;
+    }
+    
+    /**
+     * Get the <code>CombatModel</code> value.
+     *
+     * @return a <code>CombatModel</code> value
+     */
+    public final CombatModel getCombatModel() {
+        return combatModel;
+    }
+
+    /**
+     * Set the <code>CombatModel</code> value.
+     *
+     * @param newCombatModel The new CombatModel value.
+     */
+    public final void setCombatModel(final CombatModel newCombatModel) {
+        this.combatModel = newCombatModel;
+    }
+
     /**
      * Returns this game's LostCityRumour.
      * 
@@ -304,9 +393,7 @@ public class Game extends FreeColGameObject {
 
             return id;
         } else {
-            logger.warning("The client's \"Game\" was requested to give out an id.");
             throw new Error("The client's \"Game\" was requested to give out an id.");
-            // return null;
         }
     }
 
@@ -319,6 +406,15 @@ public class Game extends FreeColGameObject {
     public void addPlayer(Player player) {
         if (player.isAI() || canAddNewPlayer()) {
             players.add(player);
+            vacantNations.remove(FreeCol.getSpecification().getNation(player.getNationID()));
+            player.addPropertyChangeListener("nationID", new PropertyChangeListener() {
+                public void propertyChange(PropertyChangeEvent e) {
+                    final Nation oldNation = FreeCol.getSpecification().getNation((String) e.getOldValue());
+                    final Nation newNation = FreeCol.getSpecification().getNation((String) e.getNewValue());
+                    vacantNations.remove(newNation);
+                    vacantNations.add(oldNation);
+                }
+            });
 
             if (currentPlayer == null) {
                 currentPlayer = player;
@@ -338,6 +434,7 @@ public class Game extends FreeColGameObject {
         boolean updateCurrentPlayer = (currentPlayer == player);
 
         players.remove(players.indexOf(player));
+        vacantNations.add(FreeCol.getSpecification().getNation(player.getNationID()));
         player.dispose();
 
         if (updateCurrentPlayer) {
@@ -351,20 +448,21 @@ public class Game extends FreeColGameObject {
      * @param id The unique ID of the <code>FreeColGameObject</code>.
      * @param freeColGameObject The <code>FreeColGameObject</code> that shall
      *            be added to this <code>Game</code>.
-     * @exception NullPointerException If either <code>id</code> or
-     *                <code>freeColGameObject
-     *                                   </code> are <i>null</i>.
+     * @exception IllegalArgumentException If either <code>id</code>
+     *                or <code>freeColGameObject </code> are
+     *                <i>null</i>.
      */
     public void setFreeColGameObject(String id, FreeColGameObject freeColGameObject) {
-        if (id == null || id.equals("") || freeColGameObject == null) {
-            throw new NullPointerException();
+        if (id == null || id.equals("")) {
+            throw new IllegalArgumentException("Parameter 'id' must not be 'null' or empty string.");
+        } else if (freeColGameObject == null) {
+            throw new IllegalArgumentException("Parameter 'freeColGameObject' must not be 'null'.");
         }
 
         FreeColGameObject old = freeColGameObjects.put(id, freeColGameObject);
         if (old != null) {
-            logger.warning("Replacing FreeColGameObject: " + old.getClass() + " with " + freeColGameObject.getClass());
             throw new IllegalArgumentException("Replacing FreeColGameObject: " + old.getClass() + " with "
-                    + freeColGameObject.getClass());
+                                               + freeColGameObject.getClass());
         }
 
         if (freeColGameObjectListener != null) {
@@ -385,11 +483,11 @@ public class Game extends FreeColGameObject {
      * 
      * @param id The identifier of the <code>FreeColGameObject</code>.
      * @return The <code>FreeColGameObject</code>.
-     * @exception NullPointerException If <code>id == null</code>.
+     * @exception IllegalArgumentException If <code>id == null</code>, or <code>id = ""</code>.
      */
     public FreeColGameObject getFreeColGameObject(String id) {
         if (id == null || id.equals("")) {
-            throw new NullPointerException();
+            throw new IllegalArgumentException("Parameter 'id' must not be null or empty string.");
         }
 
         return freeColGameObjects.get(id);
@@ -403,7 +501,7 @@ public class Game extends FreeColGameObject {
      * @return game object with id or null.
      */
     public FreeColGameObject getFreeColGameObjectSafely(String id) {
-        if (id != null) {
+        if (id != null && id.length()>0) {
             return freeColGameObjects.get(id);
         } else {
             return null;
@@ -416,11 +514,11 @@ public class Game extends FreeColGameObject {
      * @param id The identifier of the <code>FreeColGameObject</code> that
      *            shall be removed from this <code>Game</code>.
      * @return The <code>FreeColGameObject</code> that has been removed.
-     * @exception NullPointerException If <code>id == null</code>.
+     * @exception IllegalArgumentException If <code>id == null</code>, or <code>id = ""</code>.
      */
     public FreeColGameObject removeFreeColGameObject(String id) {
         if (id == null || id.equals("")) {
-            throw new NullPointerException();
+            throw new IllegalArgumentException("Parameter 'id' must not be null or empty string.");
         }
 
         if (freeColGameObjectListener != null) {
@@ -454,37 +552,25 @@ public class Game extends FreeColGameObject {
      * 
      * @return A vacant nation.
      */
-    public int getVacantNation() {
-        boolean[] nationTaken = new boolean[4];
-
-        Iterator<Player> playerIterator = getPlayerIterator();
-        while (playerIterator.hasNext()) {
-            Player player = playerIterator.next();
-            if (player.getNation() < 4) {
-                nationTaken[player.getNation()] = true;
-            }
+    public Nation getVacantNation() {
+        if (vacantNations.isEmpty()) {
+            return null;
+        } else {
+            return vacantNations.get(0);
         }
-
-        for (int i = 0; i < nationTaken.length; i++) {
-            if (!nationTaken[i]) {
-                return i;
-            }
-        }
-
-        return -1;
     }
 
     /**
      * Return a <code>Player</code> identified by it's nation.
      * 
-     * @param nation The nation.
+     * @param nationID The nation.
      * @return The <code>Player</code> of the given nation.
      */
-    public Player getPlayer(int nation) {
+    public Player getPlayer(String nationID) {
         Iterator<Player> playerIterator = getPlayerIterator();
         while (playerIterator.hasNext()) {
             Player player = playerIterator.next();
-            if (player.getNation() == nation) {
+            if (player.getNationID().equals(nationID)) {
                 return player;
             }
         }
@@ -574,11 +660,8 @@ public class Game extends FreeColGameObject {
      *         number of turns by one.
      */
     public boolean isNextPlayerInNewTurn() {
-        return (players.indexOf(currentPlayer) > players.indexOf(getNextPlayer()) || currentPlayer == getNextPlayer());
-        /*
-         * int index = players.indexOf(currentPlayer) + 1; return index >=
-         * players.size();
-         */
+        return (players.indexOf(currentPlayer) > players.indexOf(getNextPlayer())
+                || currentPlayer == getNextPlayer());
     }
 
     /**
@@ -588,10 +671,10 @@ public class Game extends FreeColGameObject {
      *         <code>Game</code>.
      */
     public Player getFirstPlayer() {
-        if (players.size() > 0) {
-            return players.get(0);
-        } else {
+        if (players.isEmpty()) {
             return null;
+        } else {
+            return players.get(0);
         }
     }
 
@@ -610,7 +693,7 @@ public class Game extends FreeColGameObject {
     /**
      * Gets a <code>Player</code> specified by a name.
      * 
-     * @param name The name identifing the <code>Player</code>.
+     * @param name The name identifying the <code>Player</code>.
      * @return The <code>Player</code>.
      */
     public Player getPlayerByName(String name) {
@@ -627,23 +710,19 @@ public class Game extends FreeColGameObject {
     }
 
     /**
-     * Checks if the specfied name is in use.
+     * Checks if the specified name is in use.
      * 
      * @param username The name.
      * @return <i>true</i> if the name is already in use and <i>false</i>
      *         otherwise.
      */
     public boolean playerNameInUse(String username) {
-        Iterator<Player> playerIterator = getPlayerIterator();
-
-        while (playerIterator.hasNext()) {
-            Player player = playerIterator.next();
-
-            if (player.getUsername().equals(username)) {
+        
+        for (Player player : players) {
+            if (player.getName().equals(username)) {
                 return true;
             }
         }
-
         return false;
     }
 
@@ -663,32 +742,32 @@ public class Game extends FreeColGameObject {
      * 
      * @return The <code>Vector</code>.
      */
-    public Vector<Player> getPlayers() {
+    public List<Player> getPlayers() {
         return players;
     }
 
+    public int getNumberOfPlayers() {
+        return players.size();
+    }
+
     /**
      * Returns all the European players known by the player of this game.
      * 
      * @return All the European players known by the player of this game.
      */
-    public Vector<Player> getEuropeanPlayers() {
-        Vector<Player> europeans = new Vector<Player>();
-        Iterator<Player> playerIterator = getPlayerIterator();
-
-        while (playerIterator.hasNext()) {
-            Player player = playerIterator.next();
-
+    public List<Player> getEuropeanPlayers() {
+        List<Player> europeans = new ArrayList<Player>();
+        for (Player player : players) {
             if (player.isEuropean()) {
-                europeans.addElement(player);
+                europeans.add(player);
             }
         }
         return europeans;
     }
 
     /**
-     * Gets the maximum number of (human) players that can be added to this
-     * game.
+     * Gets the maximum number of (human) players that can be added to
+     * this game.
      * 
      * @return the maximum number of (human) players that can be added to this
      *         game
@@ -697,6 +776,16 @@ public class Game extends FreeColGameObject {
         return maxPlayers;
     }
 
+    /**
+     * Sets the maximum number of (human) players that can be added to
+     * this game.
+     *
+     * @param newMax an <code>int</code> value
+     */
+    public void setMaximumPlayers(int newMax) {
+        maxPlayers = newMax;
+    }
+
     /**
      * Checks if a new <code>Player</code> can be added.
      * 
@@ -722,16 +811,11 @@ public class Game extends FreeColGameObject {
      *         otherwise.
      */
     public boolean isAllPlayersReadyToLaunch() {
-        Iterator<Player> playerIterator = getPlayerIterator();
-
-        while (playerIterator.hasNext()) {
-            Player player = playerIterator.next();
-
+        for (Player player : players) {
             if (!player.isReady()) {
                 return false;
             }
         }
-
         return true;
     }
 
@@ -747,12 +831,14 @@ public class Game extends FreeColGameObject {
      */
     @SuppressWarnings("unchecked")
     public boolean checkIntegrity() {
+    	List<String> brokenObjects = new ArrayList<String>();
         boolean ok = true;
         Iterator<FreeColGameObject> iterator = ((HashMap<String, FreeColGameObject>) freeColGameObjects.clone())
-                .values().iterator();
+            .values().iterator();
         while (iterator.hasNext()) {
             FreeColGameObject fgo = iterator.next();
             if (fgo.isUninitialized()) {
+            	brokenObjects.add(fgo.getId());
                 logger.warning("Uinitialized object: " + fgo.getId() + " (" + fgo.getClass() + ")");
                 ok = false;
             }
@@ -761,9 +847,26 @@ public class Game extends FreeColGameObject {
             logger.info("Game integrity ok.");
         } else {
             logger.warning("Game integrity test failed.");
+            fixIntegrity(brokenObjects);
         }
         return ok;
     }
+    
+    /**
+     * Try to fix integrity problems
+     */
+    private boolean fixIntegrity(List<String> list){
+    	// try to update Units who may have missing info
+    	for(Player player : this.getPlayers()){
+    		for(Unit unit : player.getUnits()){
+    			if(unit.getOwner() == null){
+    				logger.warning("Fixing " + unit.getId() + ": owner missing");
+    				unit.setOwner(player);
+    			}
+    		}
+    	}
+    	return false;
+    }
 
     /**
      * Prepares this <code>Game</code> for a new turn.
@@ -773,16 +876,14 @@ public class Game extends FreeColGameObject {
      * 
      * @see #setFreeColGameObject
      */
-    @SuppressWarnings("unchecked")
     public void newTurn() {
         turn.increase();
         logger.info("Turn is now " + turn.toString());
 
         for (Player player : players) {
-            logger.finer("Calling newTurn for player " + player.getName());
+            logger.info("Calling newTurn for player " + player.getName());
             player.newTurn();
         }
-
     }
 
     /**
@@ -792,14 +893,6 @@ public class Game extends FreeColGameObject {
     public GameOptions getGameOptions() {
         return gameOptions;
     }
-    
-    /**
-     * Gets the <code>GameVariables</code> that is associated with this
-     * {@link Game}.
-     */
-    public GameVariables getGameVariables() {
-        return gameVariables;
-    }
 
     /**
      * Gets the amount of gold needed for inciting. This method should NEVER be
@@ -818,13 +911,15 @@ public class Game extends FreeColGameObject {
      */
     public static int getInciteAmount(Player payingPlayer, Player targetPlayer, Player attackingPlayer) {
         int amount = 0;
-        if (attackingPlayer.getTension(payingPlayer).getValue() > attackingPlayer.getTension(targetPlayer).getValue()) {
+        Tension payingTension = attackingPlayer.getTension(payingPlayer);
+        Tension targetTension = attackingPlayer.getTension(targetPlayer);
+        if (targetTension != null && 
+            payingTension.getValue() > targetTension.getValue()) {
             amount = 10000;
         } else {
             amount = 5000;
         }
-        amount += 20 * (attackingPlayer.getTension(payingPlayer).getValue() - attackingPlayer.getTension(targetPlayer)
-                .getValue());
+        amount += 20 * (payingTension.getValue() - targetTension.getValue());
 
         return Math.max(amount, 650);
     }
@@ -853,7 +948,7 @@ public class Game extends FreeColGameObject {
      *             stream.
      */
     protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
-            throws XMLStreamException {
+        throws XMLStreamException {
         // Start element:
         out.writeStartElement(getXMLElementTagName());
 
@@ -863,27 +958,35 @@ public class Game extends FreeColGameObject {
 
         out.writeAttribute("ID", getId());
         out.writeAttribute("turn", Integer.toString(getTurn().getNumber()));
-        if (currentPlayer != null) {
-            out.writeAttribute("currentPlayer", currentPlayer.getId());
-        }
+
+        writeAttribute(out, "currentPlayer", currentPlayer);
 
         if (toSavedGame) {
             out.writeAttribute("nextID", Integer.toString(nextId));
         }
 
         gameOptions.toXML(out);
-        if (gameVariables != null) {
-            gameVariables.toXML(out);
-        }
 
+        // serialize players
         Iterator<Player> playerIterator = getPlayerIterator();
         while (playerIterator.hasNext()) {
             Player p = playerIterator.next();
             p.toXML(out, player, showAll, toSavedGame);
         }
+        writeFreeColGameObject(getUnknownEnemy(), out, player, showAll, toSavedGame);
 
-        if (map != null) {
-            map.toXML(out, player, showAll, toSavedGame);
+        // serialize map
+        writeFreeColGameObject(map, out, player, showAll, toSavedGame);
+
+        // serialize messages
+        playerIterator = getPlayerIterator();
+        while (playerIterator.hasNext()) {
+            Player p = playerIterator.next();
+            if (this.isClientTrusted() || showAll || p.equals(player)) {
+                for (ModelMessage message : p.getModelMessages()) {
+                    message.toXML(out);
+                }
+            }
         }
 
         out.writeEndElement();
@@ -917,37 +1020,48 @@ public class Game extends FreeColGameObject {
 
         gameOptions = null;
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
-            if (in.getLocalName().equals(GameOptions.getXMLElementTagName())) {
+            if (in.getLocalName().equals(GameOptions.getXMLElementTagName()) || in.getLocalName().equals("game-options")) {
                 // Gets the game options:
                 if (gameOptions != null) {
                     gameOptions.readFromXML(in);
                 } else {
                     gameOptions = new GameOptions(in);
                 }
-            } else if (in.getLocalName().equals(GameVariables.getXMLElementTagName())) {
-                // Gets the game variables:
-                if (gameVariables != null) {
-                    gameVariables.readFromXML(in);
-                } else {
-                    gameVariables = new GameVariables(in);
-                }
             } else if (in.getLocalName().equals(Player.getXMLElementTagName())) {
                 Player player = (Player) getFreeColGameObject(in.getAttributeValue(null, "ID"));
                 if (player != null) {
                     player.readFromXML(in);
                 } else {
                     player = new Player(this, in);
-                    players.add(player);
+                    if (player.getName().equals(Player.UNKNOWN_ENEMY)) {
+                        setUnknownEnemy(player);
+                    } else {
+                        players.add(player);
+                    }
                 }
             } else if (in.getLocalName().equals(Map.getXMLElementTagName())) {
-                map = (Map) getFreeColGameObject(in.getAttributeValue(null, "ID"));
+                String mapId = in.getAttributeValue(null, "ID");
+                map = (Map) getFreeColGameObject(mapId);
                 if (map != null) {
                     map.readFromXML(in);
                 } else {
-                    map = new Map(this, in);
+                    map = new Map(this, mapId);
+                    map.readFromXML(in);
                 }
+            } else if (in.getLocalName().equals(ModelMessage.getXMLElementTagName())) {
+                ModelMessage message = new ModelMessage();
+                message.readFromXML(in, this);
+            } else {
+                logger.warning("Unknown tag: " + in.getLocalName() + " loading game");
+                in.nextTag();
             }
         }
+        // sanity check: we should be on the closing tag
+        if (!in.getLocalName().equals(Game.getXMLElementTagName())) {
+            logger.warning("Error parsing xml: expecting closing tag </" + Game.getXMLElementTagName() + "> "+
+                           "found instead: " +in.getLocalName());
+        }
+        
         if (gameOptions == null) {
             gameOptions = new GameOptions();
         }
@@ -961,4 +1075,62 @@ public class Game extends FreeColGameObject {
     public static String getXMLElementTagName() {
         return "game";
     }
+    
+    /**
+     * Need to overwrite behavior of equals inherited from FreeColGameObject,
+     * since two games are not the same if the have the same id.
+     */
+    public boolean equals(Object o) {
+        return this == o;
+    }
+
+    /**
+     * Returns an increasing number that can be used when creating players.
+     * @return The next player index
+     */
+    public int getNextPlayerIndex() {
+        return playerIndex++;
+    }
+    
+    /**
+     * Computes how many objects of each class have been created, 
+     * to track memory leaks over time
+     */
+    public HashMap<String, Long> getGameStatistics() {
+        
+        HashMap<String, Long> map = new HashMap<String, Long>();
+        Iterator<FreeColGameObject> iter = this.getFreeColGameObjectIterator();
+        map.put("disposed", new Long(0));
+        while (iter.hasNext()) {
+            FreeColGameObject obj = iter.next();
+            String className = obj.getClass().getSimpleName();
+            if (map.containsKey(className)) {
+                Long count = map.get(className);
+                count++;
+                map.put(className, count);
+            } else {
+                Long count = new Long(1);
+                map.put(className, count);
+            }
+            if (obj.isDisposed()) {
+                Long count = map.get("disposed");
+                count++;
+                map.put("disposed", count);
+            }
+        }
+        
+        return map;
+    }
+    
+    public HashMap<String, Long> getMemoryStatistics() {
+        System.gc();
+        long free = Runtime.getRuntime().freeMemory()/(1024*1024);
+        long total = Runtime.getRuntime().totalMemory()/(1024*1024);
+        long max = Runtime.getRuntime().maxMemory()/(1024*1024);
+        HashMap<String, Long> map = new HashMap<String, Long>();
+        map.put(Messages.message("menuBar.debug.memoryManager.free"), new Long(free));
+        map.put(Messages.message("menuBar.debug.memoryManager.total"), new Long(total));
+        map.put(Messages.message("menuBar.debug.memoryManager.max"), new Long(max));
+        return map;
+    }
 }
