@@ -31,7 +31,7 @@ import javax.xml.stream.XMLStreamWriter;
  * One of the items a DiplomaticTrade consists of.
  *
  */
-public abstract class TradeItem extends PersistentObject {
+public abstract class TradeItem extends FreeColObject {
 
     /**
      * The game this TradeItem belongs to.
@@ -48,12 +48,6 @@ public abstract class TradeItem extends PersistentObject {
      */
     private Player destination;
 
-    /**
-     * The ID, used to get a name, etc.
-     */
-    private String id;
-
-        
     /**
      * Creates a new <code>TradeItem</code> instance.
      *
@@ -64,7 +58,7 @@ public abstract class TradeItem extends PersistentObject {
      */
     public TradeItem(Game game, String id, Player source, Player destination) {
         this.game = game;
-        this.id = id;
+        setId(id);
         this.source = source;
         this.destination = destination;
     }
@@ -79,24 +73,6 @@ public abstract class TradeItem extends PersistentObject {
         this.game = game;
     }
 
-    /**
-     * Get the <code>ID</code> value.
-     *
-     * @return a <code>String</code> value
-     */
-    public final String getId() {
-        return id;
-    }
-
-    /**
-     * Set the <code>ID</code> value.
-     *
-     * @param newID The new ID value.
-     */
-    public final void setId(final String newID) {
-        this.id = newID;
-    }
-
     /**
      * Get the <code>Game</code> value.
      *
@@ -181,7 +157,7 @@ public abstract class TradeItem extends PersistentObject {
      *      during parsing.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
-        this.id = in.getAttributeValue(null, "ID");
+        setId(in.getAttributeValue(null, "ID"));
         String sourceID = in.getAttributeValue(null, "source");
         this.source = (Player) game.getFreeColGameObject(sourceID);
         String destinationID = in.getAttributeValue(null, "destination");
@@ -192,21 +168,12 @@ public abstract class TradeItem extends PersistentObject {
      * This method writes an XML-representation of this object to
      * the given stream.
      * 
-     * <br><br>
-     * 
-     * Only attributes visible to the given <code>Player</code> will 
-     * be added to that representation if <code>showAll</code> is
-     * set to <code>false</code>.
-     *  
      * @param out The target stream.
-     * @param player The <code>Player</code> this XML-representation 
-     *      should be made for, or <code>null</code> if
-     *      <code>showAll == true</code>.
      * @throws XMLStreamException if there are any problems writing
      *      to the stream.
      */
-    public void toXML(XMLStreamWriter out, Player player) throws XMLStreamException {
-        out.writeAttribute("ID", this.id);
+    public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
+        out.writeAttribute("ID", getId());
         out.writeAttribute("source", this.source.getId());
         out.writeAttribute("destination", this.destination.getId());
     }
