@@ -27,6 +27,9 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.model.Player;
+
 import org.w3c.dom.Element;
 
 /**
@@ -34,9 +37,6 @@ import org.w3c.dom.Element;
  */
 public class TradeRoute extends FreeColGameObject implements Cloneable, Ownable {
 
-
-
-
     public static final TradeRoute NO_TRADE_ROUTE = new TradeRoute();
 
     // private static final Logger logger =
@@ -65,19 +65,43 @@ public class TradeRoute extends FreeColGameObject implements Cloneable, Ownable
      */
     private ArrayList<Stop> stops = new ArrayList<Stop>();
 
-    private TradeRoute() {};
+    /**
+     * Creates a new <code>TradeRoute</code> instance.
+     *
+     */
+    private TradeRoute() {}
 
+    /**
+     * Creates a new <code>TradeRoute</code> instance.
+     *
+     * @param game a <code>Game</code> value
+     * @param name a <code>String</code> value
+     * @param player a <code>Player</code> value
+     */
     public TradeRoute(Game game, String name, Player player) {
         super(game);
         this.name = name;
         this.owner = player;
     }
 
+    /**
+     * Creates a new <code>TradeRoute</code> instance.
+     *
+     * @param game a <code>Game</code> value
+     * @param in a <code>XMLStreamReader</code> value
+     * @exception XMLStreamException if an error occurs
+     */
     public TradeRoute(Game game, XMLStreamReader in) throws XMLStreamException {
         super(game, in);
         readFromXML(in);
     }
 
+    /**
+     * Creates a new <code>TradeRoute</code> instance.
+     *
+     * @param game a <code>Game</code> value
+     * @param e an <code>Element</code> value
+     */
     public TradeRoute(Game game, Element e) {
         super(game, e);
         readFromXMLElement(e);
@@ -228,7 +252,7 @@ public class TradeRoute extends FreeColGameObject implements Cloneable, Ownable
 
         private String locationId;
 
-        private ArrayList<Integer> cargo = new ArrayList<Integer>();
+        private ArrayList<GoodsType> cargo = new ArrayList<GoodsType>();
 
         /**
          * Whether the stop has been modified. This is of interest only to the
@@ -248,13 +272,14 @@ public class TradeRoute extends FreeColGameObject implements Cloneable, Ownable
          */
         private Stop(Stop other) {
             this.locationId = other.locationId;
-            this.cargo = new ArrayList<Integer>(other.cargo);
+            this.cargo = new ArrayList<GoodsType>(other.cargo);
         }
 
         private Stop(XMLStreamReader in) throws XMLStreamException {
             locationId = in.getAttributeValue(null, "location");
-            for (int cargo : readFromArrayElement("cargo", in, new int[0])) {
-                addCargo(cargo);
+            List<GoodsType> goodsList = FreeCol.getSpecification().getGoodsTypeList();
+            for (int cargoIndex : readFromArrayElement("cargo", in, new int[0])) {
+                addCargo(goodsList.get(cargoIndex));
             }
             in.nextTag();
         }
@@ -293,8 +318,8 @@ public class TradeRoute extends FreeColGameObject implements Cloneable, Ownable
          * @return a cloned <code>ArrayList<Integer></code> value
          */
         @SuppressWarnings("unchecked")
-        public final ArrayList<Integer> getCargo() {
-            return (ArrayList<Integer>) cargo.clone();
+        public final ArrayList<GoodsType> getCargo() {
+            return (ArrayList<GoodsType>) cargo.clone();
         }
 
         /**
@@ -302,15 +327,21 @@ public class TradeRoute extends FreeColGameObject implements Cloneable, Ownable
          * 
          * @param cargo and arraylist of cargo values.
          */
-        public final void setCargo(ArrayList<Integer> cargo) {
+        public final void setCargo(ArrayList<GoodsType> cargo) {
             this.cargo.clear();
             this.cargo.addAll(cargo);
         }
 
-        public void addCargo(int newCargo) {
+        public void addCargo(GoodsType newCargo) {
             cargo.add(newCargo);
         }
 
+        /*
+          public void addCargo(int newCargoIndex) {
+          cargo.add(FreeCol.getSpecification().getGoodsType(newCargoIndex));
+          }
+        */
+
         public String toString() {
             Location l = getLocation();
             return l != null ? l.getLocationName() : locationId;
@@ -319,18 +350,18 @@ public class TradeRoute extends FreeColGameObject implements Cloneable, Ownable
         public void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
             out.writeStartElement(getStopXMLElementTagName());
             out.writeAttribute("location", this.locationId);
-            int[] cargoArray = new int[cargo.size()];
-            for (int index = 0; index < cargoArray.length; index++) {
-                cargoArray[index] = cargo.get(index).intValue();
+            int[] cargoIndexArray = new int[cargo.size()];
+            for (int index = 0; index < cargoIndexArray.length; index++) {
+                cargoIndexArray[index] = cargo.get(index).getIndex();
             }
-            toArrayElement("cargo", cargoArray, out);
+            toArrayElement("cargo", cargoIndexArray, out);
             out.writeEndElement();
         }
     }
 
 
-    public void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
-            throws XMLStreamException {
+    protected void toXMLImpl(XMLStreamWriter out, Player player, boolean showAll, boolean toSavedGame)
+        throws XMLStreamException {
         // Start element:
         out.writeStartElement(getXMLElementTagName());
 
@@ -353,15 +384,19 @@ public class TradeRoute extends FreeColGameObject implements Cloneable, Ownable
         setId(in.getAttributeValue(null, "ID"));
         setName(in.getAttributeValue(null, "name"));
         String ownerID = in.getAttributeValue(null, "owner");
-        if (ownerID.equals("unknown")) {
-            owner = Game.unknownEnemy;
-            // TODO: make this disappear
-        } else if (getGame() != null) {
-            owner = (Player) getGame().getFreeColGameObject(ownerID);
-            if (owner == null) {
-                owner = new Player(getGame(), in.getAttributeValue(null, "owner"));
+        
+        Game game = getGame();
+        if (game != null){
+            if (ownerID.equals(Player.UNKNOWN_ENEMY)) {
+                owner = game.getUnknownEnemy(); 
+            } else {
+                owner = (Player) getGame().getFreeColGameObject(ownerID);
+                if (owner == null) {
+                    owner = new Player(getGame(), in.getAttributeValue(null, "owner"));
+                }
             }
         }
+        
         while (in.nextTag() != XMLStreamConstants.END_ELEMENT) {
             if (getStopXMLElementTagName().equals(in.getLocalName())) {
                 stops.add(new Stop(in));
