@@ -78,6 +78,15 @@ public class LanguageOption extends AbstractOption {
 
 
     private Language value;
+    
+    /**
+     * Creates a new  <code>IntegerOption</code>.
+     * @param in The <code>XMLStreamReader</code> containing the data. 
+     */
+     public LanguageOption(XMLStreamReader in) throws XMLStreamException {
+         super(NO_ID);
+         readFromXML(in);
+     }   
 
     /**
      * Creates a new <code>LanguageOption</code>.
@@ -86,10 +95,7 @@ public class LanguageOption extends AbstractOption {
      *           found in an {@link OptionGroup}.
      */
     public LanguageOption(String id) {
-        super(id, null);
-        if (languages.size() == 0) {
-            prepareLanguages();
-        }
+        this(id, null);
     }
 
     public LanguageOption(String id, OptionGroup optionGroup) {
@@ -97,6 +103,7 @@ public class LanguageOption extends AbstractOption {
         if (languages.size() == 0) {
             prepareLanguages();
         }
+        value = DEFAULT;
     }
 
 
@@ -177,7 +184,7 @@ public class LanguageOption extends AbstractOption {
      * @return The Locale.
      */
     public static Locale getLocale(String languageID) {
-        if (languageID.equals(AUTO)) {
+        if (languageID == null || AUTO.equals(languageID)) {
             return Locale.getDefault();
         }
          
@@ -222,12 +229,13 @@ public class LanguageOption extends AbstractOption {
      */
     protected void toXMLImpl(XMLStreamWriter out) throws XMLStreamException {
         // Start element:
-        out.writeStartElement(getId());
+        out.writeStartElement(getXMLElementTagName());
 
+        out.writeAttribute("id", getId());
         out.writeAttribute("value", getValue().getKey());
 
         out.writeEndElement();
-    }
+     }
     
     /**
      * Initialize this object from an XML-representation of this object.
@@ -236,18 +244,21 @@ public class LanguageOption extends AbstractOption {
      *      during parsing.
      */
     protected void readFromXMLImpl(XMLStreamReader in) throws XMLStreamException {
+        final String id = in.getAttributeValue(null, "id");
         findLanguages();
-        final Language oldValue = this.value;
         
+        if (id == null && getId().equals("NO_ID")){
+            throw new XMLStreamException("invalid <" + getXMLElementTagName() + "> tag : no id attribute found.");
+        } else if(getId() == NO_ID) {
+            setId(id);
+        }
+
         Language newValue = languages.get(in.getAttributeValue(null, "value"));
         if (newValue == null) {
             newValue = languages.get(AUTO);
         }
+        setValue(newValue);
         in.nextTag();
-        
-        if (!newValue.equals(oldValue)) {
-            firePropertyChange("value", oldValue, value);
-        }
     }
 
 
