@@ -164,23 +164,6 @@ public abstract class OptionMap extends OptionGroup {
         }
     }
     
-    /**
-     * Sets the boolean value of an option.
-     *
-     * @param id The id of the option.
-     * @param value The value.
-     * @exception IllegalArgumentException If there is no boolean
-     *            value associated with the specified option.
-     * @exception NullPointerException if the given <code>Option</code> does not exist.
-     */
-    public void setBoolean(String id, boolean value) {
-        try {
-            ((BooleanOption) values.get(id)).setValue(value);
-        } catch (ClassCastException e) {
-            throw new IllegalArgumentException("No boolean value associated with the specified option.");
-        }
-    }
-    
     /**
      * Gets the <code>File</code> specified by an option.
      *
@@ -217,6 +200,10 @@ public abstract class OptionMap extends OptionGroup {
         }
     }
 
+    public void putOption(Option option) {
+        values.put(option.getId(), option);
+    }
+
     /**
      * Creates a <code>XMLStreamReader</code> for reading the given file.
      * Compression is automatically detected.
@@ -398,4 +385,24 @@ public abstract class OptionMap extends OptionGroup {
         throw new UnsupportedOperationException();
     }
 
+    /**
+     * Allow a File to be set in other ways, as from the user using the Client.
+     * 
+     * @param id
+     * @param newFileValue
+     */
+    public void setFile(String id, File newFileValue) {
+    	if( id == null || id.trim().length() == 0 )
+    		throw new IllegalArgumentException("Requires an ID");
+    	if( newFileValue == null )
+    		throw new IllegalArgumentException("Requires a File parameter");
+    	if( values.get(id) == null )
+    		throw new IllegalArgumentException("No option with ID=["+ id +"]");
+    	
+        try {
+            ((FileOption) values.get(id)).setValue(newFileValue);
+        } catch (ClassCastException e) {
+            throw new IllegalArgumentException("No File associated with option ["+ id +"].");
+        }
+    }
 }
