@@ -20,7 +20,6 @@
 package net.sf.freecol.server;
 
 import java.awt.Color;
-import java.io.BufferedInputStream;
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
@@ -34,14 +33,17 @@ import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.security.SecureRandom;
 import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Comparator;
+import java.util.Date;
 import java.util.Iterator;
+import java.util.List;
 import java.util.Random;
 import java.util.Timer;
 import java.util.TimerTask;
-import java.util.Vector;
+import java.util.jar.JarEntry;
+import java.util.jar.JarOutputStream;
 import java.util.logging.Logger;
-import java.util.zip.GZIPInputStream;
-import java.util.zip.GZIPOutputStream;
 
 import javax.xml.stream.XMLInputFactory;
 import javax.xml.stream.XMLOutputFactory;
@@ -53,10 +55,19 @@ import javax.xml.stream.XMLStreamWriter;
 import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.FreeColException;
 import net.sf.freecol.common.PseudoRandom;
+import net.sf.freecol.common.Specification;
+import net.sf.freecol.common.io.FreeColSavegameFile;
 import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.Game;
+import net.sf.freecol.common.model.GameOptions;
+import net.sf.freecol.common.model.HighScore;
+import net.sf.freecol.common.model.Nation;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.UnitType;
+import net.sf.freecol.common.model.Player.PlayerType;
+import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.common.networking.NoRouteToServerException;
@@ -69,6 +80,7 @@ import net.sf.freecol.server.control.PreGameController;
 import net.sf.freecol.server.control.PreGameInputHandler;
 import net.sf.freecol.server.control.ServerModelController;
 import net.sf.freecol.server.control.UserConnectionHandler;
+import net.sf.freecol.server.generator.IMapGenerator;
 import net.sf.freecol.server.generator.MapGenerator;
 import net.sf.freecol.server.model.ServerModelObject;
 import net.sf.freecol.server.model.ServerPlayer;
@@ -87,41 +99,40 @@ import org.w3c.dom.Element;
  */
 public final class FreeColServer {
 
-
-
-    private static Logger logger = Logger.getLogger(FreeColServer.class.getName());
-
-    private static final boolean DISABLE_SAVEGAME_COMPRESSION = false;
+    private static final Logger logger = Logger.getLogger(FreeColServer.class.getName());
 
     private static final int META_SERVER_UPDATE_INTERVAL = 60000;
 
+    private static final int NUMBER_OF_HIGH_SCORES = 10;
+    private static final String HIGH_SCORE_FILE = "HighScores.xml";
+
     /** Constant for storing the state of the game. */
-    public static final int STARTING_GAME = 0, IN_GAME = 1, ENDING_GAME = 2;
+    public static enum GameState {STARTING_GAME, IN_GAME, ENDING_GAME}
 
     /** Stores the current state of the game. */
-    private int gameState = STARTING_GAME;
+    private GameState gameState = GameState.STARTING_GAME;
 
     // Networking:
     private Server server;
 
     // Control:
-    private UserConnectionHandler userConnectionHandler;
+    private final UserConnectionHandler userConnectionHandler;
 
-    private PreGameController preGameController;
+    private final PreGameController preGameController;
 
-    private PreGameInputHandler preGameInputHandler;
+    private final PreGameInputHandler preGameInputHandler;
 
-    private InGameInputHandler inGameInputHandler;
+    private final InGameInputHandler inGameInputHandler;
 
-    private ServerModelController modelController;
+    private final ServerModelController modelController;
 
-    private InGameController inGameController;
+    private final InGameController inGameController;
 
     private Game game;
 
     private AIMain aiMain;
 
-    private MapGenerator mapGenerator;
+    private IMapGenerator mapGenerator;
 
     private boolean singleplayer;
 
@@ -135,9 +146,28 @@ public final class FreeColServer {
     /** The name of this server. */
     private String name;
 
+    private int numberOfPlayers;
+    private int advantages;
+    private boolean additionalNations;
+
     /** The provider for random numbers */
     private final ServerPseudoRandom _pseudoRandom = new ServerPseudoRandom();
 
+    /** Did the integrity check succeed */
+    private boolean integrity = false;
+
+    /**
+     * The high scores on this server.
+     */
+    private List<HighScore> highScores = null;
+
+
+    public static final Comparator<HighScore> highScoreComparator = new Comparator<HighScore>() {
+        public int compare(HighScore score1, HighScore score2) {
+            return score2.getScore() - score1.getScore();
+        }
+    };
+
 
     /**
      * Starts a new server in a specified mode and with a specified port.
@@ -158,13 +188,31 @@ public final class FreeColServer {
      *             will be logged by this class).
      * 
      */
-    public FreeColServer(boolean publicServer, boolean singleplayer, int port, String name) throws IOException, NoRouteToServerException {
+    
+    public FreeColServer(boolean publicServer, boolean singleplayer, int port, String name)
+        throws IOException, NoRouteToServerException {
+        this(publicServer, singleplayer, port, name, 4, 0, false);
+    }
+
+    public FreeColServer(boolean publicServer, boolean singleplayer, int port, String name, int players,
+                         int advantages, boolean additionalNations)
+        throws IOException, NoRouteToServerException {
         this.publicServer = publicServer;
         this.singleplayer = singleplayer;
         this.port = port;
         this.name = name;
+        this.numberOfPlayers = players;
+        this.additionalNations = additionalNations;
+        this.advantages = advantages;
+
         modelController = new ServerModelController(this);
         game = new Game(modelController);
+        if (additionalNations) {
+            game.setVacantNations(new ArrayList<Nation>(FreeCol.getSpecification().getEuropeanNations()));
+        } else {
+            game.setVacantNations(new ArrayList<Nation>(FreeCol.getSpecification().getClassicNations()));
+        }
+        game.setMaximumPlayers(players);
         mapGenerator = new MapGenerator();
         userConnectionHandler = new UserConnectionHandler(this);
         preGameController = new PreGameController(this);
@@ -217,6 +265,7 @@ public final class FreeColServer {
         preGameInputHandler = new PreGameInputHandler(this);
         inGameInputHandler = new InGameInputHandler(this);
         inGameController = new InGameController(this);
+
         try {
             server = new Server(this, port);
             server.start();
@@ -235,6 +284,10 @@ public final class FreeColServer {
             fe.initCause(e);
             throw fe;
         }
+
+        // Apply the difficulty level
+        Specification.getSpecification().applyDifficultyLevel(game.getGameOptions().getInteger(GameOptions.DIFFICULTY));        
+
         updateMetaServer(true);
         startMetaServerUpdateThread();
     }
@@ -271,18 +324,35 @@ public final class FreeColServer {
         }
         final ServerPlayer p = (ServerPlayer) getGame().getPlayerByName(username);
         synchronized (p) {
-            Unit theFlyingDutchman = new Unit(game, p.getEntryLocation(), p, Unit.FLYING_DUTCHMAN, Unit.ACTIVE);
-            new Unit(game, theFlyingDutchman, p, Unit.REVENGER, Unit.SENTRY);
-            p.setDead(false);
-            p.setColor(Color.BLACK);
-            Element updateElement = Message.createNewRootElement("update");
-            updateElement.appendChild(((FreeColGameObject) p.getEntryLocation()).toXMLElement(p, updateElement
-                    .getOwnerDocument()));
-            updateElement.appendChild(p.toXMLElement(p, updateElement.getOwnerDocument()));
-            try {
-                p.getConnection().send(updateElement);
-            } catch (IOException e) {
-                logger.warning("Could not send update");
+            List<UnitType> undeads = FreeCol.getSpecification().getUnitTypesWithAbility("model.ability.undead");
+            ArrayList<UnitType> navalUnits = new ArrayList<UnitType>();
+            ArrayList<UnitType> landUnits = new ArrayList<UnitType>();
+            for (UnitType undead : undeads) {
+                if (undead.hasAbility("model.ability.navalUnit")) {
+                    navalUnits.add(undead);
+                } else if (undead.getId().equals("model.unit.revenger")) { // TODO: softcode this
+                    landUnits.add(undead);
+                }
+            }
+            if (navalUnits.size() > 0) {
+                UnitType navalType = navalUnits.get(getPseudoRandom().nextInt(navalUnits.size()));
+                Unit theFlyingDutchman = new Unit(game, p.getEntryLocation(), p, navalType, UnitState.ACTIVE);
+                if (landUnits.size() > 0) {
+                    UnitType landType = landUnits.get(getPseudoRandom().nextInt(landUnits.size()));
+                    new Unit(game, theFlyingDutchman, p, landType, UnitState.SENTRY);
+                }
+                p.setDead(false);
+                p.setPlayerType(PlayerType.UNDEAD);
+                p.setColor(Color.BLACK);
+                Element updateElement = Message.createNewRootElement("update");
+                updateElement.appendChild(((FreeColGameObject) p.getEntryLocation()).toXMLElement(p, updateElement
+                        .getOwnerDocument()));
+                updateElement.appendChild(p.toXMLElement(p, updateElement.getOwnerDocument()));
+                try {
+                    p.getConnection().sendAndWait(updateElement);
+                } catch (IOException e) {
+                    logger.warning("Could not send update");
+                }
             }
         }
     }
@@ -293,7 +363,7 @@ public final class FreeColServer {
      * 
      * @return The <code>MapGenerator</code>.
      */
-    public MapGenerator getMapGenerator() {
+    public IMapGenerator getMapGenerator() {
         return mapGenerator;
     }
     
@@ -303,7 +373,7 @@ public final class FreeColServer {
      * 
      * @param mapGenerator The <code>MapGenerator</code>.
      */
-    public void setMapGenerator(MapGenerator mapGenerator) {
+    public void setMapGenerator(IMapGenerator mapGenerator) {
         this.mapGenerator = mapGenerator;
     }
 
@@ -333,6 +403,33 @@ public final class FreeColServer {
         this.name = name;
     }
 
+    /**
+     * Return the type of advantages used.
+     *
+     * @return a <code>int</code> value
+     */
+    public int getAdvantages() {
+        return advantages;
+    }
+
+    /**
+     * Describe <code>getAdditionalNations</code> method here.
+     *
+     * @return a <code>boolean</code> value
+     */
+    public boolean getAdditionalNations() {
+        return additionalNations;
+    }
+
+    /**
+     * Returns the number of players.
+     *
+     * @return an <code>int</code> value
+     */
+    public int getNumberOfPlayers() {
+        return numberOfPlayers;
+    }
+
     /**
      * Sends information about this server to the meta-server. The information
      * is only sent if <code>public == true</code>.
@@ -348,7 +445,7 @@ public final class FreeColServer {
         }
         Connection mc;
         try {
-            mc = new Connection(FreeCol.META_SERVER_ADDRESS, FreeCol.META_SERVER_PORT, null);
+            mc = new Connection(FreeCol.META_SERVER_ADDRESS, FreeCol.META_SERVER_PORT, null, FreeCol.SERVER_THREAD);
         } catch (IOException e) {
             logger.warning("Could not connect to meta-server.");
             return;
@@ -360,7 +457,7 @@ public final class FreeColServer {
             } else {
                 element = Message.createNewRootElement("update");
             }
-            // TODO: Add possibillity of choosing a name:
+            // TODO: Add possibility of choosing a name:
             if (name != null) {
                 element.setAttribute("name", name);
             } else {
@@ -370,9 +467,9 @@ public final class FreeColServer {
             element.setAttribute("port", Integer.toString(port));
             element.setAttribute("slotsAvailable", Integer.toString(getSlotsAvailable()));
             element.setAttribute("currentlyPlaying", Integer.toString(getNumberOfLivingHumanPlayers()));
-            element.setAttribute("isGameStarted", Boolean.toString(gameState != STARTING_GAME));
+            element.setAttribute("isGameStarted", Boolean.toString(gameState != GameState.STARTING_GAME));
             element.setAttribute("version", FreeCol.getVersion());
-            element.setAttribute("gameState", Integer.toString(getGameState()));
+            element.setAttribute("gameState", Integer.toString(getGameState().ordinal()));
             Element reply = mc.ask(element);
             if (reply != null && reply.getTagName().equals("noRouteToServer")) {
                 throw new NoRouteToServerException();
@@ -401,7 +498,7 @@ public final class FreeColServer {
         }
         Connection mc;
         try {
-            mc = new Connection(FreeCol.META_SERVER_ADDRESS, FreeCol.META_SERVER_PORT, null);
+            mc = new Connection(FreeCol.META_SERVER_ADDRESS, FreeCol.META_SERVER_PORT, null, FreeCol.SERVER_THREAD);
         } catch (IOException e) {
             logger.warning("Could not connect to meta-server.");
             return;
@@ -431,7 +528,7 @@ public final class FreeColServer {
      *         includes european players currently controlled by the AI.
      */
     public int getSlotsAvailable() {
-        Vector<Player> players = game.getPlayers();
+        List<Player> players = game.getPlayers();
         int n = game.getMaximumPlayers();
         for (int i = 0; i < players.size(); i++) {
             ServerPlayer p = (ServerPlayer) players.get(i);
@@ -451,7 +548,7 @@ public final class FreeColServer {
      * @return The number.
      */
     public int getNumberOfLivingHumanPlayers() {
-        Vector<Player> players = game.getPlayers();
+        List<Player> players = game.getPlayers();
         int n = 0;
         for (int i = 0; i < players.size(); i++) {
             if (!((ServerPlayer) players.get(i)).isAI() && !((ServerPlayer) players.get(i)).isDead()
@@ -484,19 +581,17 @@ public final class FreeColServer {
     public void saveGame(File file, String username) throws IOException {
         final Game game = getGame();
         XMLOutputFactory xof = XMLOutputFactory.newInstance();
+        JarOutputStream fos = null;
         try {
             XMLStreamWriter xsw;
-            GZIPOutputStream gzip;
-            if (DISABLE_SAVEGAME_COMPRESSION) {
-                // No compression
-                xsw = xof.createXMLStreamWriter(new FileOutputStream(file));
-            } else {
-                // Compression
-                gzip = new GZIPOutputStream(new FileOutputStream(file));
-                xsw = xof.createXMLStreamWriter(gzip);
-            }
+            fos = new JarOutputStream(new FileOutputStream(file));
+            fos.putNextEntry(new JarEntry(file.getName().split("\\.")[0] + "/" + FreeColSavegameFile.SAVEGAME_FILE));
+            xsw = xof.createXMLStreamWriter(fos, "UTF-8");
+
             xsw.writeStartDocument("UTF-8", "1.0");
+            xsw.writeComment("Game version: "+FreeCol.getRevision());
             xsw.writeStartElement("savedGame");
+            
             // Add the attributes:
             xsw.writeAttribute("owner", username);
             xsw.writeAttribute("publicServer", Boolean.toString(publicServer));
@@ -521,9 +616,6 @@ public final class FreeColServer {
             }
             xsw.writeEndElement();
             xsw.writeEndDocument();
-            if (!DISABLE_SAVEGAME_COMPRESSION) {
-                gzip.finish();
-            }
             xsw.flush();
             xsw.close();
         } catch (XMLStreamException e) {
@@ -536,6 +628,14 @@ public final class FreeColServer {
             e.printStackTrace(new PrintWriter(sw));
             logger.warning(sw.toString());
             throw new IOException(e.toString());
+        } finally {
+            try {
+                if (fos != null) {
+                    fos.close();
+                }
+            } catch (IOException e) {
+                // do nothing
+            }
         }
     }
 
@@ -543,30 +643,27 @@ public final class FreeColServer {
      * Creates a <code>XMLStreamReader</code> for reading the given file.
      * Compression is automatically detected.
      * 
-     * @param file The file to be read.
+     * @param fis The file to be read.
      * @return The <code>XMLStreamReader</code>.
      * @exception IOException if thrown while loading the game or if a
      *                <code>XMLStreamException</code> have been thrown by the
      *                parser.
      */
-    public static XMLStreamReader createXMLStreamReader(File file) throws IOException {
-        InputStream in = new BufferedInputStream(new FileInputStream(file));
-        // Automatically detect compression:
-        in.mark(10);
-        byte[] buf = new byte[5];
-        in.read(buf, 0, 5);
-        in.reset();
-        if (!(new String(buf)).equals("<?xml")) {
-            in = new BufferedInputStream(new GZIPInputStream(in));
-        }
-        XMLInputFactory xif = XMLInputFactory.newInstance();
+    public static XMLStreamReader createXMLStreamReader(FreeColSavegameFile fis) throws IOException {
         try {
+            InputStream in = fis.getSavegameInputStream();
+            XMLInputFactory xif = XMLInputFactory.newInstance();        
             return xif.createXMLStreamReader(in);
         } catch (XMLStreamException e) {
             StringWriter sw = new StringWriter();
             e.printStackTrace(new PrintWriter(sw));
             logger.warning(sw.toString());
             throw new IOException("XMLStreamException.");
+        } catch (NullPointerException e) {
+            StringWriter sw = new StringWriter();
+            e.printStackTrace(new PrintWriter(sw));
+            logger.warning(sw.toString());
+            throw new NullPointerException("NullPointerException.");
         }
     }
 
@@ -584,8 +681,10 @@ public final class FreeColServer {
      */
     public String loadGame(File file) throws IOException, FreeColException {
         boolean doNotLoadAI = false;
+        FreeColSavegameFile fis = null;
         try {
-            XMLStreamReader xsr = createXMLStreamReader(file);
+            fis = new FreeColSavegameFile(file);
+            XMLStreamReader xsr = createXMLStreamReader(fis);
             xsr.nextTag();
             final String version = xsr.getAttributeValue(null, "version");
             if (!Message.getFreeColProtocolVersion().equals(version)) {
@@ -618,8 +717,8 @@ public final class FreeColServer {
                     game = new Game(null, getModelController(), xsr, serverObjects
                             .toArray(new FreeColGameObject[0]));
                     game.setCurrentPlayer(null);
-                    gameState = IN_GAME;
-                    game.checkIntegrity();
+                    gameState = GameState.IN_GAME;
+                    integrity = game.checkIntegrity();
                 } else if (xsr.getLocalName().equals(AIMain.getXMLElementTagName())) {
                     if (doNotLoadAI) {
                         aiMain = new AIMain(this);
@@ -656,7 +755,7 @@ public final class FreeColServer {
                             new AIInGameInputHandler(this, player, aiMain));
                     aiConnection.setOutgoingMessageHandler(theConnection);
                     theConnection.setOutgoingMessageHandler(aiConnection);
-                    getServer().addConnection(theConnection, -1);
+                    getServer().addDummyConnection(theConnection);
                     player.setConnection(theConnection);
                     player.setConnected(true);
                 }
@@ -679,6 +778,10 @@ public final class FreeColServer {
             e.printStackTrace(new PrintWriter(sw));
             logger.warning(sw.toString());
             throw new IOException(e.toString());
+        } finally {
+            if (fis != null) {
+                fis.close();
+            }
         }
     }
 
@@ -758,7 +861,7 @@ public final class FreeColServer {
      * @return The <code>Controller</code>.
      */
     public Controller getController() {
-        if (getGameState() == IN_GAME) {
+        if (getGameState() == GameState.IN_GAME) {
             return inGameController;
         } else {
             return preGameController;
@@ -803,10 +906,10 @@ public final class FreeColServer {
     }
 
     /**
-     * Gets the <code>Game</code> that is beeing played.
+     * Gets the <code>Game</code> that is being played.
      * 
      * @return The <code>Game</code> which is the main class of the game-model
-     *         beeing used in this game.
+     *         being used in this game.
      */
     public Game getGame() {
         return game;
@@ -838,7 +941,7 @@ public final class FreeColServer {
      * @return One of: {@link #STARTING_GAME}, {@link #IN_GAME} and
      *         {@link #ENDING_GAME}.
      */
-    public int getGameState() {
+    public GameState getGameState() {
         return gameState;
     }
 
@@ -848,7 +951,7 @@ public final class FreeColServer {
      * @param state The new state to be set. One of: {@link #STARTING_GAME},
      *            {@link #IN_GAME} and {@link #ENDING_GAME}.
      */
-    public void setGameState(int state) {
+    public void setGameState(GameState state) {
         gameState = state;
     }
 
@@ -861,6 +964,15 @@ public final class FreeColServer {
         return server;
     }
 
+    /**
+     * Gets the integrity check result.
+     *
+     * @return The integrity check result.
+     */
+    public boolean getIntegrity() {
+        return integrity;
+    }
+
     /**
      * Get the common pseudo random number generator for the server.
      * 
@@ -980,4 +1092,196 @@ public final class FreeColServer {
             }
         }
     }
+
+    /**
+     * Describe <code>sendUpdatedTileToAll</code> method here.
+     *
+     * @param newTile a <code>Tile</code> value
+     * @param player a <code>Player</code> value
+     */
+    public void sendUpdatedTileToAll(Tile newTile, Player player) {
+        // TODO: can Player be null?
+        for (Player enemy : getGame().getPlayers()) {
+            ServerPlayer enemyPlayer = (ServerPlayer) enemy;
+            if (player != null && player.equals(enemyPlayer) || enemyPlayer.getConnection() == null) {
+                continue;
+            }
+            try {
+                if (enemyPlayer.canSee(newTile)) {
+                    Element updateElement = Message.createNewRootElement("update");
+                    updateElement.appendChild(newTile.toXMLElement(enemyPlayer, updateElement.getOwnerDocument()));
+                    enemyPlayer.getConnection().sendAndWait(updateElement);
+                }
+            } catch (IOException e) {
+                logger.warning("Could not send message to: " + enemyPlayer.getName() + " with connection "
+                        + enemyPlayer.getConnection());
+            }
+        }
+    }
+
+
+    /**
+     * Adds a new AIPlayer to the Game.
+     *
+     * @param nation a <code>Nation</code> value
+     * @return a <code>ServerPlayer</code> value
+     */
+    public ServerPlayer addAIPlayer(Nation nation) {
+        String name = nation.getRulerName();
+        DummyConnection theConnection = 
+            new DummyConnection("Server connection - " + name, getInGameInputHandler());
+        ServerPlayer aiPlayer = 
+            new ServerPlayer(getGame(), name, false, true, null, theConnection, nation);
+        DummyConnection aiConnection = 
+            new DummyConnection("AI connection - " + name,
+                                new AIInGameInputHandler(this, aiPlayer, getAIMain()));
+            
+        aiConnection.setOutgoingMessageHandler(theConnection);
+        theConnection.setOutgoingMessageHandler(aiConnection);
+
+        getServer().addDummyConnection(theConnection);
+
+        getGame().addPlayer(aiPlayer);
+
+        // Send message to all players except to the new player:
+        Element addNewPlayer = Message.createNewRootElement("addPlayer");
+        addNewPlayer.appendChild(aiPlayer.toXMLElement(null, addNewPlayer.getOwnerDocument()));
+        getServer().sendToAll(addNewPlayer, theConnection);
+        return aiPlayer;
+    }
+
+    /**
+     * Get the <code>HighScores</code> value.
+     *
+     * @return a <code>List<HighScore></code> value
+     */
+    public List<HighScore> getHighScores() {
+        if (highScores == null) {
+            try {
+                loadHighScores();
+            } catch (Exception e) {
+                logger.warning(e.toString());
+                highScores = new ArrayList<HighScore>();
+            }
+        }
+        return highScores;
+    }
+
+    /**
+     * Adds a new high score for player and returns <code>true</code>
+     * if possible.
+     *
+     * @param player a <code>Player</code> value
+     * @param nationName a <code>String</code> value
+     * @return a <code>boolean</code> value
+     */
+    public boolean newHighScore(Player player) {
+        getHighScores();
+        if (!highScores.isEmpty() && player.getScore() <= highScores.get(highScores.size() - 1).getScore()) {
+            return false;
+        } else {
+            highScores.add(new HighScore(player, new Date()));
+            Collections.sort(highScores, highScoreComparator);
+            if (highScores.size() == NUMBER_OF_HIGH_SCORES) {
+                highScores.remove(NUMBER_OF_HIGH_SCORES - 1);
+            }
+            return true;
+        }
+    }
+
+    /**
+     * Saves high scores.
+     * 
+     * @throws IOException If a problem was encountered while trying to open,
+     *             write or close the file.
+     */
+    public void saveHighScores() throws IOException {
+        if (highScores == null || highScores.isEmpty()) {
+            return;
+        }
+        Collections.sort(highScores, highScoreComparator);
+        XMLOutputFactory xof = XMLOutputFactory.newInstance();
+        FileOutputStream fos = null;
+        try {
+            XMLStreamWriter xsw;
+            fos = new FileOutputStream(new File(FreeCol.getDataDirectory(), HIGH_SCORE_FILE));
+
+            xsw = xof.createXMLStreamWriter(fos, "UTF-8");
+            xsw.writeStartDocument("UTF-8", "1.0");
+            xsw.writeStartElement("highScores");
+            int count = 0;
+            for (HighScore score : highScores) {
+                score.toXML(xsw);
+                count++;
+                if (count == NUMBER_OF_HIGH_SCORES) {
+                    break;
+                }
+            }
+            xsw.writeEndElement();
+            xsw.writeEndDocument();
+            xsw.flush();
+            xsw.close();
+        } catch (XMLStreamException e) {
+            StringWriter sw = new StringWriter();
+            e.printStackTrace(new PrintWriter(sw));
+            logger.warning(sw.toString());
+            throw new IOException("XMLStreamException.");
+        } catch (Exception e) {
+            StringWriter sw = new StringWriter();
+            e.printStackTrace(new PrintWriter(sw));
+            logger.warning(sw.toString());
+            throw new IOException(e.toString());
+        } finally {
+            try {
+                if (fos != null) {
+                    fos.close();
+                }
+            } catch (IOException e) {
+                // do nothing
+            }
+        }
+    }
+
+    /**
+     * Loads high scores.
+     * 
+     * @throws IOException If a problem was encountered while trying to open,
+     *             read or close the file.
+     * @exception IOException if thrown while loading the game or if a
+     *                <code>XMLStreamException</code> have been thrown by the
+     *                parser.
+     * @exception FreeColException if the savegame contains incompatible data.
+     */
+    public void loadHighScores() throws IOException, FreeColException {
+        highScores = new ArrayList<HighScore>();
+        XMLInputFactory xif = XMLInputFactory.newInstance();
+        FileInputStream fis = null;
+        try {
+            fis = new FileInputStream(new File(FreeCol.getDataDirectory(), HIGH_SCORE_FILE));
+            XMLStreamReader xsr = xif.createXMLStreamReader(fis);
+            xsr.nextTag();
+            while (xsr.nextTag() != XMLStreamConstants.END_ELEMENT) {
+                if (xsr.getLocalName().equals("highScore")) {
+                    highScores.add(new HighScore(xsr));
+                }
+            }
+            xsr.close();
+            Collections.sort(highScores, highScoreComparator);
+        } catch (XMLStreamException e) {
+            StringWriter sw = new StringWriter();
+            e.printStackTrace(new PrintWriter(sw));
+            logger.warning(sw.toString());
+            throw new IOException("XMLStreamException.");
+        } catch (Exception e) {
+            StringWriter sw = new StringWriter();
+            e.printStackTrace(new PrintWriter(sw));
+            logger.warning(sw.toString());
+            throw new IOException(e.toString());
+        } finally {
+            if (fis != null) {
+                fis.close();
+            }
+        }
+    }
+
 }
