@@ -34,6 +34,7 @@ import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.server.ai.mission.BuildColonyMission;
 import net.sf.freecol.server.ai.mission.CashInTreasureTrainMission;
 import net.sf.freecol.server.ai.mission.DefendSettlementMission;
+import net.sf.freecol.server.ai.mission.IdleAtColonyMission;
 import net.sf.freecol.server.ai.mission.IndianBringGiftMission;
 import net.sf.freecol.server.ai.mission.IndianDemandMission;
 import net.sf.freecol.server.ai.mission.Mission;
@@ -100,9 +101,6 @@ public class AIUnit extends AIObject implements Transportable {
         super(aiMain, unit.getId());
 
         this.unit = unit;
-        if (unit == null) {
-            throw new NullPointerException("unit == null");
-        }
 
         mission = new UnitWanderHostileMission(aiMain, this);
     }
@@ -427,6 +425,8 @@ public class AIUnit extends AIObject implements Transportable {
                 mission = new ScoutingMission(getAIMain(), in);
             } else if (in.getLocalName().equals(CashInTreasureTrainMission.getXMLElementTagName())) {
                 mission = new CashInTreasureTrainMission(getAIMain(), in);
+            } else if (in.getLocalName().equals(IdleAtColonyMission.getXMLElementTagName())) {
+                mission = new IdleAtColonyMission(getAIMain(), in);
             } else {
                 logger.warning("Could not find mission-class for: " + in.getLocalName());
                 mission = new UnitWanderHostileMission(getAIMain(), this);
