@@ -26,13 +26,17 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.logging.Logger;
 
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.model.AbstractGoods;
+import net.sf.freecol.common.model.BuildableType;
 import net.sf.freecol.common.model.Building;
+import net.sf.freecol.common.model.BuildingType;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.ColonyTile;
 import net.sf.freecol.common.model.Game;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Tile;
-import net.sf.freecol.common.model.Unit;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
@@ -49,11 +53,13 @@ import org.w3c.dom.Element;
  * @see Colony
  */
 public class ColonyPlan {
+
     @SuppressWarnings("unused")
     private static final Logger logger = Logger.getLogger(ColonyPlan.class.getName());
 
-
-
+    // What is this supposed to be? Is it the maximum number of units
+    // per building?
+    private static final int MAX_LEVEL = 3;
 
     /**
      * The FreeColGameObject this AIObject contains AI-information for.
@@ -72,12 +78,11 @@ public class ColonyPlan {
      * @param colony The colony to make a <code>ColonyPlan</code> for.
      */
     public ColonyPlan(AIMain aiMain, Colony colony) {
+        if (colony == null) {
+            throw new IllegalArgumentException("Parameter 'colony' must not be 'null'.");
+        }
         this.aiMain = aiMain;
         this.colony = colony;
-
-        if (colony == null) {
-            throw new NullPointerException("colony == null");
-        }
     }
 
     /**
@@ -98,9 +103,8 @@ public class ColonyPlan {
      * 
      * @return The list of <code>WorkLocationPlan</code>s .
      */
-    @SuppressWarnings("unchecked")
     public List<WorkLocationPlan> getWorkLocationPlans() {
-        return (List<WorkLocationPlan>) workLocationPlans.clone();
+        return new ArrayList<WorkLocationPlan>(workLocationPlans);
     }
 
     /**
@@ -127,12 +131,42 @@ public class ColonyPlan {
      * @return An iterator containing all the <code>Buildable</code> sorted by
      *         priority (highest priority first).
      */
-    public Iterator<Integer> getBuildable() {
-        ArrayList<Integer> buildList = new ArrayList<Integer>();
+    public Iterator<BuildableType> getBuildable() {
+        List<BuildableType> buildList = new ArrayList<BuildableType>();
 
-        if (!colony.getBuilding(Building.DOCK).isBuilt() && !colony.isLandLocked()
-                && colony.getBuilding(Building.DOCK).canBuildNext()) {
-            buildList.add(Building.DOCK);
+        BuildingType docks = null;
+        BuildingType customHouse = null;
+        BuildingType carpenter = null;
+        BuildingType stables = null;
+        BuildingType stockade = null;
+        BuildingType armory = null;
+        BuildingType schoolhouse = null;
+        for (BuildingType type : FreeCol.getSpecification().getBuildingTypeList()) {
+            if (type.getUpgradesFrom() != null) {
+                continue;
+            }
+            if (type.hasAbility("model.ability.produceInWater")) {
+                docks = type;
+            } else if (type.hasAbility("model.ability.export")) {
+                customHouse = type;
+            } else if (type.hasAbility("model.ability.teach")) {
+                schoolhouse = type;
+            } else if (type.getProducedGoodsType() == Goods.HAMMERS) {
+                carpenter = type;
+            } else if (type.getProducedGoodsType() == Goods.HORSES) {
+                stables = type;
+            } else if (type.getProducedGoodsType() == Goods.MUSKETS) {
+                armory = type;
+            } else if (!type.getModifierSet("model.modifier.defence").isEmpty()) {
+                stockade = type;
+            }
+        }
+
+        String ability = "model.ability.produceInWater";
+        if (!colony.hasAbility(ability)) {
+            if (colony.canBuild(docks)) {
+                buildList.add(docks);
+            }
         }
 
         Iterator<WorkLocationPlan> wlpIt = getSortedWorkLocationPlans().iterator();
@@ -140,58 +174,90 @@ public class ColonyPlan {
             WorkLocationPlan wlp = wlpIt.next();
             if (wlp.getWorkLocation() instanceof Building) {
                 Building b = (Building) wlp.getWorkLocation();
-                if (b.getType() == Building.TOWN_HALL) {
-                    if (colony.getBuilding(Building.PRINTING_PRESS).canBuildNext()) {
-                        buildList.add(new Integer(Building.PRINTING_PRESS));
-                    }
-                } else {
-                    if (b.canBuildNext()) {
-                        buildList.add(new Integer(b.getType()));
+                if (b.canBuildNext()) {
+                    buildList.add(b.getType().getUpgradesTo());
+                }
+
+                GoodsType outputType = b.getGoodsOutputType();
+                if (outputType != null) {
+                    for (Building building : colony.getBuildings()) {
+                        if (!building.getType().getModifierSet(outputType.getId()).isEmpty()
+                                && building.canBuildNext()) {
+                            buildList.add(building.getType().getUpgradesTo());
+                        }
                     }
                 }
             }
         }
 
-        if (colony.getBuilding(Building.CUSTOM_HOUSE).canBuildNext()) {
-            if (colony.getGoodsContainer().hasReachedCapacity(colony.getWarehouseCapacity())) {
-                buildList.add(0, new Integer(Building.CUSTOM_HOUSE));
+        Building buildingForExport = null;
+        ability = "model.ability.export";
+        if (!colony.hasAbility(ability)) {
+            if (colony.canBuild(customHouse) &&
+                colony.getGoodsContainer().hasReachedCapacity(colony.getWarehouseCapacity())) {
+                buildList.add(customHouse);
             }
         }
 
         // Check if we should improve the warehouse:
-        if (colony.getBuilding(Building.WAREHOUSE).canBuildNext()) {
+        Building building = colony.getWarehouse();
+        if (building.canBuildNext()) {
             if (colony.getGoodsContainer().hasReachedCapacity(colony.getWarehouseCapacity())) {
-                buildList.add(0, new Integer(Building.WAREHOUSE));
+                buildList.add(0, building.getType().getUpgradesTo());
             } else {
-                buildList.add(new Integer(Building.WAREHOUSE));
+                buildList.add(building.getType());
             }
         }
 
-        if (buildList.size() > 3 && colony.getBuilding(Building.CARPENTER).canBuildNext()) {
-            buildList.add(0, new Integer(Building.CARPENTER));
+        building = colony.getBuildingForProducing(Goods.HAMMERS);
+        if (buildList.size() > 3) {
+            if (building == null) {
+                if (colony.canBuild(carpenter)) {
+                    buildList.add(0, carpenter);
+                }
+            } else if (building.canBuildNext()) {
+                buildList.add(0, building.getType().getUpgradesTo());
+            }
         }
 
-        if (colony.getHorseProduction() > 2 && colony.getBuilding(Building.STABLES).canBuildNext()) {
-            buildList.add(new Integer(Building.STABLES));
+        building = colony.getBuildingForProducing(Goods.HORSES);
+        if (colony.getProductionOf(Goods.HORSES) > 2) {
+            if (building == null) {
+                if (colony.canBuild(stables)) {
+                    buildList.add(stables);
+                }
+            } else if (building.canBuildNext()) {
+                buildList.add(building.getType().getUpgradesTo());
+            }
         }
 
-        if (colony.getBuilding(Building.STOCKADE).canBuildNext()) {
-            buildList.add(new Integer(Building.STOCKADE));
+        building = colony.getStockade();
+        if (building == null) {
+            if (colony.canBuild(stockade)) {
+                buildList.add(stockade);
+            }
+        } else if (building.canBuildNext()) {
+            buildList.add(building.getType().getUpgradesTo());
         }
 
-        if (colony.getBuilding(Building.ARMORY).canBuildNext() && !colony.getBuilding(Building.ARMORY).isBuilt()) {
-            buildList.add(new Integer(Building.ARMORY));
+        building = colony.getBuildingForProducing(Goods.MUSKETS);
+        if (building == null) {
+            if (colony.canBuild(armory)) {
+                buildList.add(armory);
+            }
+        } else if (building.canBuildNext()) {
+            buildList.add(building.getType().getUpgradesTo());
         }
-        buildList.add(new Integer(Colony.BUILDING_UNIT_ADDITION + Unit.ARTILLERY));
+        
+        buildList.add(FreeCol.getSpecification().getUnitType("model.unit.artillery"));
 
-        // buildList.add(new Integer(Building.SCHOOLHOUSE));
-
-        if (colony.getBuilding(Building.CUSTOM_HOUSE).canBuildNext()) {
-            buildList.add(new Integer(Building.CUSTOM_HOUSE));
+        ability = "model.ability.teach";
+        if (!colony.hasAbility(ability)) {
+            if (colony.canBuild(schoolhouse)) {
+                buildList.add(schoolhouse);
+            }
         }
 
-        buildList.add(new Integer(-1));
-
         return buildList.iterator();
     }
 
@@ -216,7 +282,7 @@ public class ColonyPlan {
     /**
      * Creates a plan for this colony. That is; determines what type of goods
      * each tile should produce and what type of goods that should be
-     * manufactored.
+     * manufactured.
      */
     public void create() {
         
@@ -224,49 +290,79 @@ public class ColonyPlan {
         // Colonies should be able to specialize, determine role by colony
         // resources, buildings and specialists
         
+        final GoodsType hammersType = FreeCol.getSpecification().getGoodsType("model.goods.hammers");
+        final GoodsType toolsType = FreeCol.getSpecification().getGoodsType("model.goods.tools");
+        final GoodsType lumberType = FreeCol.getSpecification().getGoodsType("model.goods.lumber");
+        final GoodsType oreType = FreeCol.getSpecification().getGoodsType("model.goods.ore");
+        
         workLocationPlans.clear();
-
+        Building townHall = colony.getBuildingForProducing(Goods.BELLS);
+        
         // Choose the best production for each tile:
-        Iterator<ColonyTile> colonyTileIterator = getColony().getColonyTileIterator();
-        while (colonyTileIterator.hasNext()) {
-            ColonyTile ct = colonyTileIterator.next();
+        for (ColonyTile ct : colony.getColonyTiles()) {
 
-            if (ct.getWorkTile().getOwner() != null && ct.getWorkTile().getOwner() != colony || ct.isColonyCenterTile()) {
+            if (ct.getWorkTile().getOwningSettlement() != null &&
+                ct.getWorkTile().getOwningSettlement() != colony || ct.isColonyCenterTile()) {
                 continue;
             }
 
-            int goodsType = getBestGoodsToProduce(ct.getWorkTile());
+            GoodsType goodsType = getBestGoodsToProduce(ct.getWorkTile());
             WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), ct, goodsType);
             workLocationPlans.add(wlp);
         }
+        
+        // We need to find what, if any, is still required for what we are building
+        GoodsType buildingReq = null;
+        GoodsType buildingRawMat = null;
+        Building buildingReqProducer = null;
+        BuildableType currBuild = colony.getCurrentlyBuilding();
+        if(currBuild != null){
+            if(colony.getGoodsCount(hammersType) < currBuild.getAmountRequiredOf(hammersType)){
+                buildingReq = hammersType;
+                buildingRawMat = lumberType;
+            }
+            else{
+                buildingReq = toolsType;
+                buildingRawMat = oreType;
+            }
+            buildingReqProducer = colony.getBuildingForProducing(buildingReq);
+        }
 
-        // Ensure that we produce lumber:
-        if (getProductionOf(Goods.LUMBER) <= 0) {
+        // Try to ensure that we produce the raw material necessary for
+        //what we are building
+        if(buildingRawMat != null && getProductionOf(buildingRawMat) <= 0) {
             WorkLocationPlan bestChoice = null;
             int highestPotential = 0;
 
             Iterator<WorkLocationPlan> wlpIterator = workLocationPlans.iterator();
             while (wlpIterator.hasNext()) {
                 WorkLocationPlan wlp = wlpIterator.next();
+                // TODO: find out about unit working here, if any (?)
                 if (wlp.getWorkLocation() instanceof ColonyTile
-                        && ((ColonyTile) wlp.getWorkLocation()).getWorkTile().potential(Goods.LUMBER) > highestPotential) {
-                    highestPotential = ((ColonyTile) wlp.getWorkLocation()).getWorkTile().potential(Goods.LUMBER);
+                    && ((ColonyTile) wlp.getWorkLocation()).getWorkTile().potential(buildingRawMat, null) > highestPotential) {
+                    highestPotential = ((ColonyTile) wlp.getWorkLocation()).getWorkTile().potential(buildingRawMat, null);
                     bestChoice = wlp;
                 }
             }
             if (highestPotential > 0) {
+                // this must be true because it is the only way to
+                // increase highestPotential
+                assert bestChoice != null;
                 bestChoice.setGoodsType(Goods.LUMBER);
             }
         }
 
         // Determine the primary and secondary types of goods:
-        int primaryRawMaterial = -1;
+        GoodsType primaryRawMaterial = null;
         int primaryRawMaterialProduction = 0;
-        int secondaryRawMaterial = -1;
+        GoodsType secondaryRawMaterial = null;
         int secondaryRawMaterialProduction = 0;
-        for (int goodsType = 0; goodsType < Goods.NUMBER_OF_TYPES; goodsType++) {
-            if (goodsType != Goods.SUGAR && goodsType != Goods.TOBACCO && goodsType != Goods.COTTON
-                    && goodsType != Goods.FURS && goodsType != Goods.ORE) {
+        List<GoodsType> goodsTypeList = FreeCol.getSpecification().getGoodsTypeList();
+        for (GoodsType goodsType : goodsTypeList) {
+            // only consider goods that can be transformed
+            // do not consider hammers as a valid transformation
+            if (goodsType.getProducedMaterial() == null 
+                    || goodsType.getProducedMaterial() == hammersType) {
                 continue;
             }
             if (getProductionOf(goodsType) > primaryRawMaterialProduction) {
@@ -294,8 +390,13 @@ public class ColonyPlan {
                     || wlp.getGoodsType() == Goods.SILVER) {
                 continue;
             }
-            if (((ColonyTile) wlp.getWorkLocation()).getWorkTile().potential(Goods.FOOD) <= 2) {
-                if (wlp.getProductionOf(wlp.getGoodsType()) <= 2) {
+            // TODO: find out about unit working here, if any (?)
+            if (((ColonyTile) wlp.getWorkLocation()).getWorkTile().potential(Goods.FOOD, null) <= 2) {
+                if (wlp.getGoodsType() == null) {
+                    // on arctic tiles nothing can be produced
+                    wlpIterator.remove();
+                } else if (wlp.getProductionOf(wlp.getGoodsType()) <= 2) {
+                    // just a poor location
                     wlpIterator.remove();
                 }
                 continue;
@@ -304,39 +405,41 @@ public class ColonyPlan {
             wlp.setGoodsType(Goods.FOOD);
         }
 
-        // Place a carpenter:
-        if (getProductionOf(Goods.LUMBER) > 0) {
-            WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), colony.getBuilding(Building.CARPENTER),
-                    Goods.HAMMERS);
+        // Produce the goods required for what is being built, if:
+        //     - anything is being built, and
+        //     - there is either production or stock of the raw material
+        if(buildingReq != null && 
+            (getProductionOf(buildingRawMat) > 0 
+              || colony.getGoodsCount(buildingRawMat) > 0)){
+            WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(),
+                    colony.getBuildingForProducing(buildingReq), buildingReq);
             workLocationPlans.add(wlp);
         }
 
         // Place a statesman:
-        WorkLocationPlan townHallWlp = new WorkLocationPlan(getAIMain(), colony.getBuilding(Building.TOWN_HALL),
-                Goods.BELLS);
+        WorkLocationPlan townHallWlp = new WorkLocationPlan(getAIMain(), townHall, Goods.BELLS);
         workLocationPlans.add(townHallWlp);
 
         // Place a colonist to manufacture the primary goods:
-        if (primaryRawMaterial > -1) {
-            Building b = colony.getBuildingForProducing(Goods.getManufactoredGoods(primaryRawMaterial));
+        if (primaryRawMaterial != null) {
+            GoodsType producedGoods = primaryRawMaterial.getProducedMaterial();
+            Building b = colony.getBuildingForProducing(producedGoods);
             if (b != null) {
-                WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, Goods
-                        .getManufactoredGoods(primaryRawMaterial));
+                WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, producedGoods);
                 workLocationPlans.add(wlp);
             }
         }
 
         // Remove the secondary goods if we need food:
-        if (getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2
-                && (secondaryRawMaterial == Goods.SUGAR || secondaryRawMaterial == Goods.TOBACCO
-                        || secondaryRawMaterial == Goods.COTTON || secondaryRawMaterial == Goods.FURS)) {
-
+        if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION &&
+            secondaryRawMaterial.isNewWorldGoodsType()) {
             Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
             while (wlpIterator2.hasNext()) {
                 WorkLocationPlan wlp = wlpIterator2.next();
                 if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == secondaryRawMaterial) {
                     Tile t = ((ColonyTile) wlp.getWorkLocation()).getWorkTile();
-                    if (t.getMaximumPotential(Goods.FOOD) > 2) {
+                    // TODO: find out about unit working here, if any (?)
+                    if (t.getMaximumPotential(Goods.FOOD, null) > 2) {
                         wlp.setGoodsType(Goods.FOOD);
                     } else {
                         wlpIterator2.remove();
@@ -346,13 +449,14 @@ public class ColonyPlan {
         }
 
         // Remove the workers on the primary goods one-by-one if we need food:
-        if (getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
+        if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
             Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
-            while (wlpIterator2.hasNext() && getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
+            while (wlpIterator2.hasNext() && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
                 WorkLocationPlan wlp = wlpIterator2.next();
                 if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == primaryRawMaterial) {
                     Tile t = ((ColonyTile) wlp.getWorkLocation()).getWorkTile();
-                    if (t.getMaximumPotential(Goods.FOOD) > 2) {
+                    // TODO: find out about unit working here, if any (?)
+                    if (t.getMaximumPotential(Goods.FOOD, null) > 2) {
                         wlp.setGoodsType(Goods.FOOD);
                     } else {
                         wlpIterator2.remove();
@@ -362,72 +466,75 @@ public class ColonyPlan {
         }
 
         // Remove the manufacturer if we still lack food:
-        if (getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
+        if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
             Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
-            while (wlpIterator2.hasNext() && getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
+            while (wlpIterator2.hasNext() && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
                 WorkLocationPlan wlp = wlpIterator2.next();
                 if (wlp.getWorkLocation() instanceof Building) {
                     Building b = (Building) wlp.getWorkLocation();
-                    if (b.getType() != Building.CARPENTER && b.getType() != Building.TOWN_HALL) {
+                    if ( b != buildingReqProducer && b != townHall) {
                         wlpIterator2.remove();
                     }
                 }
             }
         }
 
-        // Remove the lumberjacks if we still lack food:
-        if (getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
+        // Still lacking food
+        // Remove the producers of the raw and/or non-raw materials required for the build
+        // The decision of which to start depends on existence or not of stock of
+        //raw materials
+        GoodsType buildMatToGo = buildingReq;
+        if(colony.getGoodsCount(buildingRawMat) > 0){
+            buildMatToGo = buildingRawMat;
+        }
+        if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
             Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
-            while (wlpIterator2.hasNext() && getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
+            while (wlpIterator2.hasNext() && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
                 WorkLocationPlan wlp = wlpIterator2.next();
-                if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == Goods.LUMBER) {
+                if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == buildMatToGo) {
                     wlpIterator2.remove();
                 }
             }
-        }
-
-        // Remove the carpenter if we have no lumber or lack food:
-        // TODO: Erik - run short on lumber as long as there is a stockpile!
-        if (getProductionOf(Goods.LUMBER) < 1 || getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
-            Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
-            while (wlpIterator2.hasNext() && getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
-                WorkLocationPlan wlp = wlpIterator2.next();
-                if (wlp.getWorkLocation() instanceof Building) {
-                    Building b = (Building) wlp.getWorkLocation();
-                    if (b.getType() == Building.CARPENTER) {
+            // still lacking food, removing the rest
+            if (getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+                buildMatToGo = (buildMatToGo == buildingRawMat)? buildingReq : buildingRawMat;
+                
+                wlpIterator2 = workLocationPlans.iterator();
+                while (wlpIterator2.hasNext() && getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION) {
+                    WorkLocationPlan wlp = wlpIterator2.next();
+                    if (wlp.getWorkLocation() instanceof ColonyTile && wlp.getGoodsType() == buildMatToGo) {
                         wlpIterator2.remove();
                     }
                 }
             }
-        }
-
-        // Remove all other colonists in buildings if we still are lacking food:
-        if (getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
-            Iterator<WorkLocationPlan> wlpIterator2 = workLocationPlans.iterator();
-            while (wlpIterator2.hasNext() && getProductionOf(Goods.FOOD) < workLocationPlans.size() * 2) {
-                WorkLocationPlan wlp = wlpIterator2.next();
-                if (wlp.getWorkLocation() instanceof Building) {
-                    wlpIterator2.remove();
-                }
-            }
+        }        
+        
+        // Primary allocations done
+        // Beginning secondary allocations
+        
+        // Not enough food for more allocations, save work and stop here
+        if(getFoodProduction() < workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2){
+            return;
         }
 
         int primaryWorkers = 1;
         int secondaryWorkers = 0;
-        int carpenters = 1;
+        int builders = 1;
         int gunsmiths = 0;
         boolean colonistAdded = true;
+        //XXX: This loop does not work, only goes through once, not as intended
         while (colonistAdded) {
             boolean blacksmithAdded = false;
 
             // Add a manufacturer for the secondary type of goods:
-            if (getProductionOf(Goods.FOOD) >= workLocationPlans.size() * 2 + 2 && secondaryRawMaterial > -1
-                    && 12 * secondaryWorkers + 6 <= getProductionOf(secondaryRawMaterial)
-                    && secondaryWorkers <= Building.MAX_LEVEL) {
-                Building b = colony.getBuildingForProducing(Goods.getManufactoredGoods(secondaryRawMaterial));
+            if (getFoodProduction() >= workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2 &&
+                secondaryRawMaterial != null &&
+                12 * secondaryWorkers + 6 <= getProductionOf(secondaryRawMaterial) &&
+                secondaryWorkers <= MAX_LEVEL) {
+                GoodsType producedGoods = secondaryRawMaterial.getProducedMaterial();
+                Building b = colony.getBuildingForProducing(producedGoods);
                 if (b != null) {
-                    WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, Goods
-                            .getManufactoredGoods(secondaryRawMaterial));
+                    WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, producedGoods);
                     workLocationPlans.add(wlp);
                     colonistAdded = true;
                     secondaryWorkers++;
@@ -438,13 +545,13 @@ public class ColonyPlan {
             }
 
             // Add a manufacturer for the primary type of goods:
-            if (getProductionOf(Goods.FOOD) >= workLocationPlans.size() * 2 + 2 && primaryRawMaterial > -1
+            if (getFoodProduction() >= workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2 && primaryRawMaterial != null
                     && 12 * primaryWorkers + 6 <= getProductionOf(primaryRawMaterial)
-                    && primaryWorkers <= Building.MAX_LEVEL) {
-                Building b = colony.getBuildingForProducing(Goods.getManufactoredGoods(primaryRawMaterial));
+                    && primaryWorkers <= MAX_LEVEL) {
+                GoodsType producedGoods = primaryRawMaterial.getProducedMaterial();
+                Building b = colony.getBuildingForProducing(producedGoods);
                 if (b != null) {
-                    WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, Goods
-                            .getManufactoredGoods(primaryRawMaterial));
+                    WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, producedGoods);
                     workLocationPlans.add(wlp);
                     colonistAdded = true;
                     primaryWorkers++;
@@ -455,8 +562,8 @@ public class ColonyPlan {
             }
 
             // Add a gunsmith:
-            if (blacksmithAdded && getProductionOf(Goods.FOOD) >= workLocationPlans.size() * 2 + 2
-                    && gunsmiths < Building.MAX_LEVEL) {
+            if (blacksmithAdded && getFoodProduction() >= workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2
+                    && gunsmiths < MAX_LEVEL) {
                 Building b = colony.getBuildingForProducing(Goods.MUSKETS);
                 if (b != null) {
                     WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, Goods.MUSKETS);
@@ -466,16 +573,15 @@ public class ColonyPlan {
                 }
             }
 
-            // Add carpenters:
-            if (getProductionOf(Goods.FOOD) >= workLocationPlans.size() * 2 + 2
-                    && 12 * carpenters + 6 <= getProductionOf(Goods.LUMBER) && carpenters <= Building.MAX_LEVEL) {
-                Building b = colony.getBuilding(Building.CARPENTER);
-                if (b != null) {
-                    WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), b, Goods.HAMMERS);
-                    workLocationPlans.add(wlp);
-                    colonistAdded = true;
-                    carpenters++;
-                }
+            // Add builders
+            if (getFoodProduction() >= workLocationPlans.size() * Colony.FOOD_CONSUMPTION + 2
+                    && buildingReqProducer != null 
+                    && buildingReqProducer.getProduction() * builders <= getProductionOf(buildingRawMat) 
+                    && buildingReqProducer.getMaxUnits() < builders) {
+                WorkLocationPlan wlp = new WorkLocationPlan(getAIMain(), buildingReqProducer, buildingReq);
+                workLocationPlans.add(wlp);
+                colonistAdded = true;
+                builders++;
             }
 
             // TODO: Add worker to armory.
@@ -489,13 +595,13 @@ public class ColonyPlan {
     }
 
     /**
-     * Returns the production of the given type of goods accoring to this plan.
+     * Returns the production of the given type of goods according to this plan.
      * 
      * @param goodsType The type of goods to check the production for.
      * @return The maximum possible production of the given type of goods
      *         according to this <code>ColonyPlan</code>.
      */
-    public int getProductionOf(int goodsType) {
+    public int getProductionOf(GoodsType goodsType) {
         int amount = 0;
 
         Iterator<WorkLocationPlan> wlpIterator = workLocationPlans.iterator();
@@ -505,10 +611,25 @@ public class ColonyPlan {
         }
 
         // Add values for the center tile:
-        if (goodsType == Goods.FOOD) {
-            amount += colony.getTile().getMaximumPotential(Goods.FOOD);
-        } else if (goodsType == colony.getTile().secondaryGoods()) {
-            amount += colony.getTile().getMaximumPotential(goodsType);
+        if (goodsType == colony.getTile().primaryGoods() ||
+            goodsType == colony.getTile().secondaryGoods()) {
+            // TODO: find out about unit working here, if any (?)
+            amount += colony.getTile().getMaximumPotential(goodsType, null);
+        }
+
+        return amount;
+    }
+
+    /**
+     * Returns the production of food according to this plan.
+     * 
+     * @return The maximum possible food production
+     *         according to this <code>ColonyPlan</code>.
+     */
+    public int getFoodProduction() {
+        int amount = 0;
+        for (GoodsType foodType : FreeCol.getSpecification().getGoodsFood()) {
+            amount += getProductionOf(foodType);
         }
 
         return amount;
@@ -521,50 +642,17 @@ public class ColonyPlan {
      * @param t The <code>Tile</code>.
      * @return The type of goods.
      */
-    private int getBestGoodsToProduce(Tile t) {
-        if (t.isForested() && t.hasBonus()) {
-            if (t.getType() == Tile.GRASSLANDS || t.getType() == Tile.SAVANNAH) {
-                return Goods.LUMBER;
+    private GoodsType getBestGoodsToProduce(Tile t) {
+        if (t.hasResource()) {
+            return t.getTileItemContainer().getResource().getBestGoodsType();
+        } else {
+            List<AbstractGoods> sortedPotentials = t.getSortedPotential();
+            if (sortedPotentials.isEmpty()) {
+                return null;
             } else {
-                return Goods.FURS;
+                return sortedPotentials.get(0).getType();
             }
         }
-        if (t.getAddition() == Tile.ADD_HILLS) {
-            return Goods.ORE;
-        }
-        if (t.getAddition() == Tile.ADD_MOUNTAINS) {
-            if (t.hasBonus()) {
-                return Goods.SILVER;
-            } else {
-                return Goods.ORE;
-            }
-        }
-        if (!t.isLand()) {
-            return Goods.FOOD;
-        }
-        if (t.getType() == Tile.DESERT) {
-            if (t.hasBonus()) {
-                return Goods.FOOD;
-            } else {
-                return Goods.ORE;
-            }
-        }
-        switch (t.getType()) {
-        case Tile.SWAMP:
-        case Tile.PLAINS:
-        case Tile.TUNDRA:
-        case Tile.MARSH:
-            return Goods.FOOD;
-        case Tile.PRAIRIE:
-            return Goods.COTTON;
-        case Tile.GRASSLANDS:
-            return Goods.TOBACCO;
-        case Tile.SAVANNAH:
-            return Goods.SUGAR;
-        case Tile.ARCTIC:
-        default:
-            return Goods.ORE;
-        }
     }
 
     /**
