@@ -28,8 +28,11 @@ import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.Game;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.PathNode;
+import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.model.Map.Position;
@@ -62,11 +65,9 @@ import org.w3c.dom.Element;
  * @see net.sf.freecol.common.model.Colony Colony
  */
 public class BuildColonyMission extends Mission {
+
     private static final Logger logger = Logger.getLogger(BuildColonyMission.class.getName());
 
-
-
-
     /** The <code>Tile</code> where the <code>Colony</code> should be built. */
     private Tile target;
 
@@ -176,7 +177,7 @@ public class BuildColonyMission extends Mission {
         }
 
         if (target == null || doNotGiveUp
-                && (colonyValue != getUnit().getOwner().getColonyValue(target) || target.getSettlement() != null)) {
+            && (colonyValue != getUnit().getOwner().getColonyValue(target) || target.getSettlement() != null)) {
             target = findColonyLocation(getUnit());
             if (target == null) {
                 doNotGiveUp = false;
@@ -189,13 +190,8 @@ public class BuildColonyMission extends Mission {
         // Move towards the target.
         if (getUnit().getTile() != null) {
             if (target != getUnit().getTile()) {
-                int r = moveTowards(connection, target);
-                if (r >= 0) {
-                    final int mt = getUnit().getMoveType(r);
-                    if (mt != Unit.ILLEGAL_MOVE && mt != Unit.ATTACK) {
-                        move(connection, r);
-                    }
-                }
+                Direction r = moveTowards(connection, target);
+                moveButDontAttack(connection, r);
             }
             if (getUnit().canBuildColony() && target == getUnit().getTile() && getUnit().getMovesLeft() > 0) {
                 Element buildColonyElement = Message.createNewRootElement("buildColony");
@@ -203,11 +199,16 @@ public class BuildColonyMission extends Mission {
                 buildColonyElement.setAttribute("unit", unit.getId());
 
                 try {
-                    connection.ask(buildColonyElement);
-                    colonyBuilt = true;
-                    getAIUnit().setMission(
-                            new WorkInsideColonyMission(getAIMain(), getAIUnit(), (AIColony) getAIMain().getAIObject(
-                                    unit.getTile().getSettlement())));
+                    Element reply = connection.ask(buildColonyElement);
+                    if (reply!=null && reply.getTagName().equals("buildColonyConfirmed")) {
+                        colonyBuilt = true;
+                        Settlement settlement = unit.getTile().getSettlement();
+                        AIColony aiColony = (AIColony) getAIMain().getAIObject(settlement);
+                        getAIUnit().setMission(new WorkInsideColonyMission(getAIMain(), getAIUnit(), aiColony));
+                    }
+                    else {
+                        logger.warning("Could not build an AI colony on tile "+getUnit().getTile().getPosition().toString());
+                    }
                 } catch (IOException e) {
                     logger.warning("Could not send \"buildColonyElement\"-message!");
                 }
@@ -225,14 +226,14 @@ public class BuildColonyMission extends Mission {
      */
     public Tile getTransportDestination() {
         if (target == null) {
-            if (getUnit().getLocation() instanceof Unit) {
+            if (getUnit().isOnCarrier()) {
                 return (Tile) ((Unit) getUnit().getLocation()).getEntryLocation();
             } else {
                 return (Tile) getUnit().getOwner().getEntryLocation();
             }
         }
 
-        if (getUnit().getLocation() instanceof Unit) {
+        if (getUnit().isOnCarrier()) {
             return target;
         } else if (getUnit().getLocation().getTile() == target) {
             return null;
@@ -270,54 +271,77 @@ public class BuildColonyMission extends Mission {
      *         <code>Unit</code>.
      */
     public static Tile findColonyLocation(Unit unit) {
+        
         Game game = unit.getGame();
-
-        if (unit.getTile() == null) {
+        
+        Tile startTile = null;
+        if (unit.isOnCarrier()) {
+            Unit carrier = (Unit) unit.getLocation();
+            startTile = carrier.getTile();
+        } else if (unit.getLocation() instanceof Europe) {
+            startTile = (Tile) unit.getEntryLocation();
+        } else {
+            startTile = unit.getTile();
+        }
+            
+        if (startTile == null) {
             return null;
         }
 
         Tile bestTile = null;
         int highestColonyValue = 0;
+        int maxNumberofTiles = 500;
+        int tileCounter = 0;
 
-        Iterator<Position> it = game.getMap().getFloodFillIterator(unit.getTile().getPosition());
-        for (int i = 0; it.hasNext() && i < 500; i++) {
+        //if called during the first few turns of the game,
+        //and our unit may be the starting unit (==isOnCarrier())
+        //make sure to find _some_ starting position        
+        boolean gameStart = false;
+        if (unit.getGame().getTurn().getNumber() < 10 && unit.isOnCarrier()) {
+            gameStart = true;
+        }
+        
+        Iterator<Position> it = game.getMap().getFloodFillIterator(startTile.getPosition());
+        
+        while (it.hasNext()) {
             Tile tile = game.getMap().getTile(it.next());
-            if (tile.getColonyValue() > 0) {
-                if (tile != unit.getTile()) {
-                    PathNode path;
-                    if (unit.getLocation() instanceof Unit) {
-                        Unit carrier = (Unit) unit.getLocation();
-                        path = game.getMap().findPath(unit, carrier.getTile(), tile, carrier);
-                    } else {
-                        path = game.getMap().findPath(unit, unit.getTile(), tile);
-                    }
+            int newColonyValue = -1;
 
+            if (tile.getColonyValue() > 0) {
+                if (tile != startTile) {
+                    PathNode path;
+
+                    if (unit.isOnCarrier()) {
+                        Unit carrier = (Unit) unit.getLocation();
+                        path = game.getMap().findPath(unit, startTile, tile, carrier);
+                    } else {
+                        path = game.getMap().findPath(unit, startTile, tile);
+                    }
                     if (path != null) {
-                        int newColonyValue = 10000
-                                + unit.getOwner().getColonyValue(tile)
-                                - path.getTotalTurns()
-                                * ((unit.getGame().getTurn().getNumber() < 10 && unit.getLocation() instanceof Unit) ? 25
-                                        : 4);
-                        if (newColonyValue > highestColonyValue) {
-                            highestColonyValue = newColonyValue;
-                            bestTile = tile;
-                        }
+                        newColonyValue = 10000
+                            + unit.getOwner().getColonyValue(tile)
+                            - path.getTotalTurns()
+                            * ((unit.getGame().getTurn().getNumber() < 10
+                                && unit.isOnCarrier()) ? 25 : 4);
                     }
                 } else {
-                    int newColonyValue = 10000 + unit.getOwner().getColonyValue(tile);
-                    if (newColonyValue > highestColonyValue) {
-                        highestColonyValue = newColonyValue;
-                        bestTile = tile;
-                    }
+                    newColonyValue = 10000 + unit.getOwner().getColonyValue(tile);
+                }
+                if (newColonyValue > highestColonyValue) {
+                    highestColonyValue = newColonyValue;
+                    bestTile = tile;
                 }
             }
+            //break after checking a fixed number of tiles
+            //unless this may be the first colony,
+            //in which case we'll continue until we found _some_ location.
+            if ((++tileCounter >= maxNumberofTiles)
+                && (!gameStart || bestTile!= null)) break;
         }
-
-        if (bestTile != null && bestTile.getColonyValue() > 0) {
-            return bestTile;
-        } else {
-            return null;
+        if (bestTile == null) {
+            logger.info("Unit " + unit.getId() + " unsuccessfully searched for colony spot");
         }
+        return bestTile;
     }
 
     /**
@@ -334,8 +358,8 @@ public class BuildColonyMission extends Mission {
      */
     public boolean isValid() {
         return (!colonyBuilt && (doNotGiveUp || target != null
-                && target.getSettlement() == null
-                && colonyValue <= getUnit().getOwner().getColonyValue(target)));
+                                 && target.getSettlement() == null
+                                 && colonyValue <= getUnit().getOwner().getColonyValue(target)));
     }
 
     /**
