@@ -29,11 +29,15 @@ import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
 import net.sf.freecol.common.model.Colony;
+import net.sf.freecol.common.model.CombatModel;
 import net.sf.freecol.common.model.Map;
-import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Map.Direction;
+import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.MoveType;
+import net.sf.freecol.common.model.Unit.UnitState;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.server.ai.AIMain;
@@ -124,25 +128,26 @@ public class DefendSettlementMission extends Mission {
         }
         
         if (unit.isOffensiveUnit()) {
+            CombatModel combatModel = unit.getGame().getCombatModel();
             Unit bestTarget = null;
             float bestDifference = Float.MIN_VALUE;
-            int bestDirection = -1;
+            Direction bestDirection = null;
             
-            int[] directions = map.getRandomDirectionArray();
-            for (int i=0; i<Map.NUMBER_OF_DIRECTIONS; i++) {
-                int direction = directions[i];
+            Direction[] directions = map.getRandomDirectionArray();
+            for (Direction direction : directions) {
                 Tile t = map.getNeighbourOrNull(direction, unit.getTile());
-                if (t != null
-                        && t.getDefendingUnit(unit) != null
-                        && t.getDefendingUnit(unit).getOwner().getStance(unit.getOwner()) == Player.WAR
-                        && unit.getMoveType(direction) == Unit.ATTACK) {
+                if (t==null)
+                    continue;
+                Unit defender = t.getFirstUnit();
+                if (defender != null && defender.getOwner().getStance(unit.getOwner()) == Stance.WAR
+                    && unit.getMoveType(direction) == MoveType.ATTACK) {
                     Unit enemyUnit = t.getDefendingUnit(unit);
-                    float enemyAttack = enemyUnit.getOffensePower(unit);
-                    float weAttack = unit.getOffensePower(enemyUnit);
-                    float enemyDefend = enemyUnit.getDefensePower(unit);
-                    float weDefend = unit.getDefensePower(enemyUnit);
+                    float enemyAttack = combatModel.getOffencePower(enemyUnit, unit);
+                    float weAttack = combatModel.getOffencePower(unit, enemyUnit);
+                    float enemyDefend = combatModel.getDefencePower(unit, enemyUnit);
+                    float weDefend = combatModel.getDefencePower(enemyUnit, unit);
 
-                    float difference = weAttack / (weAttack + enemyDefend) - enemyAttack / (enemyAttack + weDefend);                  
+                    float difference = weAttack / (weAttack + enemyDefend) - enemyAttack / (enemyAttack + weDefend);
                     if (difference > bestDifference) {
                         if (difference > 0 || weAttack > enemyDefend) {
                             bestDifference = difference;
@@ -154,9 +159,12 @@ public class DefendSettlementMission extends Mission {
             }
             
             if (bestTarget != null) {
+                // this must be true, since it is the only way to get
+                // a bestTarget
+                assert bestDirection != null;
                 Element element = Message.createNewRootElement("attack");
                 element.setAttribute("unit", unit.getId());
-                element.setAttribute("direction", Integer.toString(bestDirection));
+                element.setAttribute("direction", bestDirection.toString());
 
                 try {
                     connection.ask(element);
@@ -169,20 +177,15 @@ public class DefendSettlementMission extends Mission {
             
         if (unit.getTile() != settlement.getTile()) {
             // Move towards the target.
-            int r = moveTowards(connection, settlement.getTile());
-            if (r >= 0) {
-                final int mt = getUnit().getMoveType(r);
-                if (mt != Unit.ILLEGAL_MOVE && mt != Unit.ATTACK) {
-                    move(connection, r);
-                }
-            }
+            Direction r = moveTowards(connection, settlement.getTile());
+            moveButDontAttack(connection, r);
         } else {
-            if (unit.getState() != Unit.FORTIFIED
-                    && unit.getState() != Unit.FORTIFYING
-                    && unit.checkSetState(Unit.FORTIFYING)) {
+            if (unit.getState() != UnitState.FORTIFIED
+                    && unit.getState() != UnitState.FORTIFYING
+                    && unit.checkSetState(UnitState.FORTIFYING)) {
                 Element changeStateElement = Message.createNewRootElement("changeState");
                 changeStateElement.setAttribute("unit", unit.getId());
-                changeStateElement.setAttribute("state", Integer.toString(Unit.FORTIFYING));
+                changeStateElement.setAttribute("state", UnitState.FORTIFYING.toString());
                 try {
                     logger.log(Level.FINEST, "Sending fortity request...");
                     connection.sendAndWait(changeStateElement);
@@ -203,7 +206,7 @@ public class DefendSettlementMission extends Mission {
      * @return The destination for this <code>Transportable</code>.
      */    
      public Tile getTransportDestination() {
-         if (getUnit().getLocation() instanceof Unit) {
+         if (getUnit().isOnCarrier()) {
              return settlement.getTile();
          } else if (getUnit().getLocation().getTile() == settlement.getTile()) {
              return null;
