@@ -28,14 +28,16 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Goods;
 import net.sf.freecol.common.model.GoodsContainer;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.IndianSettlement;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Market;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Tension;
-import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
@@ -49,12 +51,10 @@ import org.w3c.dom.Element;
  * Mission for demanding goods from a specified player.
  */
 public class IndianDemandMission extends Mission {
+
     private static final Logger logger = Logger.getLogger(IndianDemandMission.class.getName());
 
-
-
-
-    /** The <code>Colony</code> receiving the gift. */
+    /** The <code>Colony</code> receiving the demand. */
     private Colony target;
 
     /** Whether this mission has been completed or not. */
@@ -73,9 +73,9 @@ public class IndianDemandMission extends Mission {
 
         this.target = target;
 
-        if (getUnit().getType() != Unit.BRAVE) {
-            logger.warning("Only an indian brave can be given the mission: IndianDemandMission");
-            throw new IllegalArgumentException("Only an indian brave can be given the mission: IndianDemandMission");
+        if (!getUnit().getOwner().isIndian() || !getUnit().canCarryGoods()) {
+            logger.warning("Only an indian which can carry goods can be given the mission: IndianBringGiftMission");
+            throw new IllegalArgumentException("Only an indian which can carry goods can be given the mission: IndianBringGiftMission");
         }
     }
 
@@ -118,21 +118,17 @@ public class IndianDemandMission extends Mission {
         if (!hasGift()) {
             if (getUnit().getTile() != getUnit().getIndianSettlement().getTile()) {
                 // Move to the owning settlement:
-                int r = moveTowards(connection, getUnit().getIndianSettlement().getTile());
-                if (r >= 0) {
-                    final int mt = getUnit().getMoveType(r);
-                    if (mt != Unit.ILLEGAL_MOVE && mt != Unit.ATTACK) {
-                        move(connection, r);
-                    }
-                }
+                Direction r = moveTowards(connection, getUnit().getIndianSettlement().getTile());
+                moveButDontAttack(connection, r);
             } else {
                 // Load the goods:
                 ArrayList<Goods> goodsList = new ArrayList<Goods>();
                 GoodsContainer gc = getUnit().getIndianSettlement().getGoodsContainer();
-                for (int i = 1; i <= 4; i++) {
-                    if (gc.getGoodsCount(i) >= IndianSettlement.KEEP_RAW_MATERIAL + 25) {
-                        goodsList.add(new Goods(getGame(), getUnit().getIndianSettlement(), i,
-                                getRandom().nextInt(15) + 10));
+                for (GoodsType goodsType : FreeCol.getSpecification().getNewWorldGoodsTypeList()) {
+                    if (gc.getGoodsCount(goodsType) >= IndianSettlement.KEEP_RAW_MATERIAL + 25) {
+                        goodsList.add(new Goods(getGame(), getUnit().getIndianSettlement(),
+                                                goodsType,
+                                                getRandom().nextInt(15) + 10));
                     }
                 }
 
@@ -144,9 +140,10 @@ public class IndianDemandMission extends Mission {
         } else {
             // Move to the target's colony and deliver
             Unit unit = getUnit();
-            int r = moveTowards(connection, target.getTile());
-            if (r >= 0 && getGame().getMap().getNeighbourOrNull(r, unit.getTile()) == target.getTile()
-                    && unit.getMovesLeft() > 0) {
+            Direction r = moveTowards(connection, target.getTile());
+            if (r != null &&
+                getGame().getMap().getNeighbourOrNull(r, unit.getTile()) == target.getTile()
+                && unit.getMovesLeft() > 0) {
                 // We have arrived.
                 Element demandElement = Message.createNewRootElement("indianDemand");
                 demandElement.setAttribute("unit", unit.getId());
@@ -183,11 +180,14 @@ public class IndianDemandMission extends Mission {
                 if (unit.getIndianSettlement() != null) {
                     unitTension += unit.getIndianSettlement().getOwner().getTension(enemy).getValue();
                 }
+                // TODO: make this work with DifficultyLevel
+                int difficulty = enemy.getDifficulty().getIndex();
                 if (accepted) {
                     // TODO: if very happy, the brave should convert
-                    tension = -(5 - enemy.getDifficulty()) * 50;
+                    tension = -(5 - difficulty) * 50;
                     unit.getOwner().modifyTension(enemy, tension);
-                    if (unitTension <= Tension.TENSION_HAPPY && (goods == null || goods.getType() == Goods.FOOD)) {
+                    if (unitTension <= Tension.Level.HAPPY.getLimit() &&
+                        (goods == null || goods.getType().isFoodType())) {
                         Element deliverGiftElement = Message.createNewRootElement("deliverGift");
                         deliverGiftElement.setAttribute("unit", getUnit().getId());
                         deliverGiftElement.setAttribute("settlement", target.getId());
@@ -201,13 +201,13 @@ public class IndianDemandMission extends Mission {
                         }
                     }
                 } else {
-                    tension = (enemy.getDifficulty() + 1) * 50;
+                    tension = (difficulty + 1) * 50;
                     unit.getOwner().modifyTension(enemy, tension);
-                    if (unitTension >= Tension.TENSION_CONTENT) {
+                    if (unitTension >= Tension.Level.CONTENT.getLimit()) {
                         // if we didn't get what we wanted, attack
                         Element element = Message.createNewRootElement("attack");
                         element.setAttribute("unit", unit.getId());
-                        element.setAttribute("direction", Integer.toString(r));
+                        element.setAttribute("direction", r.toString());
 
                         try {
                             connection.ask(element);
@@ -221,30 +221,7 @@ public class IndianDemandMission extends Mission {
         }
 
         // Walk in a random direction if we have any moves left:
-        Tile thisTile = getUnit().getTile();
-        Unit unit = getUnit();
-        while (unit.getMovesLeft() > 0) {
-            int direction = (int) (Math.random() * 8);
-            int j;
-            for (j = 8; j > 0
-                    && ((unit.getGame().getMap().getNeighbourOrNull(direction, thisTile) == null) || (unit
-                            .getMoveType(direction) != Unit.MOVE)); j--) {
-                direction = (int) (Math.random() * 8);
-            }
-            if (j == 0)
-                break;
-            thisTile = unit.getGame().getMap().getNeighbourOrNull(direction, thisTile);
-
-            Element moveElement = Message.createNewRootElement("move");
-            moveElement.setAttribute("unit", unit.getId());
-            moveElement.setAttribute("direction", Integer.toString(direction));
-
-            try {
-                connection.sendAndWait(moveElement);
-            } catch (IOException e) {
-                logger.warning("Could not send \"move\"-message!");
-            }
-        }
+        moveRandomly(connection);
     }
 
     /**
@@ -254,23 +231,25 @@ public class IndianDemandMission extends Mission {
      * @return The goods to demand.
      */
     public Goods selectGoods(Colony target) {
-        int tension = getUnit().getOwner().getTension(target.getOwner()).getLevel();
-        int dx = target.getOwner().getDifficulty() + 1;
+        Tension.Level tension = getUnit().getOwner().getTension(target.getOwner()).getLevel();
+        int dx = target.getOwner().getDifficulty().getIndex() + 1;
+        GoodsType food = FreeCol.getSpecification().getGoodsType("model.goods.food");
         Goods goods = null;
         GoodsContainer warehouse = target.getGoodsContainer();
-        if (tension <= Tension.CONTENT && warehouse.getGoodsCount(Goods.FOOD) >= 100) {
-            int amount = (warehouse.getGoodsCount(Goods.FOOD) * dx) / 6;
+        if (tension.compareTo(Tension.Level.CONTENT) <= 0 &&
+            warehouse.getGoodsCount(food) >= 100) {
+            int amount = (warehouse.getGoodsCount(food) * dx) / 6;
             if (amount > 0) {
-                return new Goods(getGame(), target, Goods.FOOD, amount);
+                return new Goods(getGame(), target, food, capAmount(amount, dx));
             }
-        } else if (tension <= Tension.DISPLEASED) {
+        } else if (tension.compareTo(Tension.Level.DISPLEASED) <= 0) {
             Market market = target.getOwner().getMarket();
             int value = 0;
             List<Goods> warehouseGoods = warehouse.getCompactGoods();
             for (Goods currentGoods : warehouseGoods) {
                 int goodsValue = market.getSalePrice(currentGoods);
-                if (currentGoods.getType() == Goods.FOOD || currentGoods.getType() == Goods.HORSES
-                        || currentGoods.getType() == Goods.MUSKETS) {
+                if (currentGoods.getType().isFoodType() ||
+                    currentGoods.getType().isMilitaryGoods()) {
                     continue;
                 } else if (goodsValue > value) {
                     value = goodsValue;
@@ -278,17 +257,44 @@ public class IndianDemandMission extends Mission {
                 }
             }
             if (goods != null) {
-                goods.setAmount(Math.max((goods.getAmount() * dx) / 6, 1));
+                goods.setAmount(capAmount(goods.getAmount(), dx));
                 return goods;
             }
         } else {
-            int[] preferred = new int[] { Goods.MUSKETS, Goods.HORSES, Goods.TOOLS, Goods.TRADE_GOODS, Goods.RUM,
-                    Goods.CLOTH, Goods.COATS, Goods.CIGARS };
-            for (int i = 0; i < preferred.length; i++) {
-                int amount = warehouse.getGoodsCount(preferred[i]);
-                if (amount > 0) {
-                    return new Goods(getGame(), target, preferred[i], Math.max((amount * dx) / 6, 1));
-                    
+            // military goods
+            for (GoodsType preferred : FreeCol.getSpecification().getGoodsTypeList()) {
+                if (preferred.isMilitaryGoods()) {
+                    int amount = warehouse.getGoodsCount(preferred);
+                    if (amount > 0) {
+                        return new Goods(getGame(), target, preferred, capAmount(amount, dx));
+                    }
+                }
+            }
+            // storable building materials (what do the natives need tools for?)
+            for (GoodsType preferred : FreeCol.getSpecification().getGoodsTypeList()) {
+                if (preferred.isBuildingMaterial() && preferred.isStorable()) {
+                    int amount = warehouse.getGoodsCount(preferred);
+                    if (amount > 0) {
+                        return new Goods(getGame(), target, preferred, capAmount(amount, dx));
+                    }
+                }
+            }
+            // trade goods
+            for (GoodsType preferred : FreeCol.getSpecification().getGoodsTypeList()) {
+                if (preferred.isTradeGoods()) {
+                    int amount = warehouse.getGoodsCount(preferred);
+                    if (amount > 0) {
+                        return new Goods(getGame(), target, preferred, capAmount(amount, dx));
+                    }
+                }
+            }
+            // refined goods
+            for (GoodsType preferred : FreeCol.getSpecification().getGoodsTypeList()) {
+                if (preferred.isRefined() && preferred.isStorable()) {
+                    int amount = warehouse.getGoodsCount(preferred);
+                    if (amount > 0) {
+                        return new Goods(getGame(), target, preferred, capAmount(amount, dx));
+                    }
                 }
             }
         }
@@ -305,11 +311,18 @@ public class IndianDemandMission extends Mission {
             }
         }
         if (goods != null) {
-            goods.setAmount(Math.max((goods.getAmount() * dx) / 6, 1));
+            goods.setAmount(capAmount(goods.getAmount(), dx));
         }
         return goods;
     }
 
+    private int capAmount(int amount, int difficulty) {
+        int finalAmount = Math.max((amount * difficulty) / 6, 1);
+        // natives can only carry one load of goods
+        finalAmount = Math.min(finalAmount, 100);
+        return finalAmount;
+    }
+
     /**
      * Checks if the unit is carrying a gift (goods).
      * 
