@@ -23,11 +23,14 @@ package net.sf.freecol.server.ai.mission;
 import java.io.IOException;
 import java.util.logging.Logger;
 
+import net.sf.freecol.common.model.CombatModel;
 import net.sf.freecol.common.model.GoalDecider;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Tension;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.MoveType;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.server.ai.AIMain;
@@ -43,8 +46,8 @@ import org.w3c.dom.Element;
 * you create different missions.
 */
 public abstract class Mission extends AIObject {
-    private static final Logger logger = Logger.getLogger(Mission.class.getName());
 
+    private static final Logger logger = Logger.getLogger(Mission.class.getName());
 
     protected static final int MINIMUM_TRANSPORT_PRIORITY = 60,     // A transport can be used
                                NORMAL_TRANSPORT_PRIORITY = 100;     // Transport is required
@@ -78,15 +81,6 @@ public abstract class Mission extends AIObject {
     }
 
     
-    /**
-     * Gets the ID of this mission.
-     * @return The ID.
-     */
-    public String getId() {
-        return null;
-    }
-    
-    
     /**
     * Moves the unit owning this mission towards the given <code>Tile</code>.
     * This is done in a loop until the tile is reached, there are no moves left,
@@ -98,18 +92,18 @@ public abstract class Mission extends AIObject {
     * @return The direction to take the final move (greater than or equal to zero),
     *         or {@link #NO_MORE_MOVES_LEFT} if there are no more moves left and
     *         {@link #NO_PATH_TO_TARGET} if there is no path to follow.
-    *         If a direction is returned, it is guarantied that moving in that direction
+    *         If a direction is returned, it is guaranteed that moving in that direction
     *         is not an {@link Unit#ILLEGAL_MOVE}, but a direction also gets returned
     *         if the resulting move would be an {@link Unit#ATTACK} etc. A direction
     *         can also be returned during the path, if the path has been blocked.
     */
-    protected int moveTowards(Connection connection, Tile tile) {
+    protected Direction moveTowards(Connection connection, Tile tile) {
         PathNode pathNode = getUnit().findPath(tile);
         
         if (pathNode != null) {
             return moveTowards(connection, pathNode);
         } else {
-            return NO_PATH_TO_TARGET;
+            return null;
         }
     }
 
@@ -124,29 +118,52 @@ public abstract class Mission extends AIObject {
     * @param pathNode The first node of the path.
     * @return The direction to continue moving the path (greater than or equal to zero),
     *         or {@link #NO_MORE_MOVES_LEFT} if there are no more moves left.
-    *         If a direction is returned, it is guarantied that moving in that direction
+    *         If a direction is returned, it is guaranteed that moving in that direction
     *         is not an {@link Unit#ILLEGAL_MOVE}. A directions gets returned when
     *         moving in the given direction would not be a {@link Unit#MOVE} or
     *         {@link Unit#MOVE_HIGH_SEAS}.
     */
-    protected int moveTowards(Connection connection, PathNode pathNode) {
+    protected Direction moveTowards(Connection connection, PathNode pathNode) {
         if (getUnit().getMovesLeft() <= 0) {            
-            return NO_MORE_MOVES_LEFT;
+            return null;
         }
         
-        while (pathNode.next != null && pathNode.getTurns() == 0
-                && (getUnit().getMoveType(pathNode.getDirection()) == Unit.MOVE
-                || getUnit().getMoveType(pathNode.getDirection()) == Unit.MOVE_HIGH_SEAS
-                || getUnit().getMoveType(pathNode.getDirection()) == Unit.EXPLORE_LOST_CITY_RUMOUR)) {
+        while (pathNode.next != null 
+                && pathNode.getTurns() == 0
+                && this.isValid() == true
+                && (getUnit().getMoveType(pathNode.getDirection()) == MoveType.MOVE
+                || getUnit().getMoveType(pathNode.getDirection()) == MoveType.MOVE_HIGH_SEAS
+                || getUnit().getMoveType(pathNode.getDirection()) == MoveType.EXPLORE_LOST_CITY_RUMOUR)) {
             move(connection, pathNode.getDirection());         
             pathNode = pathNode.next;
-        }        
-        if (pathNode.getTurns() == 0 && getUnit().getMoveType(pathNode.getDirection()) != Unit.ILLEGAL_MOVE) {            
+        }
+        if (pathNode.getTurns() == 0 && getUnit().getMoveType(pathNode.getDirection()) != MoveType.ILLEGAL_MOVE) {
             return pathNode.getDirection();
-        }        
-        return NO_MORE_MOVES_LEFT;
+        }
+        return null;
     }
 
+    protected void moveRandomly(Connection connection) {
+        Tile thisTile = getUnit().getTile();
+        Unit unit = getUnit();
+        Direction[] randomDirections = unit.getGame().getMap().getRandomDirectionArray();
+        while (unit.getMovesLeft() > 0) {
+            Direction direction = Direction.N;
+            int j;
+            for (j = 0; j < randomDirections.length; j++) {
+                direction = randomDirections[j];
+                if (unit.getGame().getMap().getNeighbourOrNull(direction, thisTile) != null &&
+                    unit.getMoveType(direction) == MoveType.MOVE) {
+                    break;
+                }
+            }
+            if (j == randomDirections.length)
+                break;
+            thisTile = unit.getGame().getMap().getNeighbourOrNull(direction, thisTile);
+
+            move(connection, direction);
+        }
+    }
 
     /**
     * Moves the unit owning this mission in the given direction.
@@ -155,10 +172,10 @@ public abstract class Mission extends AIObject {
     *         when communicating with the server.    
     * @param direction The direction to move the unit.         
     */
-    protected void move(Connection connection, int direction) {
+    protected void move(Connection connection, Direction direction) {
         Element moveElement = Message.createNewRootElement("move");
         moveElement.setAttribute("unit", getUnit().getId());
-        moveElement.setAttribute("direction", Integer.toString(direction));
+        moveElement.setAttribute("direction", direction.toString());
 
         try {
             connection.sendAndWait(moveElement);
@@ -167,6 +184,18 @@ public abstract class Mission extends AIObject {
         }
     }
     
+    protected void moveButDontAttack(Connection connection, Direction direction) {
+        if (direction != null) {
+            switch (getUnit().getMoveType(direction)) {
+            case MOVE: case MOVE_HIGH_SEAS: case EXPLORE_LOST_CITY_RUMOUR:
+                // The three cases acceptable to Unit.move()
+                move(connection, direction);
+                break;
+            default:
+                break;
+            }
+        }
+    }
     
     /**
      * Makes the unit explore the lost city rumour located on it's current
@@ -211,31 +240,39 @@ public abstract class Mission extends AIObject {
             }
             
             public boolean check(Unit unit, PathNode pathNode) {
+                CombatModel combatModel = getGame().getCombatModel();
                 Tile newTile = pathNode.getTile();
+                Unit defender = newTile.getDefendingUnit(unit);
                 if ((newTile.isLand() && !unit.isNaval() || !newTile.isLand() && unit.isNaval()) &&
-                        newTile.getDefendingUnit(unit) != null && newTile.getDefendingUnit(unit).getOwner() != unit.getOwner()) {
-                    
-                    int tension = unit.getOwner().getTension(newTile.getDefendingUnit(unit).getOwner()).getValue();
-                    if (unit.getIndianSettlement() != null) {
-                        tension += unit.getIndianSettlement().getAlarm(newTile.getDefendingUnit(unit).getOwner()).getValue();
+                        defender != null && defender.getOwner() != unit.getOwner()) {
+
+                    int tension = 0;
+                    Tension alarm = unit.getOwner().getTension(defender.getOwner());
+                    if (alarm != null) {
+                        tension = alarm.getValue();
                     }
-                    if (newTile.getDefendingUnit(unit).getType() == Unit.TREASURE_TRAIN) {
-                        tension += Math.min(newTile.getDefendingUnit(unit).getTreasureAmount() / 10, 600);
+                    if (unit.getIndianSettlement() != null &&
+                        unit.getIndianSettlement().getAlarm(defender.getOwner()) != null) {
+                        tension += unit.getIndianSettlement().getAlarm(defender.getOwner()).getValue();
                     }
-                    if (newTile.getDefendingUnit(unit).getType() == Unit.ARTILLERY && newTile.getSettlement() == null) {
-                        tension += 100 - newTile.getDefendingUnit(unit).getDefensePower(unit) * 2;
+                    if (defender.canCarryTreasure()) {
+                        tension += Math.min(defender.getTreasureAmount() / 10, 600);
                     }
-                    if (newTile.getDefendingUnit(unit).getType() == Unit.VETERAN_SOLDIER && !newTile.getDefendingUnit(unit).isArmed()) {
-                        tension += 50 - newTile.getDefendingUnit(unit).getDefensePower(unit) * 2;
+                    if (defender.getType().getDefence() > 0 &&
+                        newTile.getSettlement() == null) {
+                        tension += 100 - combatModel.getDefencePower(unit, defender) * 2;
+                    }
+                    if (defender.hasAbility("model.ability.expertSoldier") &&
+                        !defender.isArmed()) {
+                        tension += 50 - combatModel.getDefencePower(unit, defender) * 2;
                     }
                     // TODO-AI-CHEATING: REMOVE WHEN THE AI KNOWNS HOW TO HANDLE PEACE WITH THE INDIANS:
                     if (unit.getOwner().isIndian() 
-                            && newTile.getDefendingUnit(unit) != null
-                            && newTile.getDefendingUnit(unit).getOwner().isAI()) {
+                            && defender.getOwner().isAI()) {
                         tension -= 200;
                     }
                     // END: TODO-AI-CHEATING
-                    if (tension > Tension.TENSION_CONTENT) {
+                    if (tension > Tension.Level.CONTENT.getLimit()) {
                         if (bestTarget == null) {
                             bestTarget = pathNode;                           
                         } else if (bestTarget.getTurns() == pathNode.getTurns()) {
@@ -259,12 +296,12 @@ public abstract class Mission extends AIObject {
     */
     public Tile getTransportDestination() {
         if (getUnit().getTile() == null) {
-            if (getUnit().getLocation() instanceof Unit) {
+            if (getUnit().isOnCarrier()) {
                 return (Tile) ((Unit) getUnit().getLocation()).getEntryLocation();
             } else {
                 return (Tile) getUnit().getOwner().getEntryLocation();
             }
-        } else if (!(getUnit().getLocation() instanceof Unit)) {
+        } else if (!getUnit().isOnCarrier()) {
             return null;
         }
         
@@ -319,7 +356,7 @@ public abstract class Mission extends AIObject {
     }
     
     /**
-     * Disposes this mission by removing any referances to it.
+     * Disposes this mission by removing any references to it.
      */
     public void dispose() {
         // Nothing to do yet.
@@ -345,6 +382,11 @@ public abstract class Mission extends AIObject {
     * @return The default value: <code>true</code>.
     */
     public boolean isValid() {
+        if (getUnit() != null && getUnit().isDisposed()) {
+            // an AI unit can move accidentally into a lost city ruin and get killed
+            // the mission is was associated with should become invalid 
+            return false;
+        }
         return true;
     }
 
