@@ -31,16 +31,24 @@ import javax.xml.stream.XMLStreamException;
 import javax.xml.stream.XMLStreamReader;
 import javax.xml.stream.XMLStreamWriter;
 
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.Europe;
 import net.sf.freecol.common.model.Goods;
+import net.sf.freecol.common.model.GoodsType;
 import net.sf.freecol.common.model.Locatable;
 import net.sf.freecol.common.model.Location;
+import net.sf.freecol.common.model.Map;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Market;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.MoveType;
+import net.sf.freecol.common.model.Unit.Role;
+import net.sf.freecol.common.model.Unit.UnitState;
+import net.sf.freecol.common.model.UnitType;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.server.ai.AIColony;
@@ -165,15 +173,30 @@ public class TransportMission extends Mission {
      * Disposes this <code>Mission</code>.
      */
     public void dispose() {
-        Iterator<Transportable> ti = transportList.iterator();
+    	// a new list must be created as the first one may be changed
+    	//elsewhere in between loop calls
+    	List<Transportable> cargoList = new ArrayList<Transportable>();
+    	List<Transportable> scheduledCargoList = new ArrayList<Transportable>();
+    	
+    	Iterator<Transportable> ti = transportList.iterator();
         while (ti.hasNext()) {
             Transportable t = ti.next();
+            // the cargo is on board, add to list to be disposed of
             if (isCarrying(t)) {
-                ((AIObject) t).dispose();
+            	cargoList.add(t);
             } else {
-                t.setTransport(null);
+            	// the cargo was scheduled to be transported
+            	// cancel order
+            	scheduledCargoList.add(t);
             }
         }
+        
+        for (Transportable t : cargoList)
+        	((AIObject) t).dispose();
+        
+        for (Transportable t : scheduledCargoList)
+        	t.setTransport(null);
+        
         super.dispose();
     }
 
@@ -260,15 +283,21 @@ public class TransportMission extends Mission {
 
         int bestSourceIndex = -1;
         if (!isCarrying(newTransportable)) {
-            int bestSourceDistance;
+            
+            int distToSource;
             if (carrier.getLocation().getTile() == newSource.getTile()) {
-                bestSourceIndex = 0;
-                bestSourceDistance = 0;
+                distToSource = 0;
             } else {
-                bestSourceIndex = 0;
-                bestSourceDistance = getDistanceTo(newTransportable, ((carrier.getTile() != null) ? carrier.getTile()
+                distToSource = getDistanceTo(newTransportable, ((carrier.getTile() != null) ? carrier.getTile()
                         : carrier.getEntryLocation().getTile()), true);
+                // Sanitation
+                // Carrier cant reach source
+                if(distToSource == Map.COST_INFINITY){
+                    return;
+                }
             }
+            bestSourceIndex = 0;
+            int bestSourceDistance = distToSource;
             for (int i = 1; i < transportList.size() && bestSourceDistance > 0; i++) {
                 Transportable t1 = transportList.get(i - 1);
                 if (t1.getTransportSource() != null && t1.getTransportSource().getTile() == newSource.getTile()
@@ -279,16 +308,28 @@ public class TransportMission extends Mission {
                 }
 
             }
+            
             for (int i = 1; i < transportList.size() && bestSourceDistance > 0; i++) {
                 Transportable t1 = transportList.get(i - 1);
-                if (isCarrying(t1)
-                        && getDistanceTo(newTransportable, t1.getTransportDestination(), true) <= bestSourceDistance) {
-                    bestSourceIndex = i;
-                    bestSourceDistance = getDistanceTo(newTransportable, t1.getTransportDestination(), true);
-                } else if (!isCarrying(t1)
-                        && getDistanceTo(newTransportable, t1.getTransportSource(), true) <= bestSourceDistance) {
-                    bestSourceIndex = i;
-                    bestSourceDistance = getDistanceTo(newTransportable, t1.getTransportSource(), true);
+                 
+                if (isCarrying(t1)){
+                    int distToDestination = getDistanceTo(newTransportable, t1.getTransportDestination(), true);
+                    if(distToDestination == Map.COST_INFINITY){
+                        continue;
+                    }
+                    if(distToDestination <= bestSourceDistance) {
+                         bestSourceIndex = i;
+                         bestSourceDistance = distToDestination;
+                    }
+                } else{
+                    distToSource = getDistanceTo(newTransportable, t1.getTransportSource(), true);
+                    if(distToSource == Map.COST_INFINITY){
+                        continue;
+                    }   
+                    if (distToSource <= bestSourceDistance) {
+                        bestSourceIndex = i;
+                        bestSourceDistance = distToSource;
+                    }
                 }
             }
             transportList.add(bestSourceIndex, newTransportable);
@@ -301,7 +342,10 @@ public class TransportMission extends Mission {
             if (carrier.getTile() == newSource.getTile()) {
                 bestDestinationDistance = 0;
             } else {
-                bestDestinationDistance = getDistanceTo(newTransportable, carrier.getTile(), false);
+                int distToCarrier = getDistanceTo(newTransportable, carrier.getTile(), false);
+                if(distToCarrier != Map.COST_INFINITY){
+                    bestDestinationDistance = distToCarrier;
+                }
             }
         }
         for (int i = Math.max(bestSourceIndex, 1); i < transportList.size() && bestDestinationDistance > 0; i++) {
@@ -311,18 +355,27 @@ public class TransportMission extends Mission {
                 bestDestinationIndex = i;
                 bestDestinationDistance = 0;
             }
-
         }
         for (int i = Math.max(bestSourceIndex, 1); i < transportList.size() && bestDestinationDistance > 0; i++) {
             Transportable t1 = transportList.get(i - 1);
-            if (isCarrying(t1)
-                    && getDistanceTo(newTransportable, t1.getTransportDestination(), false) <= bestDestinationDistance) {
-                bestDestinationIndex = i;
-                bestDestinationDistance = getDistanceTo(newTransportable, t1.getTransportDestination(), false);
-            } else if (!isCarrying(t1)
-                    && getDistanceTo(newTransportable, t1.getTransportSource(), false) <= bestDestinationDistance) {
-                bestDestinationIndex = i;
-                bestDestinationDistance = getDistanceTo(newTransportable, t1.getTransportSource(), false);
+            if (isCarrying(t1)){
+                int distToDestination = getDistanceTo(newTransportable, t1.getTransportDestination(), false);
+                if(distToDestination == Map.COST_INFINITY){
+                    continue;
+                }
+                if(distToDestination <= bestDestinationDistance) {
+                    bestDestinationIndex = i;
+                    bestDestinationDistance = distToDestination;
+                }
+            } else{
+                int distToSource = getDistanceTo(newTransportable, t1.getTransportSource(), false); 
+                if(distToSource == Map.COST_INFINITY){
+                    continue;
+                }
+                if(distToSource <= bestDestinationDistance) {
+                    bestDestinationIndex = i;
+                    bestDestinationDistance =  distToSource;
+                }
             }
         }
         transportList.add(bestDestinationIndex, newTransportable);
@@ -347,7 +400,13 @@ public class TransportMission extends Mission {
      */
     private int getDistanceTo(Transportable t, Location start, boolean source) {
         // TODO: This takes to much resources - find another method:
-        return getPath(t, start, source).getTotalTurns();
+        PathNode path = getPath(t, start, source);
+        
+        if(path == null){
+            return Map.COST_INFINITY;
+        }
+        
+        return path.getTotalTurns();
     }
 
     /**
@@ -356,36 +415,22 @@ public class TransportMission extends Mission {
      * @param connection The <code>Connection</code> to the server.
      */
     public void doMission(Connection connection) {
-        Unit carrier = getUnit();
-
-        updateTransportList();
-
-        if (carrier.getLocation() instanceof Europe) {
-            if (carrier.getState() == Unit.TO_EUROPE || carrier.getState() == Unit.TO_AMERICA) {
-                return;
-            } else {
-                restockCargoAtDestination(connection);
-                buyCargo(connection);
-                restockCargoAtDestination(connection);
-
-                // Move back to america:
-                if (carrier.getOwner().getGold() < MINIMUM_GOLD_TO_STAY_IN_EUROPE || transportList.size() > 0) {
-                    Element moveToAmericaElement = Message.createNewRootElement("moveToAmerica");
-                    moveToAmericaElement.setAttribute("unit", carrier.getId());
-                    try {
-                        connection.sendAndWait(moveToAmericaElement);
-                    } catch (IOException e) {
-                        logger.warning("Could not send \"moveToAmericaElement\"-message!");
-                    }
-                }
-                return;
-            }
-        }
-
         if (transportList == null || transportList.size() <= 0) {
             updateTransportList();
         }
 
+        Unit carrier = getUnit();
+        if (carrier.getLocation() instanceof Europe) {
+            // Coming to/from Europe, do nothing
+            if (carrier.getState() == UnitState.TO_EUROPE || carrier.getState() == UnitState.TO_AMERICA) {
+                return;
+            }
+            // Actually in Europe
+            inEurope(connection);
+            return;
+            
+        }
+
         restockCargoAtDestination(connection);
 
         boolean transportListChanged = false;
@@ -422,11 +467,12 @@ public class TransportMission extends Mission {
             if (path != null) {
                 // Tile oldTile = carrier.getTile();
 
-                int r = moveTowards(connection, path);
-                if (r >= 0 && (carrier.getMoveType(r) == Unit.MOVE || carrier.getMoveType(r) == Unit.MOVE_HIGH_SEAS)) {
+                Direction r = moveTowards(connection, path);
+                if (r != null &&
+                    (carrier.getMoveType(r) == MoveType.MOVE || carrier.getMoveType(r) == MoveType.MOVE_HIGH_SEAS)) {
                     // Tile target = getGame().getMap().getNeighbourOrNull(r,
                     // carrier.getTile());
-                    if (carrier.getMoveType(r) == Unit.MOVE_HIGH_SEAS && moveToEurope) {
+                    if (carrier.getMoveType(r) == MoveType.MOVE_HIGH_SEAS && moveToEurope) {
                         Element moveToEuropeElement = Message.createNewRootElement("moveToEurope");
                         moveToEuropeElement.setAttribute("unit", carrier.getId());
                         try {
@@ -479,13 +525,18 @@ public class TransportMission extends Mission {
          * be removed after a proper implementation has been created.
          */
         if (aiPlayer.hasFewColonies()) {
+            // since we are in Europe, use the carrier entry point to search for a good settlement spot.
+            Unit carrier = getUnit();
+            Tile colonyTile = BuildColonyMission.findColonyLocation(carrier);
             int space = getAvailableSpace();
-            while (space > 0) {
+            while (colonyTile!=null && space > 0) {
                 AIUnit newUnit = getCheapestUnitInEurope(connection);
                 if (newUnit != null) {
                     if (newUnit.getUnit().isColonist() && !newUnit.getUnit().isArmed()
-                            && !newUnit.getUnit().isMounted() && !newUnit.getUnit().isPioneer()) {
-                        newUnit.setMission(new BuildColonyMission(getAIMain(), newUnit));
+                        && !newUnit.getUnit().isMounted() && newUnit.getUnit().getRole() != Role.PIONEER) {
+                        // send the colonist to build the new colony
+                        int colonyValue = aiPlayer.getPlayer().getColonyValue(colonyTile);
+                        newUnit.setMission(new BuildColonyMission(getAIMain(), newUnit, colonyTile, colonyValue));
                     }
                     addToTransportList(newUnit);
                     space--;
@@ -543,8 +594,8 @@ public class TransportMission extends Mission {
         for (int i = 0; i < aiColonies.size(); i++) {
             AIColony ac = aiColonies.get(i);
             // Assuming that all colonists which can be bought in Europe take
-            // the same space:
-            int space = getAvailableSpace(Unit.FREE_COLONIST, getUnit().getOwner().getEurope(), ac.getColony());
+            // the same space: TODO: fix this
+            int space = getAvailableSpace(getUnit().getType(), getUnit().getOwner().getEurope(), ac.getColony());
             Iterator<Wish> wishIterator = ac.getWishIterator();
             while (space > 0 && wishIterator.hasNext()) {
                 Wish w = wishIterator.next();
@@ -604,7 +655,7 @@ public class TransportMission extends Mission {
      *            be transported.
      * @return The goods.
      */
-    public AIGoods buyGoodsInEurope(Connection connection, int type, int amount, Location destination) {
+    public AIGoods buyGoodsInEurope(Connection connection, GoodsType type, int amount, Location destination) {
         AIPlayer aiPlayer = (AIPlayer) getAIMain().getAIObject(getUnit().getOwner().getId());
         Player player = aiPlayer.getPlayer();
         Market market = player.getMarket();
@@ -612,12 +663,12 @@ public class TransportMission extends Mission {
         if (player.getGold() >= market.getBidPrice(type, amount)) {
             Element buyGoodsElement = Message.createNewRootElement("buyGoods");
             buyGoodsElement.setAttribute("carrier", getUnit().getId());
-            buyGoodsElement.setAttribute("type", Integer.toString(type));
+            buyGoodsElement.setAttribute("type", type.getId());
             buyGoodsElement.setAttribute("amount", Integer.toString(amount));
             try {
                 connection.sendAndWait(buyGoodsElement);
             } catch (IOException e) {
-                logger.warning("Could not send \"trainUnitInEurope\"-message to the server.");
+                logger.warning("Could not send \"buyGoods\"-message to the server.");
                 return null;
             }
             AIGoods ag = new AIGoods(getAIMain(), getUnit(), type, amount, destination);
@@ -648,10 +699,10 @@ public class TransportMission extends Mission {
      * </ol>
      * 
      * @param connection The <code>Connection</code> to the server.
-     * @param unitType The type of {@link Unit} to be found/recuited/trained.
+     * @param unitType The type of {@link Unit} to be found/recruited/trained.
      * @return The <code>AIUnit</code>.
      */
-    private AIUnit getUnitInEurope(Connection connection, int unitType) {
+    private AIUnit getUnitInEurope(Connection connection, UnitType unitType) {
         AIPlayer aiPlayer = (AIPlayer) getAIMain().getAIObject(getUnit().getOwner().getId());
         Player player = aiPlayer.getPlayer();
         Europe europe = player.getEurope();
@@ -664,7 +715,7 @@ public class TransportMission extends Mission {
         Iterator<Unit> ui = europe.getUnitIterator();
         while (ui.hasNext()) {
             Unit u = ui.next();
-            if (unitType == -1 || unitType == u.getType()) {
+            if (unitType == null || unitType == u.getType()) {
                 return (AIUnit) getAIMain().getAIObject(u.getId());
             }
         }
@@ -675,40 +726,15 @@ public class TransportMission extends Mission {
             for (int i = 0; i < 3; i++) {
                 // Note, used to be 1-3 but the method expects 0-2
                 if (europe.getRecruitable(i) == unitType) {
-                    Element recruitUnitInEuropeElement = Message.createNewRootElement("recruitUnitInEurope");
-                    recruitUnitInEuropeElement.setAttribute("slot", Integer.toString(i));
-                    try {
-                        Element reply = connection.ask(recruitUnitInEuropeElement);
-                        if (reply.getTagName().equals("recruitUnitInEuropeConfirmed")) {
-                            return (AIUnit) getAIMain().getAIObject(
-                                    ((Element) reply.getChildNodes().item(0)).getAttribute("ID"));
-                        } else {
-                            logger.warning("Could not recruit the specified unit in europe.");
-                            continue;
-                        }
-                    } catch (IOException e) {
-                        logger.warning("Could not send \"recruitUnitInEurope\"-message to the server.");
-                    }
+                    return aiPlayer.recruitAIUnitInEurope(i);
                 }
             }
         }
 
         // Try training the unit:
-        if (unitType != Unit.ARTILLERY && Unit.getPrice(unitType) >= 0 && player.getGold() >= Unit.getPrice(unitType)
-                || unitType == Unit.ARTILLERY && player.getGold() >= europe.getArtilleryPrice()) {
-            Element trainUnitInEuropeElement = Message.createNewRootElement("trainUnitInEurope");
-            trainUnitInEuropeElement.setAttribute("unitType", Integer.toString(unitType));
-            try {
-                Element reply = connection.ask(trainUnitInEuropeElement);
-                if (reply.getTagName().equals("trainUnitInEuropeConfirmed")) {
-                    return (AIUnit) getAIMain().getAIObject(
-                            ((Element) reply.getChildNodes().item(0)).getAttribute("ID"));
-                } else {
-                    logger.warning("Could not train the specified unit in europe.");
-                }
-            } catch (IOException e) {
-                logger.warning("Could not send \"trainUnitInEurope\"-message to the server.");
-            }
+        if (unitType.hasPrice() && europe.getUnitPrice(unitType) >= 0 &&
+                player.getGold() >= europe.getUnitPrice(unitType)) {
+            return aiPlayer.trainAIUnitInEurope(unitType);
         }
 
         return null;
@@ -729,6 +755,9 @@ public class TransportMission extends Mission {
         if (!(getUnit().getLocation() instanceof Europe)) {
             throw new IllegalStateException("Carrier not in Europe");
         }
+        if (!player.canRecruitUnits()) {
+            return null;
+        }
 
         // Check if there are any units on the docks:
         Iterator<Unit> ui = europe.getUnitIterator();
@@ -739,40 +768,26 @@ public class TransportMission extends Mission {
             }
         }
 
-        // Try recruiting the unit:
-        if (player.getGold() >= player.getRecruitPrice()
-                && player.getRecruitPrice() < Unit.getPrice(Unit.EXPERT_ORE_MINER)) {
-            Element recruitUnitInEuropeElement = Message.createNewRootElement("recruitUnitInEurope");
-            // TODO: Take the best unit (Seasoned scout, pioneer, soldier etc):
-            recruitUnitInEuropeElement.setAttribute("slot", Integer.toString(1));
-            try {
-                Element reply = connection.ask(recruitUnitInEuropeElement);
-                if (reply.getTagName().equals("recruitUnitInEuropeConfirmed")) {
-                    return (AIUnit) getAIMain().getAIObject(
-                            ((Element) reply.getChildNodes().item(0)).getAttribute("ID"));
-                } else {
-                    logger.warning("Could not recruit the specified unit in europe.");
-                }
-            } catch (IOException e) {
-                logger.warning("Could not send \"recruitUnitInEurope\"-message to the server.");
+        int priceTrained = 0;
+        UnitType cheapestTrained = null;
+        List<UnitType> unitTypes = FreeCol.getSpecification().getUnitTypesTrainedInEurope();
+        for (UnitType unitType : unitTypes) {
+            int price = europe.getUnitPrice(unitType);
+            if (cheapestTrained == null || price < priceTrained) {
+            	cheapestTrained = unitType;
+            	priceTrained = price;
             }
         }
+        // Try recruiting the unit:
+        if (player.getGold() >= player.getRecruitPrice() && cheapestTrained != null
+                && player.getRecruitPrice() < priceTrained) {
+            // TODO: Take the best unit (Seasoned scout, pioneer, soldier etc)
+            return aiPlayer.recruitAIUnitInEurope(1);
+        }
 
         // Try training the unit:
-        if (player.getGold() >= Unit.getPrice(Unit.EXPERT_ORE_MINER)) {
-            Element trainUnitInEuropeElement = Message.createNewRootElement("trainUnitInEurope");
-            trainUnitInEuropeElement.setAttribute("unitType", Integer.toString(Unit.EXPERT_ORE_MINER));
-            try {
-                Element reply = connection.ask(trainUnitInEuropeElement);
-                if (reply.getTagName().equals("trainUnitInEuropeConfirmed")) {
-                    return (AIUnit) getAIMain().getAIObject(
-                            ((Element) reply.getChildNodes().item(0)).getAttribute("ID"));
-                } else {
-                    logger.warning("Could not train the specified unit in europe.");
-                }
-            } catch (IOException e) {
-                logger.warning("Could not send \"trainUnitInEurope\"-message to the server.");
-            }
+        if (cheapestTrained != null && player.getGold() >= priceTrained) {
+            return aiPlayer.trainAIUnitInEurope(cheapestTrained);
         }
 
         return null;
@@ -856,7 +871,7 @@ public class TransportMission extends Mission {
             Unit u = (Unit) t.getTransportLocatable();
             return getAvailableSpace(u.getType(), t.getTransportSource(), t.getTransportDestination());
         } else {
-            return getAvailableSpace(-1, t.getTransportSource(), t.getTransportDestination());
+            return getAvailableSpace(null, t.getTransportSource(), t.getTransportDestination());
         }
     }
 
@@ -864,14 +879,14 @@ public class TransportMission extends Mission {
      * Returns the available space for the given type of <code>Unit</code> at
      * the given <code>Location</code>.
      * 
-     * @param unitType The type of {@link Unit} or <code>-1</code> for
+     * @param unitType The type of {@link Unit} or <code>null</code> for
      *            {@link Goods}
      * @param source The source for the unit. This is where the unit is
      *            presently located.
      * @param destination The destination for the unit.
      * @return The space available
      */
-    public int getAvailableSpace(int unitType, Location source, Location destination) {
+    public int getAvailableSpace(UnitType unitType, Location source, Location destination) {
         // TODO: Implement this method properly:
         return Math.max(0, getUnit().getSpaceLeft() - transportList.size());
     }
@@ -923,7 +938,7 @@ public class TransportMission extends Mission {
                 if (mission != null && mission.isValid()) {
                     if (au.getTransportDestination() != null
                             && au.getTransportDestination().getTile() == carrier.getTile()
-                            && carrier.getState() != Unit.TO_EUROPE && carrier.getState() != Unit.TO_AMERICA) {
+                            && carrier.getState() != UnitState.TO_EUROPE && carrier.getState() != UnitState.TO_AMERICA) {
                         if (u.getLocation() instanceof Europe || u.getColony() != null) {
                             Element leaveShipElement = Message.createNewRootElement("leaveShip");
                             leaveShipElement.setAttribute("unit", u.getId());
@@ -944,7 +959,10 @@ public class TransportMission extends Mission {
                                 au.getTransportDestination().getTile(), carrier);
                         if (p != null) {
                             final PathNode dropNode = p.getTransportDropNode();
-                            if (dropNode != null && dropNode.getTile().getDistanceTo(carrier.getTile()) <= 1) {
+                            int distToCarrier = dropNode.getTile().getDistanceTo(carrier.getTile());
+                            if (dropNode != null &&
+                                    distToCarrier != Map.COST_INFINITY &&
+                                    distToCarrier <= 1) {
                                 mission.doMission(connection);
                                 if (u.getLocation() != getUnit()) {
                                     removeFromTransportList(au);
@@ -984,7 +1002,7 @@ public class TransportMission extends Mission {
                 AIGoods ag = (AIGoods) t;
                 if (ag.getTransportDestination() != null
                         && ag.getTransportDestination().getTile() == carrier.getLocation().getTile()
-                        && carrier.getState() != Unit.TO_EUROPE && carrier.getState() != Unit.TO_AMERICA) {
+                        && carrier.getState() != UnitState.TO_EUROPE && carrier.getState() != UnitState.TO_AMERICA) {
                     if (carrier.getLocation() instanceof Europe) {
                         // TODO-AI-CHEATING: REMOVE WHEN THE AI IS GOOD ENOUGH:
                         Player p = carrier.getOwner();
@@ -1049,8 +1067,8 @@ public class TransportMission extends Mission {
             if (t instanceof AIUnit) {
                 AIUnit au = (AIUnit) t;
                 Unit u = au.getUnit();
-                if (u.getTile() == carrier.getTile() && carrier.getState() != Unit.TO_EUROPE
-                        && carrier.getState() != Unit.TO_AMERICA) {
+                if (u.getTile() == carrier.getTile() && carrier.getState() != UnitState.TO_EUROPE
+                        && carrier.getState() != UnitState.TO_AMERICA) {
                     Element boardShipElement = Message.createNewRootElement("boardShip");
                     boardShipElement.setAttribute("unit", u.getId());
                     boardShipElement.setAttribute("carrier", carrier.getId());
@@ -1064,12 +1082,12 @@ public class TransportMission extends Mission {
                 }
             } else if (t instanceof AIGoods) {
                 AIGoods ag = (AIGoods) t;
-                if (ag.getGoods().getTile() == carrier.getTile() && carrier.getState() != Unit.TO_EUROPE
-                        && carrier.getState() != Unit.TO_AMERICA) {
+                if (ag.getGoods().getTile() == carrier.getTile() && carrier.getState() != UnitState.TO_EUROPE
+                        && carrier.getState() != UnitState.TO_AMERICA) {
                     if (carrier.getLocation() instanceof Europe) {
                         Element buyGoodsElement = Message.createNewRootElement("buyGoods");
                         buyGoodsElement.setAttribute("carrier", carrier.getId());
-                        buyGoodsElement.setAttribute("type", Integer.toString(ag.getGoods().getType()));
+                        buyGoodsElement.setAttribute("type", Integer.toString(ag.getGoods().getType().getIndex()));
                         buyGoodsElement.setAttribute("amount", Integer.toString(ag.getGoods().getAmount()));
                         try {
                             connection.sendAndWait(buyGoodsElement);
@@ -1109,7 +1127,11 @@ public class TransportMission extends Mission {
      * @return <code>true</code>
      */
     public boolean isValid() {
-        return true;
+        Unit unit = this.getAIUnit().getUnit();
+        if (unit.isNaval() && unit.isUnderRepair())
+            return false;
+        else
+            return true;
     }
 
     /**
@@ -1130,6 +1152,28 @@ public class TransportMission extends Mission {
         return 0;
     }
 
+    /**
+     * unit is in Europe, unload cargo on board, buy required goods and board unit 
+     * @param connection  The <code>Connection</code> to the server.
+     */
+    private void inEurope(Connection connection){
+        restockCargoAtDestination(connection);
+        buyCargo(connection);
+        restockCargoAtDestination(connection);
+
+        // Move back to America:
+        Unit carrier = getUnit();
+        if (carrier.getOwner().getGold() < MINIMUM_GOLD_TO_STAY_IN_EUROPE || transportList.size() > 0) {
+            Element moveToAmericaElement = Message.createNewRootElement("moveToAmerica");
+            moveToAmericaElement.setAttribute("unit", carrier.getId());
+            try {
+                connection.sendAndWait(moveToAmericaElement);
+            } catch (IOException e) {
+                logger.warning("Could not send \"moveToAmericaElement\"-message!");
+            }
+        }        
+    }
+    
     /**
      * Finds the best path to <code>Europe</code>.
      * 
@@ -1207,6 +1251,22 @@ public class TransportMission extends Mission {
     public static String getXMLElementTagName() {
         return "transportMission";
     }
+    
+    /**
+     * Gets debugging information about this mission. This string is a short
+     * representation of this object's state.
+     * 
+     * @return The <code>String</code>: "(x, y) z" or "(x, y) z!" where
+     *         <code>x</code> and <code>y</code> is the coordinates of the
+     *         target tile for this mission, and <code>z</code> is the value
+     *         of building the colony. The exclamation mark is added if the unit
+     *         should continue searching for a colony site if the targeted site
+     *         is lost.
+     */
+    public String getDebuggingInfo() {
+        Unit carrier = getUnit();
+        return this.toString();
+    }
 
     /**
      * Creates a <code>String</code> representation of this mission to be used
