@@ -31,14 +31,16 @@ import net.sf.freecol.common.model.Colony;
 import net.sf.freecol.common.model.FreeColGameObject;
 import net.sf.freecol.common.model.GoalDecider;
 import net.sf.freecol.common.model.Location;
-import net.sf.freecol.common.model.Map;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Ownable;
 import net.sf.freecol.common.model.PathNode;
 import net.sf.freecol.common.model.Player;
+import net.sf.freecol.common.model.Player.Stance;
 import net.sf.freecol.common.model.Settlement;
 import net.sf.freecol.common.model.Tension;
 import net.sf.freecol.common.model.Tile;
 import net.sf.freecol.common.model.Unit;
+import net.sf.freecol.common.model.Unit.MoveType;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
 import net.sf.freecol.server.ai.AIMain;
@@ -53,8 +55,7 @@ import org.w3c.dom.Element;
 */
 public class UnitSeekAndDestroyMission extends Mission {
     private static final Logger logger = Logger.getLogger(UnitSeekAndDestroyMission.class.getName());
-
-
+    
     /**
      * The object we are trying to destroy. This can be a
      * either <code>Settlement</code> or a <code>Unit</code>.
@@ -128,63 +129,41 @@ public class UnitSeekAndDestroyMission extends Mission {
         }
         
         PathNode pathToTarget = null;
-        if (unit.getLocation() instanceof Unit) {
-            if (unit.getTile() == null) {
-                pathToTarget = null;
-            } else {
+        if (unit.isOnCarrier()) {
+            if (unit.getTile() != null) {
                 pathToTarget = getDisembarkPath(unit, unit.getTile(), target.getTile(), (Unit) unit.getLocation());
                 if (pathToTarget.getTransportDropNode() != pathToTarget) {
                     pathToTarget = null;
                 }
             }
         } else {
-            if (unit.getTile() == null) {
-                pathToTarget = null;
-            } else {
+            if (unit.getTile() != null) {
                 pathToTarget = getUnit().findPath(target.getTile());
             }
         }
         
         if (pathToTarget != null) {
-            int direction = moveTowards(connection, pathToTarget);
-            while (direction >= 0) {
+            Direction direction = moveTowards(connection, pathToTarget);
+            if (direction != null 
+                && unit.getMoveType(direction) == MoveType.ATTACK) {
                 Tile newTile = getGame().getMap().getNeighbourOrNull(direction, unit.getTile());
-                if (unit.getMoveType(direction) == Unit.ATTACK
-                        && (unit.getOwner().getStance(newTile.getDefendingUnit(unit).getOwner()) == Player.WAR
-                                || ((Ownable) target).getOwner() == newTile.getDefendingUnit(unit).getOwner())) {
-                    Element element = Message.createNewRootElement("attack");
-                    element.setAttribute("unit", unit.getId());
-                    element.setAttribute("direction", Integer.toString(direction));
-
-                    try {
-                        connection.ask(element);
-                    } catch (IOException e) {
-                        logger.warning("Could not send message!");
-                    }
-                    break;
-                } else if (unit.getMoveType(direction) != Unit.ATTACK                        
-                        && unit.getMoveType(direction) != Unit.ILLEGAL_MOVE) {
-                    Element element = Message.createNewRootElement("move");
-                    element.setAttribute("unit", unit.getId());
-                    element.setAttribute("direction", Integer.toString(direction));
-
-                    try {
-                        connection.sendAndWait(element);
-                    } catch (IOException e) {
-                        logger.warning("Could not send message!");
-                    }
-
-                    // Hmmm... if we are an AI player, our Unit will be updated 
-                    // immediately. Therefore we don't need to check a server
-                    // response.
-                    pathToTarget = getUnit().findPath(target.getTile());
-                    if (pathToTarget != null) {
-                        direction = moveTowards(connection, pathToTarget);
-                    } else {
-                        break;
-                    }
+                Unit defender = newTile.getDefendingUnit(unit);
+                if (defender == null) {
+                    logger.warning("MoveType is ATTACK, but no defender is present!");
                 } else {
-                    break;
+                    Player enemy = defender.getOwner();
+                    if (unit.getOwner().getStance(enemy) == Stance.WAR
+                        || ((Ownable) target).getOwner() == enemy) {
+                        Element element = Message.createNewRootElement("attack");
+                        element.setAttribute("unit", unit.getId());
+                        element.setAttribute("direction", direction.toString());
+
+                        try {
+                            connection.ask(element);
+                        } catch (IOException e) {
+                            logger.warning("Could not send message!");
+                        }
+                    }
                 }
             }
         }
@@ -206,7 +185,7 @@ public class UnitSeekAndDestroyMission extends Mission {
             public boolean check(Unit u, PathNode pathNode) {
                 goal = pathNode;
                 if (pathNode.getTile().getSettlement() == null) {
-                    for (int direction=0; direction < Map.NUMBER_OF_DIRECTIONS; direction++) {
+                    for (Direction direction : Direction.values()) {
                         Tile attackTile = u.getGame().getMap().getNeighbourOrNull(direction, pathNode.getTile());
                         if (end == attackTile 
                                 && attackTile.getSettlement() != null 
@@ -249,10 +228,12 @@ public class UnitSeekAndDestroyMission extends Mission {
         }
 
         targetPlayer = ((Ownable) target).getOwner();
-        int stance = owner.getStance(targetPlayer);
+        Stance stance = owner.getStance(targetPlayer);
 
-        return targetPlayer != owner && (stance == Player.WAR 
-                || owner.isIndian() && owner.getTension(targetPlayer).getLevel() >= Tension.CONTENT);
+        return targetPlayer != owner &&
+            (stance == Stance.WAR 
+             || owner.isIndian() 
+             && owner.getTension(targetPlayer).getLevel().compareTo(Tension.Level.CONTENT) >= 0);
     }
 
     
@@ -273,7 +254,7 @@ public class UnitSeekAndDestroyMission extends Mission {
         Tile dropTarget = target.getTile();
         if (getUnit().getTile() == null) {
             return dropTarget;
-        } else if (getUnit().getLocation() instanceof Unit) {
+        } else if (getUnit().isOnCarrier()) {
             PathNode p = getDisembarkPath(getUnit(), 
                     getUnit().getTile(), 
                     target.getTile(), 
@@ -283,7 +264,7 @@ public class UnitSeekAndDestroyMission extends Mission {
             }
         }
         
-        if (getUnit().getLocation() instanceof Unit) {
+        if (getUnit().isOnCarrier()) {
             return dropTarget;
         } else if (getUnit().getLocation().getTile() == target) {
             return null;
