@@ -24,6 +24,10 @@ import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.logging.Logger;
 
+import net.sf.freecol.FreeCol;
+import net.sf.freecol.common.FreeColException;
+import net.sf.freecol.common.model.Nation;
+import net.sf.freecol.common.model.NationType;
 import net.sf.freecol.common.model.Player;
 import net.sf.freecol.common.networking.Connection;
 import net.sf.freecol.common.networking.Message;
@@ -73,6 +77,11 @@ public final class PreGameInputHandler extends InputHandler {
                 return nation(connection, element);
             }
         });
+        register("setNationType", new NetworkRequestHandler() {
+            public Element handle(Connection connection, Element element) {
+                return nationType(connection, element);
+            }
+        });
         register("setColor", new NetworkRequestHandler() {
             public Element handle(Connection connection, Element element) {
                 return color(connection, element);
@@ -148,7 +157,7 @@ public final class PreGameInputHandler extends InputHandler {
     }
 
     /**
-     * Handles a "nation"-message from a client.
+     * Handles a "setNation"-message from a client.
      * 
      * @param connection The connection the message came from.
      * @param element The element containing the request.
@@ -156,12 +165,16 @@ public final class PreGameInputHandler extends InputHandler {
     private Element nation(Connection connection, Element element) {
         ServerPlayer player = getFreeColServer().getPlayer(connection);
         if (player != null) {
-            int nation = Integer.parseInt(element.getAttribute("value"));
-            player.setNation(nation);
-            Element updateNation = Message.createNewRootElement("updateNation");
-            updateNation.setAttribute("player", player.getId());
-            updateNation.setAttribute("value", Integer.toString(nation));
-            getFreeColServer().getServer().sendToAll(updateNation, player.getConnection());
+            Nation nation = FreeCol.getSpecification().getNation(element.getAttribute("value"));
+            if (nation.isClassic() || (nation.isSelectable() && getFreeColServer().getAdditionalNations())) {
+                player.setNation(nation);
+                Element updateNation = Message.createNewRootElement("updateNation");
+                updateNation.setAttribute("player", player.getId());
+                updateNation.setAttribute("value", nation.getId());
+                getFreeColServer().getServer().sendToAll(updateNation, player.getConnection());
+            } else {
+                logger.warning("Selected non-selectable nation.");
+            }
         } else {
             logger.warning("Nation from unknown connection.");
         }
@@ -169,7 +182,36 @@ public final class PreGameInputHandler extends InputHandler {
     }
 
     /**
-     * Handles a "color"-message from a client.
+     * Handles a "setNationType"-message from a client.
+     * 
+     * @param connection The connection the message came from.
+     * @param element The element containing the request.
+     */
+    private Element nationType(Connection connection, Element element) {
+        ServerPlayer player = getFreeColServer().getPlayer(connection);
+        if (player != null) {
+            NationType nationType = FreeCol.getSpecification().getNationType(element.getAttribute("value"));
+            NationType fixedNationType = FreeCol.getSpecification().getNation(player.getNationID()).getType();
+            int advantages = getFreeColServer().getAdvantages();
+            if (advantages == net.sf.freecol.client.gui.panel.AdvantageCellRenderer.SELECTABLE
+                || (advantages == net.sf.freecol.client.gui.panel.AdvantageCellRenderer.FIXED
+                    && nationType.equals(fixedNationType))) {
+                player.setNationType(nationType);
+                Element updateNationType = Message.createNewRootElement("updateNationType");
+                updateNationType.setAttribute("nationType", player.getId());
+                updateNationType.setAttribute("value", nationType.getId());
+                getFreeColServer().getServer().sendToAll(updateNationType, player.getConnection());
+            } else {
+                logger.warning("NationType is not selectable");
+            }
+        } else {
+            logger.warning("NationType from unknown connection.");
+        }
+        return null;
+    }
+
+    /**
+     * Handles a "setColor"-message from a client.
      * 
      * @param connection The connection the message came from.
      * @param element The element containing the request.
@@ -178,7 +220,7 @@ public final class PreGameInputHandler extends InputHandler {
         ServerPlayer player = getFreeColServer().getPlayer(connection);
         if (player != null) {
             String color = element.getAttribute("value");
-            player.setColor(color);
+            player.setColor(new Color(Integer.decode(color)));
             Element updateColor = Message.createNewRootElement("updateColor");
             updateColor.setAttribute("player", player.getId());
             updateColor.setAttribute("value", color);
@@ -207,15 +249,15 @@ public final class PreGameInputHandler extends InputHandler {
         }
         // Check that no two players have the same color or nation
         Iterator<Player> playerIterator = freeColServer.getGame().getPlayerIterator();
-        LinkedList<Integer> nations = new LinkedList<Integer>();
+        LinkedList<Nation> nations = new LinkedList<Nation>();
         LinkedList<Color> colors = new LinkedList<Color>();
         while (playerIterator.hasNext()) {
             ServerPlayer player = (ServerPlayer) playerIterator.next();
-            final int nation = player.getNation();
+            final Nation nation = FreeCol.getSpecification().getNation(player.getNationID());
             final Color color = player.getColor();
             // Check the nation.
             for (int i = 0; i < nations.size(); i++) {
-                if (nations.get(i).intValue() == nation) {
+                if (nations.get(i) == nation) {
                     Element reply = Message.createNewRootElement("error");
                     reply
                             .setAttribute("message",
@@ -224,7 +266,7 @@ public final class PreGameInputHandler extends InputHandler {
                     return reply;
                 }
             }
-            nations.add(new Integer(nation));
+            nations.add(nation);
             // Check the color.
             for (int i = 0; i < colors.size(); i++) {
                 if (colors.get(i).equals(color)) {
@@ -243,7 +285,15 @@ public final class PreGameInputHandler extends InputHandler {
             reply.setAttribute("messageID", "server.notAllReady");
             return reply;
         }
-        ((PreGameController) freeColServer.getController()).startGame();
+        try {
+            ((PreGameController) freeColServer.getController()).startGame();
+        } catch (FreeColException e) {
+            // send an error message to the client(s)
+            Element reply = Message.createNewRootElement("error");
+            reply.setAttribute("message", "An error occurred while starting the game!");
+            reply.setAttribute("messageID", "server.errorStartingGame");
+            return reply;
+        }
         return null;
     }
 
