@@ -20,14 +20,19 @@
 package net.sf.freecol.server.generator;
 
 import java.util.ArrayList;
-import java.util.Hashtable;
 import java.util.Iterator;
+import java.util.List;
 import java.util.logging.Logger;
 
-import net.sf.freecol.common.PseudoRandom;
+import net.sf.freecol.FreeCol;
 import net.sf.freecol.common.model.Map;
+import net.sf.freecol.common.model.Map.Direction;
 import net.sf.freecol.common.model.Tile;
+import net.sf.freecol.common.model.TileImprovement;
+import net.sf.freecol.common.model.TileItemContainer;
+import net.sf.freecol.common.model.TileType;
 import net.sf.freecol.common.model.Map.Position;
+import net.sf.freecol.server.model.ServerRegion;
 
 
 /**
@@ -37,24 +42,64 @@ public class River {
 
     private static final Logger logger = Logger.getLogger(MapGenerator.class.getName());
 
+    private static final TileType greatRiver = FreeCol.getSpecification().getTileType("model.tile.greatRiver");
 
     /**
      * Directions a river may flow in.
      * @see net.sf.freecol.common.model.Map
      */
-    private static int[] directions = {1, 3, 5, 7};
+    public static final Direction[] directions = {
+        Direction.NE, Direction.SE, Direction.SW, Direction.NW};
 
     /**
      * Possible direction changes for a river.
      * @see net.sf.freecol.common.model.Map
      */
-    private static int[] change = { 0, 2, -2 };
+    private static enum DirectionChange {
+        STRAIGHT_AHEAD,
+        RIGHT_TURN,
+        LEFT_TURN;
+
+        public Direction getNewDirection(Direction oldDirection) {
+            switch(this) {
+            case STRAIGHT_AHEAD:
+                return oldDirection;
+            case RIGHT_TURN:
+                switch(oldDirection) {
+                case NE:
+                    return Direction.SE;
+                case SE:
+                    return Direction.SW;
+                case SW:
+                    return Direction.NW;
+                case NW:
+                    return Direction.NE;
+                default:
+                    return oldDirection;
+                }
+            case LEFT_TURN:
+                switch(oldDirection) {
+                case NE:
+                    return Direction.NW;
+                case SE:
+                    return Direction.NE;
+                case SW:
+                    return Direction.SE;
+                case NW:
+                    return Direction.SW;
+                default:
+                    return oldDirection;
+                }
+            }
+            return oldDirection;
+        }
+    }
 
     /**
      * Current direction the river is flowing in.
      */
-    private int direction = 1;
-
+    private Direction direction;
+    
     /**
      * The map on which the river flows.
      */
@@ -63,17 +108,27 @@ public class River {
     /**
      * A list of river sections.
      */
-    private ArrayList<Section> sections = new ArrayList<Section>();
+    private List<RiverSection> sections = new ArrayList<RiverSection>();
 
     /**
      * The next river.
      */
     private River nextRiver = null;
 
+    /**
+     * The ServerRegion this River belongs to.
+     */
+    private ServerRegion region;
+
     /**
      * A hashtable of position-river pairs.
      */
-    private Hashtable<Position, River> riverMap;
+    private java.util.Map<Position, River> riverMap;
+
+    /**
+     * Whether the river is connected to Europe.
+     */
+    private boolean connected = false;
 
 
     /**
@@ -82,10 +137,18 @@ public class River {
      * @param map The map on which the river flows.
      * @param riverMap A hashtable of position-river pairs.
      */
-    public River(Map map, Hashtable<Position, River> riverMap) {
+    public River(Map map, java.util.Map<Position, River> riverMap, ServerRegion region) {
         this.map = map;
         this.riverMap = riverMap;
-        logger.fine("Starting new river");
+        this.region = region;
+        int length = directions.length;
+        int index = map.getGame().getModelController().getPseudoRandom().nextInt(length);
+        direction = directions[index];
+        logger.fine("Starting new river flowing " + direction.toString());
+    }
+
+    public List<RiverSection> getSections() {
+        return sections;
     }
 
     /**
@@ -97,18 +160,36 @@ public class River {
         return this.sections.size();
     }
 
-    public Section getLastSection() {
+    public RiverSection getLastSection() {
         return this.sections.get(sections.size() - 1);
     }
 
+    /**
+     * Get the <code>ServerRegion</code> value.
+     *
+     * @return a <code>ServerRegion</code> value
+     */
+    public final ServerRegion getRegion() {
+        return region;
+    }
+
+    /**
+     * Set the <code>ServerRegion</code> value.
+     *
+     * @param newServerRegion The new ServerRegion value.
+     */
+    public final void setRegion(final ServerRegion newServerRegion) {
+        this.region = newServerRegion;
+    }
+
     /**
      * Adds a new section to this river.
      *
      * @param position Where this section is located.
      * @param direction The direction the river is flowing in.
      */
-    public void add(Map.Position position, int direction) {
-        this.sections.add(new Section(position, direction));
+    public void add(Map.Position position, Direction direction) {
+        this.sections.add(new RiverSection(position, direction));
     }
 
     /**
@@ -117,26 +198,25 @@ public class River {
      * @param lastSection The last section of the river flowing into this one.
      * @param position The position of the confluence.
      */
-    public void grow(Section lastSection, Map.Position position) {
+    public void grow(RiverSection lastSection, Map.Position position) {
         
         boolean found = false;
-        Section section = null;
         
-        Iterator<Section> sectionIterator = sections.iterator();
-        while (sectionIterator.hasNext()) {
-            section = sectionIterator.next();
+        for (RiverSection section : sections) {
             if (found) {
                 section.grow();
-            } else if (section.position.equals(position)) {
-                section.addBranch(oppositeDirection(lastSection.direction),
-                        lastSection.size);
+            } else if (section.getPosition().equals(position)) {
+                section.setBranch(lastSection.direction.getReverseDirection(),
+                        lastSection.getSize());
                 section.grow();
                 found = true;
             }
         }
         drawToMap();
         if (nextRiver != null) {
-            nextRiver.grow(section, Map.getAdjacent(section.position, section.direction));
+            RiverSection section = sections.get(sections.size() - 1);
+            Position neighbor = Map.getAdjacent(section.getPosition(), section.direction);
+            nextRiver.grow(section, neighbor);
         }
     }
 
@@ -147,8 +227,8 @@ public class River {
      * @return true if the given position is next to this river.
      */
     public boolean isNextToSelf(Map.Position p) {
-        for (int i = 0; i < 4; i++) {
-            Map.Position px = Map.getAdjacent(p, directions[i]);
+        for (Direction direction : directions) {
+            Map.Position px = Map.getAdjacent(p, direction);
             if (this.contains(px)) {
                 return true;
             }
@@ -163,8 +243,8 @@ public class River {
      * @return true if the given position is next to a river, lake or sea.
      */
     public boolean isNextToWater(Map.Position p) {
-        for (int i = 0; i < 4; i++) {
-            Map.Position px = Map.getAdjacent(p, directions[i]);
+        for (Direction direction : directions) {
+            Map.Position px = Map.getAdjacent(p, direction);
             final Tile tile = map.getTile(px);
             if (tile == null) {
                 continue;
@@ -183,9 +263,9 @@ public class River {
      * @return true if this river already contains the given position.
      */
     public boolean contains(Map.Position p) {
-        Iterator<Section> sectionIterator = sections.iterator();
+        Iterator<RiverSection> sectionIterator = sections.iterator();
         while (sectionIterator.hasNext()) {
-            Map.Position q = sectionIterator.next().position;
+            Map.Position q = sectionIterator.next().getPosition();
             if (p.equals(q)) {
                 return true;
             }
@@ -201,17 +281,10 @@ public class River {
      */
     public boolean flowFromSource(Map.Position position) {
         Tile tile = map.getTile(position);
-        if (!tile.isLand()) {
-            logger.fine("Tile at " + position + " is water.");
-            return false;
-        } else if (tile.getAddition() != Tile.ADD_NONE) {
-            logger.fine("Tile at " + position + " has additions.");
-            return false;
-        } else if (tile.getType() == Tile.DESERT) {
-            logger.fine("Tile at " + position + " is desert.");
-            return false;
-        } else if (tile.getType() == Tile.ARCTIC) {
-            logger.fine("Tile at " + position + " is arctic.");
+        if (!tile.getType().canHaveRiver()) {
+            // Mountains, ocean cannot have rivers
+            logger.fine("Tile (" + tile.getType().getName() + ") at "
+                        + position + " cannot have rivers.");
             return false;
         } else if (isNextToWater(position)) {
             logger.fine("Tile at " + position + " is next to water.");
@@ -229,23 +302,29 @@ public class River {
      * @return true if a river was created, false otherwise.
      */
     private boolean flow(Map.Position source) {
-        //Tile t = map.getTile(source);
         
         if (sections.size() % 2 == 0) {
             // get random new direction
-            this.direction = newDirection(-1);
+            int length = DirectionChange.values().length;
+            int index = map.getGame().getModelController().getPseudoRandom().nextInt(length);
+            DirectionChange change = DirectionChange.values()[index];
+            this.direction = change.getNewDirection(this.direction);
             logger.fine("Direction is now " + direction);
         }
         
-        for (int i = 0; i < 3; i++) {
-            int dir = newDirection(i);
+        for (DirectionChange change : DirectionChange.values()) {
+            Direction dir = change.getNewDirection(direction);
             Map.Position newPosition = Map.getAdjacent(source, dir);
             Tile nextTile = map.getTile(newPosition);
             
+            if (nextTile == null) {
+                continue;
+            }
             // is the tile suitable for this river?
-            if (nextTile.getAddition() == Tile.ADD_HILLS ||
-                    nextTile.getAddition() == Tile.ADD_MOUNTAINS) {
-                logger.fine("Tile at " + newPosition + " too high.");
+            if (!nextTile.getType().canHaveRiver()) {
+                // Mountains, ocean cannot have rivers
+                logger.fine("Tile (" + nextTile.getType().getName() + ") at "
+                            + newPosition + " cannot have rivers.");
                 continue;
             } else if (this.contains(newPosition)) {
                 logger.fine("Tile at " + newPosition + " is already in river.");
@@ -255,17 +334,17 @@ public class River {
                 continue;
             } else {
                 // find out if an adjacent tile is next to water
-                for (i = 0; i < 3; i++) {
-                    int lastDir = newDirection(i, dir);
+                for (DirectionChange change2 : DirectionChange.values()) {
+                    Direction lastDir = change2.getNewDirection(dir);
                     Map.Position px = Map.getAdjacent(newPosition, lastDir);
                     Tile tile = map.getTile(px);
-                    if (tile == null || !tile.isLand() || tile.hasRiver()) {
+                    if (tile != null && (!tile.isLand() || tile.hasRiver())) {
                         
-                        sections.add(new Section(source, dir));
-                        Section lastSection = new Section(newPosition, lastDir);
+                        sections.add(new RiverSection(source, dir));
+                        RiverSection lastSection = new RiverSection(newPosition, lastDir);
                         sections.add(lastSection);
                         
-                        if (tile.hasRiver()) {
+                        if (tile.hasRiver() && tile.isLand()) {
                             logger.fine("Point " + newPosition + " is next to another river.");
                             drawToMap();
                             // increase the size of another river
@@ -276,11 +355,13 @@ public class River {
                             logger.fine("Point " + newPosition + " is next to water.");
                             River someRiver = riverMap.get(px);
                             if (someRiver == null) {
-                                sections.add(new Section(px, oppositeDirection(lastDir)));
+                                sections.add(new RiverSection(px, lastDir.getReverseDirection()));
                             } else {
-                                Section waterSection = someRiver.getLastSection();
-                                waterSection.addBranch(oppositeDirection(lastDir), 1);
+                                RiverSection waterSection = someRiver.getLastSection();
+                                waterSection.setBranch(lastDir.getReverseDirection(),
+                                                       TileImprovement.SMALL_RIVER);
                             }
+                            connected = tile.isConnected();
                             drawToMap();
                         }
                         return true;
@@ -288,167 +369,50 @@ public class River {
                 }
                 // this is not the case
                 logger.fine("Tile at " + newPosition + " is suitable.");
-                sections.add(new Section(source, dir));
+                sections.add(new RiverSection(source, dir));
                 return flow(newPosition);
             }
         }
-        sections = new ArrayList<Section>();
+        sections = new ArrayList<RiverSection>();
         return false;
     }
     
-    /**
-     * Returns a new direction, but never the opposite of the current
-     * direction.
-     *
-     * @param index The index of the direction change.
-     * @return A new direction. @see net.sf.freecol.common.model.Map.
-     */
-    private int newDirection(int index) {
-        return newDirection(index, this.direction);
-    }
-
-    /**
-     * Returns a new direction, but never the opposite of the given
-     * direction.
-     *
-     * @param index The index of the direction change.
-     * @param currentDirection The current direction.
-     * @return A new direction. @see net.sf.freecol.common.model.Map.
-     */
-    private int newDirection(int index, int currentDirection) {
-        int offset = 0;
-        if (index < 0 || index > 2) {
-            PseudoRandom random =
-                map.getGame().getModelController().getPseudoRandom();
-            offset = change[random.nextInt(3)];
-        } else {
-            offset = change[index];
-        }
-        return (currentDirection + offset + Map.NUMBER_OF_DIRECTIONS) 
-                % Map.NUMBER_OF_DIRECTIONS;
-    }
-
-    /**
-     * Returns the opposite of the direction given.
-     * 
-     * @param direction The direction to reverse.
-     * @return The opposite of the direction given.
-     */
-    private int oppositeDirection(int direction) {
-        return (direction + Map.NUMBER_OF_DIRECTIONS/2)
-                % Map.NUMBER_OF_DIRECTIONS;
-    }
-
     /**
      * Draws the completed river to the map.
      */
     private void drawToMap() {
-        Section oldSection = null;
+        RiverSection oldSection = null;
         
-        Iterator<Section> sectionIterator = sections.iterator();
-        while (sectionIterator.hasNext()) {
-            Section section = sectionIterator.next();
-            riverMap.put(section.position, this);
+        for (RiverSection section : sections) {
+            riverMap.put(section.getPosition(), this);
             if (oldSection != null) {
-                section.addBranch(oppositeDirection(oldSection.direction),
-                        oldSection.size);
+                section.setBranch(oldSection.direction.getReverseDirection(),
+                        oldSection.getSize());
             }
-            Tile tile = map.getTile(section.position);
-            switch (section.size) {
-            case 1:
-                tile.addRiver(Tile.ADD_RIVER_MINOR, section.getBranches());
-                logger.fine("Added minor river to tile at " + section.position);
-                break;
-            case 2:
-                tile.addRiver(Tile.ADD_RIVER_MAJOR, section.getBranches());
-                logger.fine("Added major river to tile at " + section.position);
-                break;
-            default:
-                tile.setType(Tile.OCEAN);
-            logger.fine("Created fjord at " + section.position);
+            Tile tile = map.getTile(section.getPosition());
+            if (tile.isLand()) {
+                TileItemContainer container = tile.getTileItemContainer();
+                if (container == null) {
+                    container = new TileItemContainer(tile.getGame(), tile);
+                    tile.setTileItemContainer(container);
+                }
+                if (section.getSize() == TileImprovement.SMALL_RIVER || 
+                    section.getSize() == TileImprovement.LARGE_RIVER) {
+                    container.addRiver(section.getSize(), section.encodeStyle());
+                    logger.fine("Added river (magnitude: " + section.getSize() +
+                                ") to tile at " + section.getPosition());
+                } else if (section.getSize() >= TileImprovement.FJORD_RIVER) {
+                    tile.setType(greatRiver);   // changing the type resets the improvements
+                    container.addRiver(section.getSize(), section.encodeStyle());
+                    if (connected) {
+                        tile.setConnected(true);
+                    }
+                    logger.fine("Added fjord (magnitude: " + section.getSize() +
+                                ") to tile at " + section.getPosition());
+                }
+                region.addTile(tile);
+                oldSection = section;
             }
-            oldSection = section;
         }
     }
-
-    public static int updateRiver(int oldRiver, int direction, int addition) {
-        //System.out.println("old = " + oldRiver + ", direction = " + direction +
-        //", addition = " + addition);
-        int[] base = {0, 1, 0, 3, 0, 9, 0, 27};
-        if (base[direction] == 0) {
-            // ignore these directions
-            return oldRiver;
-        }
-        int branch = 0;
-        if (addition == Tile.ADD_RIVER_MINOR) {
-            branch = 1;
-        } else if (addition == Tile.ADD_RIVER_MAJOR) {
-            branch = 2;
-        }
-
-        int tmpRiver = oldRiver;
-        int value = 0;
-        for (int index = base.length - 1; index > direction ; index -= 2) {
-            value = tmpRiver / base[index];
-            tmpRiver -= value * base[index];
-        }
-        value = tmpRiver / base[direction];
-        //System.out.println("value = " + value + ", tmpRiver = " + tmpRiver);
-        if (value == branch) {
-            // no changes
-            return oldRiver;
-        } else {
-            int newRiver = oldRiver + (branch - value) * base[direction];
-            //System.out.println("new = " + newRiver);
-            return newRiver;
-        }
-    }
-        
-        
-
-    /**
-     * A river section.
-     */
-    private class Section {
-        private int[] base = {1, 3, 9, 27};
-
-        public Map.Position position;
-        public int size = 1;
-        public int direction = -1;
-        public int branch[] = {0, 0, 0, 0};
-
-        public Section(Map.Position position, int direction) {
-            this.position = position;
-            this.direction = direction;
-            this.branch[direction/2] = 1;
-        }
-
-        public void addBranch(int direction, int size) {
-            if (size != 1) {
-                size = 2;
-            }
-            this.branch[direction/2] = size;
-        }
-
-        /**
-         * Returns the type of river.
-         * @return The type of river.
-         */
-        public int getBranches() {
-            int result = 0;
-            for (int i = 0; i < 4; i++) {
-                result += branch[i] * base[i];
-            }
-            return result;
-        }
-
-        /**
-         * Increases the size of this section by one.
-         */
-        public void grow() {
-            this.size++;
-            this.branch[direction/2] = 2;
-        }
-
-    }
 }
